// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
// 		*********************88
//
//
// This file is a mess, a playground to test things out. When things start
// to make some sense, a better system for managing the application will
// be designed and put in place.
//
//
//
//
//
//

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "kit.h"

#include "tree_controller.h"
#include "document_controller.h"

// This class is just a heinous mess to help test things...

#include "foundation/macros.h"
#include "foundation/command.h"
#include "foundation/command_undo.h"
#include "foundation/command_redo.h"
#include "foundation/command_callback.h"
#include "foundation/undo_event.h"
#include "foundation/undo_manager.h"
#include "foundation/utility.h"

#include "drawing/image.h"
#include "drawing/factory.h"

#include "toolkit/factory.h"
#include "toolkit/file_selection.h"
#include "toolkit/log.h"
#include "toolkit/menu.h"
#include "toolkit/message_box.h"
#include "toolkit/palette.h"
#include "toolkit/statusbar.h"
#include "toolkit/toolbar.h"
#include "toolkit/tree.h"

#include "reflective/changes.h"
#include "reflective/debug_change_listener.h"

#include "uml/uml.h"
#include "uml/foundation/data_types/name.h"
#include "uml/model_management/model.h"
#include "uml/model_management/package.h"
#include "uml/writer.h"
#include "uml/handler.h"
#include "uml/foundation/extension_mechanisms/tagged_value.h"

#include "xml/sax_parser.h"

#include <sstream>
#include <fstream>
using namespace std;

Kit * Kit::mInstance = 0;

void
Kit::instantiate(const KitWidgets & widgets, const KitCommands & commands) {
    ASSERT(!mInstance);
    mInstance = new Kit(widgets, commands);
}

void
Kit::shutdown() {
    ASSERT(mInstance);
    delete mInstance;
    mInstance = 0;
}

Kit *
Kit::instance() {
    ASSERT(mInstance);
    return mInstance;
}

Kit::Kit(const KitWidgets & widgets, const KitCommands & commands) : Object(),
    mWidgets(widgets),
    mCommands(commands),
    mUML(0),
    mChanged(false) {

    initialise();
}

Kit::~Kit() {
    if(mUML) {
	mUML->ref_set_root(0);
	mUML->unref();
    }

    mFileMenu->unref();
    mEditMenu->unref();
    mHelpMenu->unref();

    //

    mCommandClearLog->unref();

    mCommandUndo->unref();
    mCommandRedo->unref();

    mCommandNew->unref();
    mCommandOpen->unref();
    mCommandSave->unref();
    mCommandSaveAs->unref();

    //

    mTreeController->unref();
    mDocumentController->unref();

    mCommands.about->unref();
    mCommands.exit->unref();

    mWidgets.menubar->unref();
    mWidgets.toolbar->unref();
    mWidgets.statusbar->unref();
    mWidgets.tree->unref();
    mWidgets.log->unref();
}

void
Kit::log(const string & message, Toolkit::LogLevel level) {
    mWidgets.log->log(message, level);
}

bool
Kit::can_exit() {
    Toolkit::Factory * factory = Toolkit::Factory::instance();

    if(mChanged) {
	Toolkit::MessageBox * message_box =
	    factory->create_message_box("The model has changed. Really exit ?",
					Toolkit::MBK_QUESTION,
					mWidgets.menubar);

	Toolkit::ResponseKind response;
	response = message_box->go();
	message_box->unref();

	if(response == Toolkit::RK_YES) {
	    return true;
	}
	else {
	    return false;
	}
    }
    else {
	return true;
    }
}

void
Kit::initialise() {
    create_commands();
    create_menus();
    create_toolbar();
    create_connections();
    create_controllers();
    new_();
}

//
//
//

void
Kit::create_commands() {
    mCommandNew = command_callback(this, &Kit::new_);
    mCommandNew->set_instructions("Create a new model");

    mCommandOpen = command_callback(this, &Kit::open);
    mCommandOpen->set_instructions("Open an existing model");

    mCommandSave = command_callback(this, &Kit::save);
    mCommandSave->set_instructions("Save the model");

    mCommandSaveAs = command_callback(this, &Kit::save_as);
    mCommandSaveAs->set_instructions("Save the model using a different name");

    mCommandUndo = new CommandUndo();
    mCommandRedo = new CommandRedo();

    mCommandClearLog = command_callback(mWidgets.log, &Toolkit::Log::clear);
    mCommandClearLog->set_instructions("Clear the log");
}

void
Kit::create_menus() {
    Toolkit::Factory * factory = Toolkit::Factory::instance();

    // File menu

    mFileMenu = factory->create_menu();

    mFileMenu->add_push(mCommandNew, "New");
    mFileMenu->add_push(mCommandOpen, "Open...");
    mFileMenu->add_push(mCommandSave, "Save");
    mFileMenu->add_push(mCommandSaveAs, "Save As...");
    mFileMenu->add_separator();
    mFileMenu->add_push(mCommands.exit, "Exit");

    // Edit menu

    mEditMenu = factory->create_menu();

    mEditMenu->add_push(mCommandUndo, "Undo", false);
    mEditMenu->add_push(mCommandRedo, "Redo", false);
    mEditMenu->add_separator();
    mEditMenu->add_push(mCommandClearLog, "Clear Log");

    // Help menu

    mHelpMenu = factory->create_menu();

    mHelpMenu->add_push(mCommands.about, "About...");

    //

    mWidgets.menubar->add_submenu(mFileMenu, "File");
    mWidgets.menubar->add_submenu(mEditMenu, "Edit");
    mWidgets.menubar->add_submenu(mHelpMenu, "Help");
}

void
Kit::create_toolbar() {
    Drawing::Factory * factory = Drawing::Factory::instance();
    Drawing::Image * image;

    //

    image = factory->create_image("toolbar/new.png");

    mWidgets.toolbar->add_element(mCommandNew, image, true);

    image->unref();

    //

    image =factory->create_image("toolbar/open.png");

    mWidgets.toolbar->add_element(mCommandOpen, image, true);

    image->unref();

    //

    image = factory->create_image("toolbar/save.png");

    mWidgets.toolbar->add_element(mCommandSave, image, false);

    image->unref();

    //

    image = factory->create_image("toolbar/save-as.png");

    mWidgets.toolbar->add_element(mCommandSaveAs, image, true);

    image->unref();

    //

    mWidgets.toolbar->add_separator();

    //

    image = factory->create_image("toolbar/undo.png");

    mWidgets.toolbar->add_element(mCommandUndo, image, false);

    image->unref();

    //

    image = factory->create_image("toolbar/redo.png");

    mWidgets.toolbar->add_element(mCommandRedo, image, false);

    image->unref();

    //

    mWidgets.toolbar->add_separator();

    //

    image = factory->create_image("toolbar/quit.png");

    mWidgets.toolbar->add_element(mCommands.exit, image, true);

    image->unref();
}

void
Kit::create_connections() {
    // connect callbacks

    UndoManager * undo_manager = UndoManager::instance();

    undo_manager->add(static_cast<UndoListener *>(this));

#if 0
    undo_manager->add_change_listener(new REF::DebugChangeListener(cerr));
#endif

    undo_manager->add_change_listener(static_cast<ChangeListener *>(this));
}

void
Kit::create_controllers() {

  //mTreeController = new TreeController(mWidgets.tree);
  //mTreeController = new TreeController(mWidgets.tree);

    mTreeController = new TreeController("hierarchy", mWidgets.tree);
    mTreeController->set_context(mUML);

    mDocumentController = new DocumentController("hierarchy", mWidgets.document);
    mDocumentController->set_context(mUML);
}

//
//
//

void
Kit::new_() {
    bool really = true;

    if(mUML && mChanged) {
	Toolkit::Factory * factory = Toolkit::Factory::instance();

	Toolkit::MessageBox * message_box =
	    factory->create_message_box("Model has changed. Continue anyway ?",
					Toolkit::MBK_QUESTION,
					mWidgets.menubar);

	really = (message_box->go() == Toolkit::RK_YES);
	message_box->unref();
    }

    if(really) {
	if(mUML) {
	    mTreeController->set_context(0);
	    mDocumentController->set_context(0);

	    mUML->ref_set_root(0);
	    mUML->unref();
	    mUML = 0;
	}

	UndoManager * undo_manager = UndoManager::instance();

	undo_manager->clear();

	//
	//
	//

	ASSERT(!mUML);

	mUML = new UML::UML();

	UML::Model * model = new UML::Model();

	UML::Name * name = new UML::Name();

	UML::TaggedValue * doco = 0;

	name->ref_set_value("Model");
	model->set_name(name);

	doco = new UML::TaggedValue();
	name->set_value("doco");
	doco->set_name(name);
	name->set_value("");
	doco->set_data_value(name);
	model->add_tagged_value(doco);
	doco->unref();

	//

	UML::Package * package;

	package = new UML::Package();
	doco = new UML::TaggedValue();
	name->set_value("User View");
	package->set_name(name);
	name->set_value("doco");
	doco->set_name(name);
	name->set_value("The user view should develop the software model based on the user"
			" perspective, using objects from the user domain.  This may consist"
			" of Use Cases and Requirement lists");
	doco->set_data_value(name);
	package->add_tagged_value(doco);
	model->add_owned_element(package);
	doco->unref();
	package->unref();
	
	package = new UML::Package();
	doco = new UML::TaggedValue();
	name->set_value("Static View");
	package->set_name(name);
	name->set_value("doco");
	doco->set_name(name);
	name->set_value("The static view should present the software design.  It may consist"
			" of Class diagrams and Object diagrams");
	doco->set_data_value(name);
	package->add_tagged_value(doco);
	model->add_owned_element(package);
	doco->unref();
	package->unref();

	package = new UML::Package();
	doco = new UML::TaggedValue();
	name->set_value("Dynamic View");
	package->set_name(name);
	name->set_value("doco");
	doco->set_name(name);
	name->set_value("The dynamic view should illustrate the interaction between classes"
			" in the analysis model.  It may use interaction and Collaboration"
			"diagrams");
	doco->set_data_value(name);
	package->add_tagged_value(doco);
	model->add_owned_element(package);
	doco->unref();
	package->unref();

	package = new UML::Package();
	doco = new UML::TaggedValue();
	name->set_value("Deployment View");
	package->set_name(name);
	name->set_value("doco");
	doco->set_name(name);
	name->set_value("The deployment view shows the structure of the software in deployment."
			" It may consist of deployment diagrams.");
	doco->set_data_value(name);
	package->add_tagged_value(doco);
	model->add_owned_element(package);
	doco->unref();
	package->unref();

	//

	name->unref();

	mUML->ref_set_root(model);

	model->unref();

	//

	undo_manager->clear();

	//

	mTreeController->set_context(mUML);
	mDocumentController->set_context(mUML);

	mChanged = false;		// reset the changed flag
	mFileName = "";			// reset the file name
    }
}

void
Kit::open() {
    Toolkit::Factory * factory = Toolkit::Factory::instance();

    Toolkit::FileSelection * file_selection = factory->create_file_selection("Open file");

    string file_name;
    Toolkit::ResponseKind response;

    response = file_selection->go(file_name);

    file_selection->unref();

    if(response == Toolkit::RK_OK) {
	if(mChanged) {
	    Toolkit::MessageBox * message_box =
		factory->create_message_box("The model has changed. Really open ?",
					    Toolkit::MBK_QUESTION,
					    mWidgets.menubar);

	    response = message_box->go();
	    message_box->unref();

	    if(response != Toolkit::RK_OK) {
		return;
	    }
	}

	ifstream ifs(file_name.c_str());

	if(!ifs) {
	    Toolkit::MessageBox * message_box =
		factory->create_message_box("Couldn't open: \"" + file_name + "\" for reading",
					    Toolkit::MBK_ERROR,
					    mWidgets.menubar);
	    message_box->go();
	    message_box->unref();
	}
	else {
	    mTreeController->set_context(0);
	    mDocumentController->set_context(0);

	    mUML->ref_set_root(0);
	    mUML->unref();
	    mUML = 0;

	    // read it

	    UML::Handler * handler = new UML::Handler();
	    XML::SAXParser * parser = new XML::SAXParser(handler);

	    try {
		parser->push();
		parser->parse(ifs, 2);

                ASSERT(parser->top().size() == 1);
		REF::Class * root = known_cast<REF::Class *>(parser->top().back());

		root->ref();

		mUML = new UML::UML();
		mUML->ref_set_root(root);
		root->unref();

		mTreeController->set_context(mUML);
		mDocumentController->set_context(mUML);

		UndoManager * undo_manager = UndoManager::instance();
		undo_manager->clear();

		mChanged = false;
	    }
	    catch(...) {
		ERROR("Exception occurred whilst reading from file: " << file_name);
	    }

	    parser->unref();
	    handler->unref();
	}
    }
}

void
Kit::save() {
    ASSERT(!mFileName.empty());

    ofstream ofs(mFileName.c_str());

    if(!ofs) {
	Toolkit::Factory * factory = Toolkit::Factory::instance();

	Toolkit::MessageBox * message_box =
	    factory->create_message_box("Couldn't open: \"" + mFileName + "\" for writing",
					Toolkit::MBK_ERROR,
					mWidgets.menubar);
	message_box->go();
	message_box->unref();
    }
    else {
	UML::Writer * writer = new UML::Writer(mUML->ref_get_root());

	try {
	    writer->write(ofs);
	    mChanged = false;
	}
	catch(...) {
	    ERROR("Exception occurred whilst writing to file: " << mFileName);
	}

	writer->unref();
    }
}

void
Kit::save_as() {
    Toolkit::Factory * factory = Toolkit::Factory::instance();

    Toolkit::FileSelection * file_selection = factory->create_file_selection("Open file");

    string file_name;
    Toolkit::ResponseKind response;

    response = file_selection->go(file_name);

    file_selection->unref();

    if(response == Toolkit::RK_OK) {
	// TODO if file exists ask whether to overwrite

	mFileName = file_name;

	MESSAGE("Got file: " << file_name << ", invoking save");

	save();
    }
    else {
	MESSAGE("No OK");
    }
}

//
//
//

void
Kit::changes_occurred(const list<ChangeEvent *> & changes) throw() {
    mChanged = true;
}

void
Kit::undo_event(const UndoEvent * undo_event) {
    ostringstream ost;

    ost << "Undo event (" << undo_event->get_past() << ", " << undo_event->get_future() << ")";

    mEditMenu->set_sensitive(mCommandUndo, undo_event->get_past() > 0);
    mEditMenu->set_sensitive(mCommandRedo, undo_event->get_future() > 0);

    mWidgets.toolbar->set_sensitive(mCommandUndo, undo_event->get_past() > 0);
    mWidgets.toolbar->set_sensitive(mCommandRedo, undo_event->get_future() > 0);

    log(ost.str());
}
