// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

//
//		*********************88
//
//
// This file is a mess, a playground to test things out. When things start
// to make some sense, a better system for managing the application will
// be designed and put in place.
//
//
//
//
//
//


#ifndef KIT_H
#define KIT_H

#include "foundation/object.h"
#include "foundation/undo_listener.h"
#include "foundation/change_listener.h"

#include "toolkit/log_level.h"

#include <string>
using namespace std;

class Command;
class TreeController;
class DocumentController;

namespace Toolkit {

    class Toolbar;
    class Menu;
    class Statusbar;
    class Palette;
    class Tree;
    class Document;
    class Log;

} // namespace Toolbar

namespace UML {

    class UML;
    class ModelElement;
    class Diagram;

} // namespace UML

struct KitWidgets {
    Toolkit::Menu * menubar;
    Toolkit::Toolbar * toolbar;
    Toolkit::Statusbar * statusbar;
    Toolkit::Tree * tree;
    Toolkit::Document * document;
    Toolkit::Log * log;
};

struct KitCommands {
    Command * about;
    Command * exit;
};

class Kit : public Object, protected UndoListener, protected ChangeListener {
public:
    static void instantiate(const KitWidgets & widgets, const KitCommands & commands);
    static void shutdown();
    static Kit * instance();

    void log(const string & message, Toolkit::LogLevel level = Toolkit::LL_INFO);
    void set_diagram(UML::Diagram * diagram);
    bool can_exit();

protected:
    void initialise();

    // construction routines broken into parts

    void create_palettes();
    void create_commands();
    void create_menus();
    void create_toolbar();
    void create_connections();
    void create_controllers();

    // globals operations on the model

    void new_();
    void open();
    void save();
    void save_as();

    // mutation operations on the model

    void changes_occurred(const list<ChangeEvent *> & changes) throw();
    void undo_event(const UndoEvent * undo_event);

private:
    Kit(const KitWidgets & widgets, const KitCommands & commands);
    ~Kit();

    // widgets supplied to us during construction

    KitWidgets mWidgets;

    // commands supplied to us during construction

    KitCommands mCommands;

    // the current model
    UML::UML * mUML;

 private:
    bool mChanged;
    string mFileName;

    // the menus

    Toolkit::Menu * mFileMenu;
    Toolkit::Menu * mEditMenu;
    Toolkit::Menu * mHelpMenu;

    // the commands created by us

    Command * mCommandNew;
    Command * mCommandOpen;
    Command * mCommandSave;
    Command * mCommandSaveAs;

    Command * mCommandUndo;
    Command * mCommandRedo;

    Command * mCommandClearLog;

    // controllers

    TreeController * mTreeController;
    DocumentController * mDocumentController;

    //

    static Kit * mInstance;
};

#endif // KIT_H
