// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TREE_CONTROLLER_H
#define TREE_CONTROLLER_H

#include "foundation/object.h"
#include "toolkit/tree.h"

namespace REF {
  class Context;
}



#define USING_PYTHON

#ifdef USING_PYTHON

// only needed for PyObject
#include <Python.h>


class TreeController : public Object {
  PyObject * mObject;

public:
  explicit TreeController(const char * package, Toolkit::Tree * tree);
  virtual ~TreeController();

  void set_context(REF::Context * context);
};


#else

// stuff in "#if 1 .... #endif" has to go

#if 0
namespace UML {

    class UML;
    class ModelElement;
    class Element;
    class Diagram;
    class Class;
    class Package;
    class Operation;

} // namespace UML
#endif

class TreeController : public Object,
		       protected ChangeListener,
		       protected Toolkit::TreeMaintainer,
		       protected Toolkit::MouseListener {
public:
    explicit TreeController(Toolkit::Tree * tree);
    ~TreeController();

    void set_context(REF::Context * context);

protected:
    void changes_occurred(const list<ChangeEvent *> & changes) throw();

    //

    bool can_relocate(Toolkit::Tree * tree,
		      Object * node, Object * old_parent, Object * new_parent) const;
    void relocated(Toolkit::Tree * tree, Object * node, Object * new_parent);
    void renamed(Toolkit::Tree * tree, Object * node, const string & name);
    void selection(const set<Object *> & nodes);

    //

    bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
    bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
    bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
    bool mouse_released(const Toolkit::MouseEvent * mouse_event);
    bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
    bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);

    //

#if 1
    void cut(UML::Element * element);

    void add_operation(UML::Class * klass);
    void add_attribute(UML::Class * klass);
    void add_parameter(UML::Operation * operation);
    void add_class(UML::Package * package);
    void add_sub_class(UML::Class * klass);
    void add_sub_package(UML::Package * package);
    void add_class_diagram(UML::Package * package);
    void add_use_case_diagram(UML::Package * package);

    //

    void edit_diagram(UML::Diagram * diagram);
#endif

    //

    void recursive_build(REF::Class * parent, REF::Class * child, REF::Class * pos, bool expand);

private:

    // the tree that we maintain

    Toolkit::Tree * mTree;

    // the context of the model

    REF::Context * mContext;

    // layout rules

    struct Rule {
	string type;
	string role_name;
	string other_type;
	bool expand;
    };

    static const Rule RULES[];
    static const int NUM_RULES;

    // icon table

    struct Icon {
	string type;
	string icon;
    };

    static const Icon ICONS[];
    static const int NUM_ICONS;
};

#endif
#endif // TREE_CONTROLLER_H
