"""
  Package to translate UML Model into Tree Representation
  This module translated the model into a simple hierarchy based on ownship.

"""


icon_table = {
    "Package": "palette/package.png",
    "Model": "palette/model.png",
    "Class": "palette/class.png",
    "Operation": "palette/operation-public.png",
    "Attribute": "palette/attribute-public.png",
    "Association": "palette/aggregation.png",
    "AssociationEnd": "palette/association-end.png",
    "Generalization": "palette/generalization.png",
    "Parameter": "palette/parameter-in.png",
    "Diagram": "palette/class-diagram.png",
    "TaggedValue": "palette/tagged-value.png" }


rule_table = [
    ("ModelElement",            "tagged_value",                 "TaggedValue",          0),
    ("Namespace",               "owned_element",                "ModelElement",         1),
    ("Association",             "connection",                   "AssociationEnd",       0),
    ("Classifier",              "feature",                      "Feature",              0),
    ("BehavioralFeature",       "parameter",                    "Parameter",            0),
    ("Stereotype",              "stereotype_constraint",        "Constraint",           0),
    ("Stereotype",              "defined_tag",                  "TagDefinition",        0) ]


import Drawing
import Toolkit
import Reflective
import Foundation


used_images = [ ]
global_context = None

class Menu_Add_Role:
    def __init__(self, role_name, type_name):
        self.role_name = role_name
        self.type_name = type_name
    def add_slot(self, object):
        klass = Reflective.Class(object)
        print "Add slot", klass, klass.get_type()
        #context = klass.get_context()
        global global_context
        Foundation.begin_transaction()
        new_klass = global_context.create_class(self.type_name)
        klass.add_role(self.role_name, new_klass)
        new_klass.set_tagged_value("doco", "")
        Foundation.end_transaction()

menu_table = {
        # type_name : { stereo_type : [ menu_entries, ...], ... }
        "Class" :
        { "default" :
          [ ("add sub class", Menu_Add_Role("owned_element", "class")),
            ("add operation", Menu_Add_Role("owned_element", "operation")),
            ("add attribute", Menu_Add_Role("owned_element", "attribute")) ] },
        "Operation" :
        { "default" :
          [ ("add parameter", Menu_Add_Role("owned_element", "parameter")) ] },
        "Package" :
        { "default" :
          [ ("add Class", Menu_Add_Role("owned_element", "class")),
            ("add Sub Package", Menu_Add_Role("owned_element", "package")),
            ("add Class Diagram", Menu_Add_Role("owned_element", "diagram")) ] } }
                                          

class TreeController:
    def __init__(self, tree):
        print "New TreeController..."
        self.mTree = tree
        self.mContext = None

        # connect as a ChangeListener
        Reflective.set_change_listener(self)
        
        # connect to tree maintainer
        self.mTree.set_maintainer(self)
        
        # add a mouse listener
        self.mTree.listener_add(self)


    #================================================================================
    # Change Listener
    def changes_occurred(self, change_list):
        """Called for change in model with list of changes of the form
           get_type() returns: 'AttributeChanged'
                               'RoleChanged'
                               'RoleAdded'
                               'RoleRemoved'
           get_object() returns the class changed
           get_attr_name() gets the name of the attribute changed
           get_role_name() gets the name of the role changed/added/removed
           get_value() or get_new_value() gets the value changed/added/removed
           get_old_value() gets the old value except for role added
        """
        if not self.mContext:
            return
        
        for change in change_list:
            if change.get_type() == "AttributeChanged":
                element = change.get_object()
                attr_name = change.get_attr_name()
                if attr_name == "name":
                    new_value = change.get_new_value()
                    if self.mTree.contains(element):
                        self.mTree.set_label(element, new_value)
            
            if change.get_type() == "RoleChanged":
                pass
            if change.get_type() == "RoleAdded":
                element = change.get_object()
                if self.mTree.contains(element):
                    role_name = change.get_role_name()
                    new_value = change.get_value()
                    if not self.mTree.contains(new_value):
                        #print "RoleAdded", role_name
                        for type, name, other_type, expanded in rule_table:
                            if element.is_a_type_of(type) and role_name == name:
                                self._recursive_build(element, new_value, expanded)
            if change.get_type() == "RoleRemoved":
                #element = change.get_object()
                value = change.get_value()
                if self.mTree.contains(value):
                    self.mTree.remove(value)
            

    #================================================================================
    # Tree Maintainer
    def can_relocate(self, tree, node, old_parent, new_parent):
        print "TreeMaintainer: can_relocate"
        
    def relocated(self, tree, node, new_parent):
        print "TreeMaintainer: relocated"
        
    def renamed(self, tree, node, name):
        print "TreeMaintainer: renamed"
        #begin_transaction();
        klass = Reflective.Class(node)
        klass.set_attribute("name", name);
        #end_transaction();
        
    def selection(self, nodes):
        print "TreeMaintainer : selection"
    

    #================================================================================
    # Mouse Listener
    #
    def mouse_entered(self, event):
        print "MouseListener: entered"
    def mouse_exited(self, event):
        print "MouseListener: exited"
    def mouse_pressed(self, event):
        print "MouseListener: pressed"
        if event.get_button() == 1:
            if event.get_click_count() == 2:
                object = self.mTree.point_to_node(event.get_point())
                print "Double click on", object

        if event.get_button() == 3:
            object = self.mTree.point_to_node(event.get_point())
            klass = Reflective.Class(object)
            if object:
                menu = Toolkit.Menu()
                #menu.add_push(None, None, "Copy")
                #menu.add_push(None, None, "Paste")
                #menu.add_separator()
                if self._add_insert_menu(menu, klass):
                    menu.add_separator()
                if klass.get_type() == "Diagram":
                    menu.add_push(self._menu_slot_edit_diagram, klass, "Edit Diagram")
                menu.add_push(None, None, "Properties...")
                print "MENU POPUP"
                menu.popup(event)
                
    def mouse_released(self, event):
        print "MouseListener: released"
    def mouse_moved(self, event):
        print "MouseListener: moved"
    def mouse_dragged(self, event):
        print "MouseListener: dragged"
    
    #================================================================================

    def set_context(self, context):
        print "Tree Set Context", context
        global global_context
        global_context = context
        if self.mContext != context:
            if self.mContext:
                print "Clear the old tree"
                mTree.clear()

            self.mContext = context
            if self.mContext:
                print "Build a new tree"
                klass = self.mContext.get_root()
                self._recursive_build(None, klass, 1)
        print "Set context complete"

    def _recursive_build(self, parent, child, expanded):
        if self.mTree.contains(child):
            print "Child already in tree..."
            return

        print "Processing", child.get_attribute('name'), child.get_type()
        image = Drawing.Image(icon_table[child.get_type()])
        name = child.get_attribute("name")
        global used_images
        used_images.append(image)
        if name == "":
            name = "<unnamed>"
        print "Tree add", child, image, name, parent, expanded
        self.mTree.add(child, image, name, expanded, parent)

        for type, role_name, other_type, exp in rule_table:
            print "\tRule for ", type, role_name, other_type
            if child.is_a_type_of(type):
                for obj in child.get_role_list(role_name):
                    print "Recursive build ", obj
                    self._recursive_build(child, obj, exp)
        print "Build complete"

    def _menu_slot_add_role(self, klass, type_name, role_name):
        print "menu_add ..."
    def _menu_slot_add_sub_class(self, klass):
        print "menu_add sub class"
    def _menu_slot_add_operation(self, klass):
        print "menu add operation"
    def _menu_slot_edit_diagram(self, klass):
        print "menu edit diagram"
        
    def _add_insert_menu(self, menu, klass):
        # Hard coded for now (you can insert everything anywhere)
        type = klass.get_type()
        stereotype_list = klass.get_role_list('stereotype')
        if menu_table.has_key(type):
            list = menu_table[type]['default']
            for label, handler in list:
                print "  Adding", label
                menu.add_push(handler.add_slot, klass, label)
        return 1
    
# --------------------------------------------------------------------------------

def create(tree):
    print "Creating a tree controller for", tree
    return TreeController(tree)

