// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef COMMON_CPP_STREAM_H
#define COMMON_CPP_STREAM_H

#include <string>
#include <stack>
#include <iosfwd>
#include <sstream>
using namespace std;

namespace Common {

    enum BracketKind {
	BK_ROUND,
	BK_CURLY,
	BK_SQUARE
    };

    class CppStream {
    public:
	CppStream(ostream & ost);
	~CppStream();

	void write(const string & s);
	void checkpoint(const string & m);

	void set_indent_width(int new_value);
	int get_indent_width() const;

    protected:
	void error(const string & m) const;

    private:
	CppStream & operator = (const CppStream &) { return *this; }
	CppStream(const CppStream & xos) : mOs(xos.mOs) {  }

	stack<pair<BracketKind, int> > mStack;

	int mIndentLevel;
	int mIndentWidth;

	static const int DEFAULT_INDENT_WIDTH = 4;

	string mBuffer;

	ostream & mOs;

	string mLastCheckpoint;
    };

    template<class T>
    CppStream &
    operator << (CppStream & fos, const T & data) {
	stringstream ss;

	ss << data;

	fos.write(ss.str());

	return fos;
    }

} // namespace Common

#endif // COMMON_CPP_STREAM_H
