// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "file_utilities.h"

#include "foundation/macros.h"
#include "foundation/exception.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

namespace Common {

    void
    ensure_directory(const string & dir) {
	struct stat buf;

	if(stat(dir.c_str(), &buf) == 0) {
	    // exists, make sure it is a directory

	    if(!S_ISDIR(buf.st_mode)) {
		ERROR("Node exists, but is not a directory: " << dir);
		throw Exception("FileError",
				"Node exists, but is not a directory: " + dir);
	    }
	}
	else {
	    // doesn't exist, have to create it

	    string::size_type index = 0;

	    do {
		index = dir.find('/', ++index);

		string part = dir.substr(0, index);

		if(stat(part.c_str(), &buf) == 0) {
		    // exists, make sure it is a directory

		    if(!S_ISDIR(buf.st_mode)) {
			ERROR("Node exists, but is not a directory: " << part);
			throw Exception("FileError",
					"Node exists, but is not a directory: " + dir);
		    }
		}
		else {
		    // doesn't exist, have to create it

		    if(mkdir(part.c_str(), 0777)) {
			ERROR("Error occured creating directory: " << part);
			throw Exception("FileError", "Error creating directory: " + part);
		    }
		}
	    } while(index != string::npos);
	}
    }

} // namespace Common
