// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "type_code_handler.h"

#include "foundation/macros.h"

namespace Common {

    TypeCodeHandler::TypeCodeHandler() : XML::SAXHandler() {
	TRACE(10, "TypeCodeHandler created");

	mHandlerMap["XMI.CorbaTypeCode"] =
	    make_pair(&TypeCodeHandler::xmi__corba_type_code__start,
		      &TypeCodeHandler::xmi__corba_type_code__end);

	mHandlerMap["XMI.CorbaTcEnum"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_enum__start,
		      &TypeCodeHandler::xmi__corba_tc_enum__end);

	mHandlerMap["XMI.CorbaTcEnumLabel"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_enum_label__start,
		      &TypeCodeHandler::xmi__corba_tc_enum_label__end);

	mHandlerMap["XMI.CorbaTcLong"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_long__start,
		      &TypeCodeHandler::xmi__corba_tc_long__end);

	mHandlerMap["XMI.CorbaTcString"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_string__start,
		      &TypeCodeHandler::xmi__corba_tc_string__end);

	mHandlerMap["XMI.CorbaTcBoolean"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_boolean__start,
		      &TypeCodeHandler::xmi__corba_tc_boolean__end);

	mHandlerMap["XMI.CorbaTcStruct"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_struct__start,
		      &TypeCodeHandler::xmi__corba_tc_struct__end);

	mHandlerMap["XMI.CorbaTcField"] =
	    make_pair(&TypeCodeHandler::xmi__corba_tc_field__start,
		      &TypeCodeHandler::xmi__corba_tc_field__end);
    }

    TypeCodeHandler::~TypeCodeHandler() {
	TRACE(10, "TypeCodeHandler destroyed");
    }

    void
    TypeCodeHandler::warning(XML::SAXParser * parser, const string & msg) {
	WARNING("Unhandled warning: " << msg);
    }

    void
    TypeCodeHandler::error(XML::SAXParser * parser, const string & msg) {
	WARNING("Unhandled error: " << msg);
    }

    void
    TypeCodeHandler::fatal(XML::SAXParser * parser, const string & msg) {
	WARNING("Unhandled fatal: " << msg);
    }

    void
    TypeCodeHandler::comment(XML::SAXParser * parser, const string & comment) {
    }

    void
    TypeCodeHandler::start_document(XML::SAXParser * parser) {
    }

    void
    TypeCodeHandler::end_document(XML::SAXParser * parser) {
    }

    void
    TypeCodeHandler::start_element(XML::SAXParser * parser, const string & name,
				   const XML::Properties & properties) {
	TRACE(5, "Start: " << name);

	mCharacters.resize(0);

	hash_map<string, HandlerPair>::iterator iter = mHandlerMap.find(name);

	if(iter == mHandlerMap.end()) {
	    if(mUnhandledElements.insert(name).second) {
		WARNING("Unhandled element: " << name);
	    }
	}
	else {
	    HandlerPair handler_pair = iter->second;
	    (this->*handler_pair.first)(parser, properties);
	}
    }

    void
    TypeCodeHandler::end_element(XML::SAXParser * parser, const string & name) {
	hash_map<string, HandlerPair>::iterator iter = mHandlerMap.find(name);

	if(iter == mHandlerMap.end()) {
	}
	else {
	    HandlerPair handler_pair = iter->second;
	    (this->*handler_pair.second)(parser);
	}

	TRACE(5, "End: " << name);
    }

    void
    TypeCodeHandler::characters(XML::SAXParser * parser, const string & data) {
	mCharacters += data;
    }

    void
    TypeCodeHandler::reference(XML::SAXParser * parser, const string & name) {
	WARNING("Unhandled Reference: " << name);
    }

    void
    TypeCodeHandler::cdata_block(XML::SAXParser * parser, const string & data) {
	WARNING("Unhandled CDATA: " << data);
    }

    //
    //
    //

    //
    // support a limited range of data-types with a simplistic scheme.
    //
    //   Support:
    //       enums
    //       primitives (long, string, bool)
    //       structs of primitives (not nested)

    void
    TypeCodeHandler::xmi__corba_type_code__start(SAXParser * parser,
						 const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_type_code__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_enum__start(SAXParser * parser,
					       const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_enum__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_enum_label__start(SAXParser * parser,
						     const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_enum_label__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_long__start(SAXParser * parser,
					       const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_long__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_string__start(SAXParser * parser,
						 const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_string__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_boolean__start(SAXParser * parser,
						  const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_boolean__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_struct__start(SAXParser * parser,
						 const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_struct__end(SAXParser * parser) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_field__start(SAXParser * parser,
						const Properties & properties) {
    }

    void
    TypeCodeHandler::xmi__corba_tc_field__end(SAXParser * parser) {
    }

} // namespace Common
