// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef COMMON_UUID_H
#define COMMON_UUID_H

#include <iosfwd>
#include <string>
using namespace std;

namespace Common {

    class UUID {
    public:
	UUID();
	~UUID();

	static UUID create();

	void generate();
	void clear();

	string get_string() const;
	void set_string(const string & s);

	operator bool () const;
	bool operator ! () const;
	
    protected:
	void generate_random();
	void generate_time();

    private:
	union {
	    struct {
		unsigned int mTimeLow;
		unsigned short int mTimeMid;
		unsigned short int mTimeHiAndVersion;
		unsigned short int mClockSeq;
		unsigned char mNode[6];
	    };

	    char mRaw[16];
	};

	static bool mInitialised;
	static bool mHaveRandom;
	static unsigned char mNodeID[6];
    };

    bool operator == (const UUID & uuid1, const UUID & uuid2);
    bool operator < (const UUID & uuid1, const UUID & uuid2);

    ostream & operator << (ostream & ost, const UUID & uuid);
    istream & operator >> (istream & ist, UUID & uuid);

} // namespace Common

#endif // COMMON_UUID
