// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef COMMON_XMI_HANDLER_H
#define COMMON_XMI_HANDLER_H

#include "xml/sax_handler.h"

#include "foundation/hash_functions.h"
#include "foundation/hash.h"

#include <set>
using namespace std;

namespace Common {

    using namespace XML;

    class XMIHandler : public SAXHandler {
    public:
	explicit XMIHandler(SAXHandler * content_handler);
	~XMIHandler();

	Object * get_content() { return mContent; }

	void warning(SAXParser * parser, const string & msg);
	void error(SAXParser * parser, const string & msg);
	void fatal(SAXParser * parser, const string & msg);

	void comment(SAXParser * parser, const string & comment);
	void start_document(SAXParser * parser);
	void end_document(SAXParser * parser);
	void start_element(SAXParser * parser, const string & name, const Properties & properties);
	void end_element(SAXParser * parser, const string & name);
	void characters(SAXParser * parser, const string & data);
	void reference(SAXParser * parser, const string & name);
	void cdata_block(SAXParser * parser, const string & data);

    private:
	void xmi__start(SAXParser * parser, const Properties & properties);
	void xmi__end(SAXParser * parser);

	void xmi__header__start(SAXParser * parser, const Properties & properties);
	void xmi__header__end(SAXParser * parser);

	void xmi__documentation__start(SAXParser * parser, const Properties & properties);
	void xmi__documentation__end(SAXParser * parser);

	void xmi__owner__start(SAXParser * parser, const Properties & properties);
	void xmi__owner__end(SAXParser * parser);

	void xmi__contact__start(SAXParser * parser, const Properties & properties);
	void xmi__contact__end(SAXParser * parser);

	void xmi__long_description__start(SAXParser * parser, const Properties & properties);
	void xmi__long_description__end(SAXParser * parser);

	void xmi__short_description__start(SAXParser * parser, const Properties & properties);
	void xmi__short_description__end(SAXParser * parser);

	void xmi__exporter__start(SAXParser * parser, const Properties & properties);
	void xmi__exporter__end(SAXParser * parser);

	void xmi__exporter_version__start(SAXParser * parser, const Properties & properties);
	void xmi__exporter_version__end(SAXParser * parser);

	void xmi__exporter_id__start(SAXParser * parser, const Properties & properties);
	void xmi__exporter_id__end(SAXParser * parser);

	void xmi__notice__start(SAXParser * parser, const Properties & properties);
	void xmi__notice__end(SAXParser * parser);

	void xmi__model__start(SAXParser * parser, const Properties & properties);
	void xmi__model__end(SAXParser * parser);

	void xmi__metamodel__start(SAXParser * parser, const Properties & properties);
	void xmi__metamodel__end(SAXParser * parser);

	void xmi__metametamodel__start(SAXParser * parser, const Properties & properties);
	void xmi__metametamodel__end(SAXParser * parser);

	void xmi__import__start(SAXParser * parser, const Properties & properties);
	void xmi__import__end(SAXParser * parser);

	void xmi__content__start(SAXParser * parser, const Properties & properties);
	void xmi__content__end(SAXParser * parser);

	void xmi__extensions__start(SAXParser * parser, const Properties & properties);
	void xmi__extensions__end(SAXParser * parser);

	void xmi__extension__start(SAXParser * parser, const Properties & properties);
	void xmi__extension__end(SAXParser * parser);

	void xmi__difference__start(SAXParser * parser, const Properties & properties);
	void xmi__difference__end(SAXParser * parser);

	void xmi__delete__start(SAXParser * parser, const Properties & properties);
	void xmi__delete__end(SAXParser * parser);

	void xmi__add__start(SAXParser * parser, const Properties & properties);
	void xmi__add__end(SAXParser * parser);

	void xmi__replace__start(SAXParser * parser, const Properties & properties);
	void xmi__replace__end(SAXParser * parser);

	void xmi__reference__start(SAXParser * parser, const Properties & properties);
	void xmi__reference__end(SAXParser * parser);

	void xmi__any__start(SAXParser * parser, const Properties & properties);
	void xmi__any__end(SAXParser * parser);

	//
	//
	//

	string mCharacters;

	typedef void (XMIHandler::*StartHandler)(SAXParser *, const Properties &);
	typedef void (XMIHandler::*EndHandler)(SAXParser *);
	typedef pair<StartHandler, EndHandler> HandlerPair;

	hash_map<string, HandlerPair> mHandlerMap;

	set<string> mUnhandledElements;

	SAXHandler * mContentHandler;
	Object * mContent;
    };

} // namespace Common

#endif // COMMON_XMI_HANDLER_H
