// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_FIG_EDGE_H
#define DIAGRAM_FIG_EDGE_H

#include "fig_poly.h"
#include "toolkit/mouse_listener.h"

#include "drawing/drawable.h"

#include <list>
using namespace std;

namespace Diagram {

    class FigNode;
    class FigEdgeText;

    // TODO this probably should become a MouseListener, just like FigNode

    class FigEdge : public FigPoly, public Toolkit::MouseListener {
    public:
	enum Part {
	    SOURCE,
	    DESTINATION,
	    EDGE
	};

	FigEdge(Object * owner, FigNode * source, FigNode * destination);
	~FigEdge();

	//
	//
	//

	void resize(const Drawing::Vector & size);

	FigNode * get_source_fig() { return mSourceFig; }
	FigNode * get_destination_fig() { return mDestinationFig; }

	void set_source_fig(FigNode * source_fig_node);
	void set_destination_fig(FigNode * destination_fig_node);
	void remove_node(FigNode * node);

	void cleanup();

	void add_point(const Drawing::Point & point);
	void set_point(Handle & handle, const Drawing::Point & point);
	void remove_point(int index);
	void insert_point(int index, const Drawing::Point & point);

	Selection * create_selection();

	void set_layer(Layer * layer);

	virtual void update_edge_to_nodes();

	//
	// optional overrides
	//

	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);

    protected:
	virtual void compute_route();
	virtual void compute_text_positions();

	void update_bounds_to_points();

	bool mUseNearest;
	bool mHighlight;

	FigNode * mSourceFig;
	FigNode * mDestinationFig;

	list<FigEdgeText *> mAnnotations;

    private:
	void solve(const Drawing::Point & point, const Drawing::Rectangle & rectangle,
		   Drawing::Point & result);
    };

} // namespace Diagram

#endif // DIAGRAM_FIG_EDGE_H
