// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_FIG_NODE_H
#define DIAGRAM_FIG_NODE_H

#include "fig_group.h"
#include "toolkit/mouse_listener.h"

#include <list>
#include <vector>
using namespace std;

namespace Drawing {

    class Point;

} // namespace Drawing

namespace Diagram {

    class FigEdge;

    class FigNode : public FigGroup, public Toolkit::MouseListener {
    public:
	explicit FigNode(Object * owner);
	~FigNode();

	//
	//
	//

	void relocate(const Drawing::Point & location);
	void resize(const Drawing::Vector & size);

	void add_edge(FigEdge * fig_edge);
	void remove_edge(FigEdge * fig_edge);

	const list<FigEdge *> & get_edges() { return mFigEdges; }

	void paint(Drawing::Drawable * drawable) const;

	void set_highlight(bool highlight);
	bool get_highlight() const { return mHighlight; }

	void cleanup();

	Selection * create_selection();

	//
	// optional overrides
	//

	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);

    protected:
	list<FigEdge *> mFigEdges;

	// list<FigNode *> mOccupants;	TODO

	virtual void update_edges();

    private:
	bool mHighlight;
    };

} // namespace Diagram

#endif // DIAGRAM_FIG_NODE_H
