// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by The Regents of the University of California.

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_FIG_SPLINE_H
#define DIAGRAM_FIG_SPLINE_H

#include "fig_poly.h"

#include "drawing/point.h"

namespace Diagram {

    class FigSpline : public FigPoly {
    public:
	FigSpline();
	~FigSpline();

	void paint(Drawing::Drawable * drawable) const;

	bool hit(const Drawing::Rectangle & rect) const;

	void relocate(const Drawing::Point & position);
	void resize(const Drawing::Vector & size);

    protected:
	void make_bezier(const Drawing::Point & p1, const Drawing::Point & p2,
			 const Drawing::Point & p3);
	void set_junction_point(const Drawing::Point & p1, const Drawing::Point & p2,
				const Drawing::Point & p3);

	void update_bounds_to_points();

	void update_curve();

    private:
	vector<Drawing::Point> mCurve;
	vector<Drawing::Point> mKnots;

	Drawing::Point mJunction;

	static const int SPLINE_THRESHOLD = 2;
	static const double OTHER_THRESHOLD = 5.0;
	static const int ARR_WIDTH = 5;
	static const int ARR_HEIGHT = 16;
    };

} // namespace Diagram

#endif // DIAGRAM_FIG_SPLINE_H
