#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_text.h"

#include "drawing/drawable.h"
#include "drawing/factory.h"

#include "foundation/macros.h"

namespace Diagram {

    const string FigText::DEFAULT_TEXT_COLOR = "#000000";

    FigText::FigText() : Fig(),
        mTextColor(DEFAULT_TEXT_COLOR) {

	Drawing::Factory * factory = Drawing::Factory::instance();

	mText = factory->create_text();

	mText->set_alignment(Drawing::Text::A_CENTER);
	mText->set_wrap_mode(Drawing::Text::WM_CHAR);
    }

    FigText::~FigText() {
	mText->unref();
    }

    void
    FigText::paint(Drawing::Drawable * drawable) const {
#ifdef DEBUG
	//drawable->set_color("#ff8888");
	//drawable->draw_rectangle(mBounds, true);
#endif

	drawable->set_color(get_text_color());
	drawable->draw_text(mText, Drawing::Point(mBounds.get_x(), mBounds.get_y()));
    }

    bool
    FigText::hit(const Drawing::Rectangle & rect) const {
	return mBounds & rect;
    }

    void
    FigText::set_text(const string & text) {
	mText->set_text(text);
	update_bounds_to_text();
    }

    void
    FigText::set_text_color(const string & text_color) {
	if(mTextColor != text_color) {
	    mTextColor = text_color;

	    damage();
	}
    }

    void
    FigText::set_font(const string & font) {
	mText->set_font(font);
	update_bounds_to_text();
    }

    bool
    FigText::mouse_entered(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::mouse_exited(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::mouse_released(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::mouse_moved(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
	return true;
    }

    bool
    FigText::key_pressed(const Toolkit::KeyEvent * key_event) {
	return true;
    }

    bool
    FigText::key_released(const Toolkit::KeyEvent * key_event) {
	return true;
    }

    void
    FigText::set_width(int width) {
	if(mBounds.get_w() != width) {
	    int height = mText->set_width(width);

	    damage();
	    mBounds.resize(width, height);
	    damage();
	}
    }

    void
    FigText::resize(const Drawing::Vector & size) {
	if(mBounds.get_w() != size.get_x() || mBounds.get_h() != size.get_y()) {
	    damage();
	    mText->set_width(size.get_x());
	    mBounds.resize(size.get_x(), size.get_y());
	    damage();
	}
    }

    void
    FigText::update_bounds_to_text() {
	int width = mBounds.get_w();

	int height = mText->set_width(width);

	if(mBounds.get_w() != width || mBounds.get_h() != height) {
	    damage();
	    mBounds.resize(width, height);
	    damage();
	}
    }

} // namespace Diagram
