// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_FIG_TEXT_H
#define DIAGRAM_FIG_TEXT_H

#include "fig.h"

#include "toolkit/mouse_listener.h"
#include "toolkit/key_listener.h"

#include "drawing/text.h"

#include <string>
using namespace std;

namespace Diagram {

    class FigText : public virtual Fig, public Toolkit::MouseListener, public Toolkit::KeyListener {
    public:
	FigText();
	~FigText();

	void paint(Drawing::Drawable * drawable) const;

	bool hit(const Drawing::Rectangle & rect) const;

	virtual void set_text(const string & text);

	virtual void set_text_color(const string & text_color);
	const string & get_text_color() const { return mTextColor; }

	virtual void set_font(const string & font);

	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);

	bool key_pressed(const Toolkit::KeyEvent * key_event);
	bool key_released(const Toolkit::KeyEvent * key_event);

	void set_width(int width);

	void resize(const Drawing::Vector & size);

    protected:
	virtual void update_bounds_to_text();

	static const string DEFAULT_TEXT_COLOR;
	static const string DEFAULT_FONT;

    private:
	string mTextColor;

	Drawing::Text * mText;
    };

} // namespace Diagram

#endif // DIAGRAM_FIG_TEXT_H
