// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_LAYER_MANAGER_H
#define DIAGRAM_LAYER_MANAGER_H

#include "foundation/object.h"

#include <list>
using namespace std;

namespace Drawing {

    class Drawable;
    class Rectangle;

} // namespace Drawing

namespace Diagram {

    class Layer;
    class Fig;
    class Editor;

    class LayerManager : public Object {
    public:
	explicit LayerManager(Editor * editor);

	void add(Layer * layer);
	void remove(Layer * layer);

	void add(Fig * fig);
	void remove(Fig * fig);
	void clear();

	void paint(Drawing::Drawable * drawable) const;
	void print(Drawing::Drawable * drawable) const;

	void set_editor(Editor * editor);
	Editor * get_editor() const;

	Fig * hit(const Drawing::Rectangle & rect) const;
	Fig * hit_not(const Drawing::Rectangle & rect, Fig * disregard) const;

	const list<Fig *> & get_figs();

	void set_active_layer(Layer * active_layer);
	Layer * get_active_layer() { return mActiveLayer; }

    protected:
	~LayerManager();

    private:
	Editor * mEditor;
	Layer * mActiveLayer;

	list<Layer *> mLayers;
    };

} // namespace Diagram

#endif // DIAGRAM_LAYER_MANAGER_H
