// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "mode.h"

#include "toolkit/mouse_event.h"
#include "foundation/macros.h"

#include "editor.h"

namespace Diagram {

    const string Mode::DEFAULT_LINE_COLOR = "#0055ff";
    const string Mode::DEFAULT_FILL_COLOR = "#00ff00";

    Mode::Mode() : Toolkit::MouseListener(),
        mCursor(Toolkit::C_DEFAULT),
	mRunning(false),
	mReplaceable(true),
	mSticky(false),
	mLineColor(DEFAULT_LINE_COLOR),
	mFillColor(DEFAULT_FILL_COLOR) {
    }

    Mode::~Mode() {
	if(mRunning) {
	    WARNING("Running during destruction");
	}
    }

    //

    bool
    Mode::mouse_moved(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Mode::mouse_dragged(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Mode::mouse_pressed(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Mode::mouse_released(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Mode::mouse_exited(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    bool
    Mode::mouse_entered(const Toolkit::MouseEvent * mouse_event) {
	return false;
    }

    //

    bool
    Mode::key_pressed(const Toolkit::KeyEvent * key_event) {
	return false;
    }

    bool
    Mode::key_released(const Toolkit::KeyEvent * key_event) {
	return false;
    }
    //

    void
    Mode::damage() {
	Editor * editor = Editor::current();

	RETURN_IF_FAIL(editor);

	editor->damage(this);
    }

    //

    void
    Mode::start(const Toolkit::MouseEvent * mouse_event) {
	mRunning = true;

	Editor * editor = Editor::current();

	RETURN_IF_FAIL(editor);

	editor->push(this);		// This makes sure that we are on the top of the editor
					// stack, just in case we aren't.
					//
					// If we are an indirect mode, such as one of the
					// ones created by ModeSelect, then we might not
					// be on the top of the stack
    }

    void
    Mode::finish(const Toolkit::MouseEvent * mouse_event) {
	if(mRunning) {
	    mBounds.clear();

	    Editor * editor = Editor::current();

	    ASSERT(editor);

	    editor->pop();

	    mRunning = false;
	}
	else {
	    WARNING("Not running");
	}
    }

    void
    Mode::cancel() {
	if(mRunning) {
	    damage();

	    mBounds.clear();

	    mRunning = false;
	}
	else {
	    WARNING("Not running");
	}
    }

    bool
    Mode::intersects(const Drawing::Rectangle & rect) const {
	return mBounds & rect;
    }

    void
    Mode::paint(Drawing::Drawable * drawable) const {
	// most modes are invisible, but subclasses can override this
    }

} // namespace Diagram
