// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_MODE_H
#define DIAGRAM_MODE_H

#include "foundation/object.h"
#include "drawing/rectangle.h"
#include "toolkit/mouse_listener.h"
#include "toolkit/key_listener.h"
#include "toolkit/cursor.h"

namespace Drawing {

    class Drawable;

} // namespace Drawing


namespace Toolkit {

    class MouseEvent;
    class KeyEvent;

} // namespace Toolkit

#include <string>
using namespace std;

namespace Diagram {

    class Mode : public Object, public Toolkit::MouseListener, public Toolkit::KeyListener {
    public:
	Mode();
	~Mode();

	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);

	bool key_pressed(const Toolkit::KeyEvent * key_event);
	bool key_released(const Toolkit::KeyEvent * key_event);

	void damage();

	const Drawing::Rectangle & get_bounds() const { return mBounds; }

	void set_replaceable(bool replaceable) { mReplaceable = replaceable; }
	void set_sticky(bool sticky) { mSticky = sticky; }

	bool intersects(const Drawing::Rectangle & rect) const;

	bool is_running() const { return mRunning; }

	bool is_replaceable() const { return mReplaceable; }		// pushed mode replaces it
	bool is_sticky() const { return mSticky; }			// doesn't get popped

	virtual string get_instructions() const = 0;

	virtual void start(const Toolkit::MouseEvent * mouse_event);
	virtual void finish(const Toolkit::MouseEvent * mouse_event);
	virtual void cancel();

	virtual void paint(Drawing::Drawable * drawable) const;

	const string & get_line_color() const { return mLineColor; }
	const string & get_fill_color() const { return mFillColor; }

	Toolkit::Cursor get_cursor() const { return mCursor; }

    protected:
	Drawing::Rectangle mBounds;

	static const string DEFAULT_LINE_COLOR;
	static const string DEFAULT_FILL_COLOR;

	Toolkit::Cursor mCursor;

    private:
	bool mRunning;
	bool mReplaceable;
	bool mSticky;

	string mLineColor, mFillColor;
    };

} // namespace Diagram

#endif // DIAGRAM_MODE_H
