// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_MODE_MANAGER_H
#define DIAGRAM_MODE_MANAGER_H

#include "foundation/object.h"
#include "toolkit/mouse_listener.h"
#include "toolkit/key_listener.h"

#include <stack>
#include <list>
using namespace std;

namespace Toolkit {

    class MouseEvent;

} // namespace Toolkit

namespace Drawing {

    class Drawable;

} // namespace Drawing

namespace Diagram {

    class ModeChangeListener;
    class Mode;

    class ModeManager : public Object, public Toolkit::MouseListener, public Toolkit::KeyListener {
    public:
	ModeManager();

	void add(ModeChangeListener * mode_change_listener);
	void remove(ModeChangeListener * mode_change_listener);

	void push(Mode * mode);
	void replace(Mode * mode);
	void pop();

	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);
	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);

	bool key_pressed(const Toolkit::KeyEvent * key_event);
	bool key_released(const Toolkit::KeyEvent * key_event);

	void paint(Drawing::Drawable * drawable) const;

    protected:
	~ModeManager();

    private:
	Mode * mGrabbedMode;

	stack<Mode *> mModes;
	list<ModeChangeListener *> mModeChangeListeners;
    };

} // namespace Diagram

#endif // DIAGRAM_MODE_MANAGER_H
