// This code is a modified port of GEF (Graphical Editing Framework)
// http://gef.tigris.org/
// GEF is copyrighted by the Regents of the University of California

// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DIAGRAM_SELECTION_H
#define DIAGRAM_SELECTION_H

#include "foundation/object.h"
#include "toolkit/mouse_listener.h"
#include "toolkit/key_listener.h"

#include "drawing/rectangle.h"

#include "fig.h"

#include <string>
using namespace std;

namespace Drawing {

    class Drawable;
    class Point;
    class Vector;

} // namespace Drawing

namespace Toolkit {

    class MouseEvent;

} // namespace Toolkit

namespace Diagram {

    class Handle;

    class Selection : public Object, public Toolkit::MouseListener, public Toolkit::KeyListener {
    public:
	explicit Selection(Fig * fig);

	bool mouse_moved(const Toolkit::MouseEvent * mouse_event);
	bool mouse_dragged(const Toolkit::MouseEvent * mouse_event);
	bool mouse_pressed(const Toolkit::MouseEvent * mouse_event);
	bool mouse_released(const Toolkit::MouseEvent * mouse_event);
	bool mouse_entered(const Toolkit::MouseEvent * mouse_event);
	bool mouse_exited(const Toolkit::MouseEvent * mouse_event);

	bool key_pressed(const Toolkit::KeyEvent * key_event);
	bool key_released(const Toolkit::KeyEvent * key_event);

	bool contains(const Drawing::Point & point) const;

	const Drawing::Rectangle & get_bounds() const { return mBounds; }
	const Drawing::Rectangle & get_contents_bounds() const { return mContent->get_bounds(); }

	void damage();

	void set_content(Fig * content);
	Fig * get_content() { return mContent; }
	bool is_content(const Fig * fig) const { return mContent == fig; }

	virtual void drag_handle(const Drawing::Point & m, const Drawing::Point & an,
				 Handle & handle) = 0;
	virtual void hit_handle(const Drawing::Rectangle & rect, Handle & handle) const = 0;

	bool hit(const Drawing::Rectangle & rect) const;
	bool intersects(const Drawing::Rectangle & rect) const;

	virtual void paint(Drawing::Drawable * drawable) const = 0;

	void translate(const Drawing::Vector & offset);

	virtual void cleanup();

	const string & get_line_color() const { return mLineColor; }
	const string & get_fill_color() const { return mFillColor; }

	void begin_interaction();
	void end_interaction();

    protected:
	~Selection();

	static const int HAND_SIZE;
	static const int BORDER_WIDTH;
	static const string DEFAULT_LINE_COLOR;
	static const string DEFAULT_FILL_COLOR;

	virtual void update_bounds_to_content();

	// subclasses have access to these so that they can keep them up-to-date
	Drawing::Rectangle mBounds;
	Fig * mContent;

	void paint_rectangle(const Drawing::Point & point, Drawing::Drawable * drawable) const;

    private:
	string mLineColor, mFillColor;
    };

} // namespace Diagram

#endif // DIAGRAM_SELECTION_H
