// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DRAWING_DRAWABLE_H
#define DRAWING_DRAWABLE_H

#include "foundation/object.h"

#include "line_style.h"

#include <string>
#include <vector>
using namespace std;

namespace Drawing {

    class Rectangle;
    class Point;
    class Vector;
    class Text;
    class Image;

    class Drawable : public virtual Object {
    public:
	Drawable();

	virtual void set_color(const string & color_string) = 0;
	virtual void set_line_width(int width) = 0;
	virtual void set_line_style(LineStyle line_style) = 0;

	virtual void draw_point(const Point & point) = 0;
	virtual void draw_polygon(const vector<Point> & points, bool filled) = 0;
	virtual void draw_lines(const vector<Point> & points) = 0;
	virtual void draw_rectangle(const Rectangle & rectangle, bool filled) = 0;
	virtual void draw_line(const Point & point1, const Point & point2) = 0;
	virtual void draw_oval(const Rectangle & rectangle, bool filled) = 0;
	virtual void draw_text(Text * text, const Point & point) = 0;
	virtual void draw_image(Image * image, const Point & point) = 0;

	virtual void push_clip(const Rectangle & rectangle) = 0;
	virtual void pop_clip() = 0;
	virtual const Rectangle & get_clip() const = 0;		// FIXME change to take a reference

	virtual const Vector & get_size() const = 0;

    protected:
	~Drawable();
    };

} // namespace Drawing

#endif // DRAWING_DRAWABLE_H
