// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "image.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

#include "foundation/macros.h"
#include "foundation/standard_exceptions.h"

namespace Drawing {

    Image::Image() : Object() {
    }

    Image::~Image() {
    }

    string
    Image::get_path_to_image(const string & filename) const {
	const string SEARCH_PATH = IMAGE_SEARCH_PATH;

	string::size_type i = 0;
	string::size_type j = 0;

	do {
	    j = SEARCH_PATH.find(':', i);

	    string s = SEARCH_PATH.substr(i, j - i);

	    string path = s + "/" + filename;

	    struct stat buf;

	    if(!stat(path.c_str(), &buf)) {
		return path;
	    }

	    i = j + 1;
	} while(j != string::npos);

	throw FileNotFoundException(filename);
    }

    string
    Image::get_raw_name(const string & filename) const {
	string raw = filename;

	string::size_type i;

	i = raw.rfind("/");

	if(i != string::npos) {
	    raw = raw.substr(i + 1);
	}

	i = raw.rfind(".");

	if(i != string::npos) {
	    raw = raw.substr(0, raw.length() - i);
	}

	return raw;
    }

} // namespace Drawing
