// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DRAWING_PAGE_H
#define DRAWING_PAGE_H

#include "foundation/object.h"

// Paper Size              Dimension (in points)
// ------------------      ---------------------
// Comm #10 Envelope       297 x 684
// C5 Envelope             461 x 648
// DL Envelope             312 x 624
// Folio                   595 x 935
// Executive               522 x 756
// Letter                  612 x 792
// Legal                   612 x 1008
// Ledger                  1224 x 792
// Tabloid                 792 x 1224
// A0                      2384 x 3370
// A1                      1684 x 2384
// A2                      1191 x 1684
// A3                      842 x 1191
// A4                      595 x 842
// A5                      420 x 595
// A6                      297 x 420
// A7                      210 x 297
// A8                      148 x 210
// A9                      105 x 148
// B0                      2920 x 4127
// B1                      2064 x 2920
// B2                      1460 x 2064
// B3                      1032 x 1460
// B4                      729 x 1032
// B5                      516 x 729
// B6                      363 x 516
// B7                      258 x 363
// B8                      181 x 258
// B9                      127 x 181
// B10                     91 x 127

namespace Drawing {

    enum PaperSize {
	PS_US_LETTER,
	PS_US_LEGAL,
	PS_A3,
	PS_A4,
	PS_A5,
	PS_B4,
	PS_B5,
	PS_B5_JAPAN,
	PS_HALF_LETTER,
	PS_EXECUTIVE,
	PS_TABLOID_LEDGER,
	PS_MONARCH,
	PS_SUPERB,
	PS_ENVELOPE_COMMERCIAL,
	PS_ENVELOPE_MONARCH,
	PS_ENVELOPE_DL,
	PS_ENVELOPE_C5,
	PS_EURO_POSTCARD
    };

    enum OrientationType {
	OT_PORTRAIT,
	OT_LANDSCAPE
    };

    enum {
	ST_PAGE,
	ST_ADJUST,
	ST_FIT_TO
    };

    class Page : virtual public Object {
    public:
	float get_header() const { return mHeader; }
	float get_footer() const { return mFooter; }

	void set_header(float header) { mHeader = header; }
	void set_footer(float footer) { mFooter = footer; }

    protected:
	Page();
	~Page();

    private:
	bool mCenterHorizontally;
	bool mCenterVertically;

	float mLeftMargin;
	float mRightMargin;
	float mHeader;
	float mFooter;
    };

} // namespace Drawing

#endif // DRAWING_PAGE_H
