// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef DRAWING_VECTOR_H
#define DRAWING_VECTOR_H

#include <iosfwd>
using namespace std;

namespace Drawing {

    class Point;

    class Vector {
    public:
	Vector();
	Vector(int x, int y);

	void set(int x, int y) { mX = x; mY = y; }

	int get_x() const { return mX; }
	int get_y() const { return mY; }

	void set_x(int x) { mX = x; }
	void set_y(int y) { mY = y; }

	operator bool () const { return mX != 0 || mY != 0; }
	bool operator ! () const { return !operator bool(); }

	Vector & operator += (const Vector & v);
	Vector & operator -= (const Vector & v);

	double get_length() const;

	Vector get_normal() const;

    private:
	int mX, mY;
    };

    bool operator == (const Vector & p1, const Vector & p2);
    bool operator != (const Vector & p1, const Vector & p2);

    Vector operator + (const Vector & v1, const Vector & v2);
    Vector operator - (const Vector & v1, const Vector & v2);

    Vector operator - (const Point & p1, const Point & p2);

    Vector operator - (const Vector & v);

    int operator * (const Vector & v1, const Vector & v2);

    Vector operator * (const Vector & v, double s);
    inline Vector operator * (double s, const Vector & v) { return v * s; }

    ostream & operator << (ostream & ost, const Vector & v);
    istream & operator >> (istream & ist, Vector & v);

} // namespace Drawing

#endif // DRAWING_VECTOR_H
