// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include <gtk/gtkmain.h>
#include <gtk/gtkwindow.h>
#include <gtk/gtkvbox.h>

#include "gtk/menu.h"

#include "foundation/macros.h"
#include "foundation/exception.h"
#include "foundation/command_debug.h"
#include "foundation/command_callback.h"

#ifdef DEBUG
int trace_level = 0;
#endif

extern "C" {
    int FT_Get_Next_Char;
    int FT_Get_First_Char;
}

namespace {

    gint
    delete_event_callback(GtkWidget * widget, GdkEvent * event, gpointer data) {
	return FALSE;		// this will cause the destroy event to be emitted
    }

    void
    destroy_callback(GtkWidget * widget, gpointer data) {
	gtk_main_quit();
    }

} // namespace {anonymous}

int
main(int argc, char * argv[]) try {
    gtk_init(&argc, &argv);

    GtkWidget * toplevel = gtk_window_new(GTK_WINDOW_TOPLEVEL);

    g_signal_connect(G_OBJECT(toplevel), "delete_event", G_CALLBACK(&delete_event_callback), 0);
    g_signal_connect(G_OBJECT(toplevel), "destroy", G_CALLBACK(&destroy_callback), 0);

    GtkWidget * vbox = gtk_vbox_new(FALSE, 0);

    gtk_container_add(GTK_CONTAINER(toplevel), vbox);

    //

    GTK::Menu * _menu = new GTK::Menu(true);

    GtkWidget * menu = _menu->get_base_widget();

    gtk_box_pack_start(GTK_BOX(vbox), menu, FALSE, FALSE, 0);

    //

    {
	GTK::Menu * _submenu;

	//
	
	_submenu = new GTK::Menu();

	_submenu->add_push(new CommandDebug("New"), "New");
	_submenu->add_push(command_callback(&gtk_main_quit), "Exit");

	_menu->add_submenu(_submenu, "File");

	//

	_submenu = new GTK::Menu();

	_submenu->add_check(new CommandDebug("Check1"), "Check1");
	_submenu->add_check(new CommandDebug("Check2"), "Check2");
	_submenu->add_check(new CommandDebug("Check3"), "Check3");
	_submenu->add_check(new CommandDebug("Check4"), "Check4");

	_submenu->add_separator();

	Command * group;

	_submenu->add_radio(group = new CommandDebug("Radio1"), "Radio1", 0);
	_submenu->add_radio(new CommandDebug("Radio2"), "Radio2", group);
	_submenu->add_radio(new CommandDebug("Radio3"), "Radio3", group);
	_submenu->add_radio(new CommandDebug("Radio4"), "Radio4", group);

	_menu->add_submenu(_submenu, "Edit");

	//

	_submenu = new GTK::Menu();

	_submenu->add_push(new CommandDebug("About"), "About");

	_menu->add_submenu(_submenu, "Help");
    }

    gtk_widget_show(menu);

    //

    gtk_widget_show(vbox);

    gtk_widget_show(toplevel);

    gtk_main();
}
catch(Exception & exception) {
    cerr << "Died in main with exception" << endl << exception << endl;
}
catch(...) {
    cerr << "Unknown exception in main" << endl;
}
