// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_ALGORITHM_H
#define FOUNDATION_ALGORITHM_H

template<class InputIter, class Pred, class Func>
void for_each_if(InputIter first, InputIter last, Pred p, Func f) {
    for(; first != last; ++first) {
	if(p(*first)) {
	    f(*first);
	}
    }
}

template<class InputIter, class Pred, class Func>
void for_each_if_else(InputIter first, InputIter last, Pred p, Func f1, Func f2) {
    for(; first != last; ++first) {
	if(p(*first)) {
	    f1(*first);
	}
	else {
	    f2(*first);
	}
    }
}

template<class InputIter, class OutputIter, class Pred>
void copy_if(InputIter first, InputIter last, OutputIter out, Pred p) {
    for(; first != last; ++first) {
	if(p(*first)) {
	    *out++ = *first;
	}
    }
}

template<class InputIter, class OutputIter, class Pred, class Operation>
void transform_if(InputIter first, InputIter last, OutputIter out, Pred p, Operation op) {
    for(; first != last; ++first) {
	if(p(*first)) {
	    *out++ = op(*first);
	}
    }
}

template<class InputIter>
InputIter find_nth(InputIter first, InputIter last, int n) {
    for(; first != last && n > 0; ++first, --n) {
    }

    return first;
}

template<class InputIter>
int count(InputIter first, InputIter last) {
    int i = 0;

    while(first != last) {
	++first;
	++i;
    }

    return i;
}

#endif // FOUNDATION_ALGORITHM_H
