// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_COMMAND_CALLBACK_H
#define FOUNDATION_COMMAND_CALLBACK_H

#include "command.h"

#include <string>
using namespace std;

template<class R>
class CommandCallback : public Command {
public:
    CommandCallback(R (*m1)(), R (*m2)()) : Command(), mMethod1(m1), mMethod2(m2) {
    }

    ~CommandCallback() { }

    void execute() {
	(*mMethod1)();
    }

    void unexecute() {
	if(mMethod2) {
	    (*mMethod2)();
	}
    }

private:
    R (*mMethod1)();
    R (*mMethod2)();
};

template<class R>
CommandCallback<R> *
command_callback(R (*m1)(), R (*m2)() = 0) {
    return new CommandCallback<R>(m1, m2);
}

//
//
//

template<class T, class R>
class CommandCallback0 : public Command {
public:
    CommandCallback0(T * o, R (T::*m1)(), R (T::*m2)()) : Command(),
	mObject(o), mMethod1(m1), mMethod2(m2) {
	mObject->ref();
    }

    ~CommandCallback0() {
	mObject->unref();
    }

    void execute() {
	(mObject->*mMethod1)();
    }

    void unexecute() {
	if(mMethod2) {
	    (mObject->*mMethod2)();
	}
    }

private:
    T * mObject;
    R (T::*mMethod1)();
    R (T::*mMethod2)();
};

template<class T, class R>
CommandCallback0<T, R> *
command_callback(T * o, R (T::*m1)(), R (T::*m2)() = 0) {
    return new CommandCallback0<T, R>(o, m1, m2);
}

//
//
//

template<class T1, class T2, class R>
class CommandCallback1 : public Command {
public:
    CommandCallback1(T1 * o, T2 * a, R (T1::*m1)(T2 *), R (T1::*m2)(T2 *)) : Command(),
	mObject(o), mArgument(a), mMethod1(m1), mMethod2(m2) {
	mObject->ref();

	if(mArgument) {
	    mArgument->ref();
	}
    }

    ~CommandCallback1() {
	if(mArgument) {
	    mArgument->unref();
	}

	mObject->unref();
    }

    void execute() {
	(mObject->*mMethod1)(mArgument);
    }

    void unexecute() {
	if(mMethod2) {
	    (mObject->*mMethod2)(mArgument);
	}
    }

private:
    T1 * mObject;
    T2 * mArgument;
    R (T1::*mMethod1)(T2 *);
    R (T1::*mMethod2)(T2 *);
};

template<class T1, class T2, class R>
CommandCallback1<T1, T2, R> *
command_callback(T1 * o, T2 * a, R (T1::*m1)(T2 *), R (T1::*m2)(T2 *) = 0) {
    return new CommandCallback1<T1, T2, R>(o, a, m1, m2);
}

#endif // FOUNDATION_COMMAND_CALLBACK_H
