// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_ITERATOR_H
#define FOUNDATION_ITERATOR_H

template<class T>
class associative_insert_iterator {
public:
    explicit associative_insert_iterator(T & t) : container(t) { }

    associative_insert_iterator<T> & operator = (const typename T::value_type & value) {
	container.insert(value);
	return *this;
    }

    associative_insert_iterator<T> & operator * () { return *this; }
    associative_insert_iterator<T> & operator ++ () { return *this; }
    associative_insert_iterator<T> & operator ++ (int) { return *this; }

private:
    T & container;
};

template<class T>
associative_insert_iterator<T> associative_inserter(T & t) {
    return associative_insert_iterator<T>(t);
}

//

// this ostream_iterator differs from the standard one in that the
// separator text comes before the item

template <class T>
class ostream_iterator2 {
protected:
    ostream * mOst;
    const char * mSeparator;
public:
    typedef output_iterator_tag iterator_category;
    typedef void value_type;
    typedef void difference_type;
    typedef void pointer;
    typedef void reference;

    ostream_iterator2(ostream & s) : mOst(&s), mSeparator(0) { }
    ostream_iterator2(ostream & s, const char * c) : mOst(&s), mSeparator(c) { }

    ostream_iterator2<T>& operator=(const T & value) {
	if(mSeparator) *mOst << mSeparator;
	*mOst << value;
	return *this;
    }

    ostream_iterator2<T>& operator*() { return *this; }
    ostream_iterator2<T>& operator++() { return *this; }
    ostream_iterator2<T>& operator++(int) { return *this; }
};

#endif // FOUNDATION_ITERATOR_H
