// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_MACROS_H
#define FOUNDATION_MACROS_H

#include <cstdlib>
#include <iostream>
using namespace std;

// debug utilities

#if !(defined (STMT_START) && defined (STMT_END))
#  if defined (__GNUC__) && !defined (__STRICT_ANSI__) && !defined (__cplusplus)
#    define STMT_START (void)(
#    define STMT_END )
#  else
#    if (defined (sun) || defined (__sun__))
#      define STMT_START if (1)
#      define STMT_END else (void)0
#    else
#      define STMT_START do
#      define STMT_END while (0)
#    endif
#  endif
#endif

// the pretty function is a bit wide for the screen and causes annoying
// wrapping to occur, so it can be disabled

#if 0
#  define LOCATION __FILE__ << ":" << __LINE__ << ":(" << __PRETTY_FUNCTION____ << ") "
#else
#  define LOCATION __FILE__ << ":" << __LINE__ << ":(" << __FUNCTION__ << ") "
#endif

// errors, warnings, etc

#define MESSAGE(message) cerr << "MESSAGE: " << message << endl

#define WARNING(message) cerr << "WARNING: " << LOCATION << ": " << message << endl

#define ERROR(message) cerr << "ERROR: " << LOCATION << ": " << message << endl

#define FATAL(message) \
    STMT_START { \
        cerr << "FATAL: " << LOCATION << ": " << message << endl; abort(); \
    } STMT_END

// traces

#ifdef DEBUG
    extern int trace_level;
#  define TRACE(level, message) \
    STMT_START { \
        if(level <= trace_level) { \
            cerr << "DEBUG: " << LOCATION << ": " << message << endl; \
        } \
    } STMT_END
#else
#  define TRACE(level, message) \
    STMT_START { \
    } STMT_END
#endif


// pause, waits for the enter key before continuing

#define PAUSE(message) \
    STMT_START { \
        cout << message << endl; \
        char c; cin.get(c); \
    } STMT_END

// not yet implemented

#define NYI() cerr << "NOT YET IMPLEMENTED " << LOCATION << endl

// assertions

#define ASSERT_NOT_REACHED() \
    STMT_START { \
        cerr << "SHOULD NOT BE REACHED: " << LOCATION << endl; \
        abort(); \
    } STMT_END

#define ASSERT(expr) \
    STMT_START { \
        if(!(expr)) { \
            cerr << "ASSERTION FAILED: (" << #expr << ")  " << LOCATION << endl; \
            abort(); \
        } \
    } STMT_END

#define RETURN_IF_FAIL(expr) \
    STMT_START { \
        if(!(expr)) { \
            cerr << "RETURN IF FAIL: (" << #expr << ")  " << LOCATION << endl; \
            return; \
        } \
    } STMT_END

#define RETURN_VAL_IF_FAIL(expr, val) \
    STMT_START { \
        if(!(expr)) { \
            cerr << "RETURN IF FAIL: (" << #expr << ")  " << LOCATION << endl; \
            return val; \
        } \
    } STMT_END

#endif // FOUNDATION_MACROS_H
