// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_OBJECT_H
#define FOUNDATION_OBJECT_H

class ChangeEvent;
class Command;

#ifdef DEBUG
#include <set>
#include <string>
#include <typeinfo>
using namespace std;
#endif // DEBUG

class Object {
public:
#ifdef DEBUG
    void ref() throw();
    void unref();
    static const set<Object *> & get_live_objects() { return mLiveObjects; }
#else
    void ref() throw() { ++mCounter; }
    void unref() { if(--mCounter == 0) delete this; }
#endif

    void show_details();
    static void show_remaining();

protected:
#ifdef DEBUG
    static void watch_type(const type_info & type);

    Object();
    Object(const Object & object);
#else
    Object() : mCounter(1) { }
    Object(const Object & object) : mCounter(1) { }
#endif
    virtual ~Object();		// should be created on the heap

    static void begin_transaction();
    static void end_transaction();
    static void log_command(Command * command);
    static void log_change(ChangeEvent * change_event);

private:
    int mCounter;

#ifdef DEBUG
    static set<Object *> mLiveObjects;
    static set<string> mWatchedObjects;
#endif
};

#endif // FOUNDATION_OBJECT_H
