// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef FOUNDATION_UNDO_MANAGER_H
#define FOUNDATION_UNDO_MANAGER_H

#include "object.h"

#include <list>
using namespace std;

class Command;
class CompositeCommand;
class UndoEvent;
class UndoListener;
class ChangeEvent;
class ChangeListener;
class VetoableChangeListener;

class UndoManager : protected Object {
public:
    static UndoManager * instance();
    static void shutdown();

    void begin_transaction();
    void end_transaction();

    void log_command(Command * command);
    void log_change(ChangeEvent * change_event);

    void undo();
    void redo();

    bool can_undo();
    bool can_redo();

    void clear();
    void clear_undo();
    void clear_redo();

    // 0 means no limit
    void set_undo_limit(unsigned int i);

    void add(UndoListener * undo_listener);
    void remove(UndoListener * undo_listener);

    void add_change_listener(ChangeListener * change_listener);
    void remove_change_listener(ChangeListener * change_listener);

    void add_vetoable_change_listener(VetoableChangeListener * vetoable_change_listener);
    void remove_vetoable_change_listener(VetoableChangeListener * vetoable_change_listener);

protected:

private:
    UndoManager();
    ~UndoManager();

    void flush_changes();

    static const unsigned int DEFAULT_UNDO_LIMIT;

    unsigned int mUndoLimit;
    unsigned int mTransactionDepth;
    bool mTransactionsForbidden;
    bool mUndoingOrRedoing;

    list<CompositeCommand *> mPast;
    list<CompositeCommand *> mFuture;

    list<ChangeEvent *> mChangeEvents;

    list<UndoListener *> mUndoListeners;

    list<ChangeListener *> mChangeListeners;
    list<VetoableChangeListener *> mVetoableChangeListeners;

    static UndoManager * mInstance;
};

#endif // FOUNDATION_UNDO_MANAGER_H
