// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GENERATOR_COLLECTION_TYPE_H
#define GENERATOR_COLLECTION_TYPE_H

#include "foundation/object.h"

#include <string>
using namespace std;

namespace MOF {

    class CollectionType;

} // namespace MOF

namespace Common {

    class CppStream;

} // namespace Common

namespace Generator {

    class CollectionType : public Object {
    public:
	CollectionType(MOF::CollectionType * collection_type, const string & model_name);

	virtual void generate(const string & output_dir, const string & include_dir);

    protected:
	~CollectionType();

	virtual void load_details();

	//
	// header
	//

	void generate_header(Common::CppStream & hpp);
	void generate_header_includes(Common::CppStream & hpp);
	void generate_header_public_members(Common::CppStream & hpp);
	void generate_header_private_members(Common::CppStream & hpp);

	//
	// source
	//

	void generate_source(Common::CppStream & cpp);
	void generate_source_includes(Common::CppStream & cpp);
	void generate_source_statics(Common::CppStream & cpp);
	void generate_source_body(Common::CppStream & cpp);

	//

	MOF::CollectionType * mCollectionType;
	string mModelName;

	struct CollectionTypeDetails {
	    MOF::CollectionType * collection_type;
	    string name;
	    string role;
	    string variable;
	    string qualified_name;
	    string qualified_path;
	};

	CollectionTypeDetails mCollectionTypeDetails;

    private:
	bool mDetailsLoaded;
    };

} // namespace Generator

#endif // GENERATOR_COLLECTION_TYPE_H

