// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GENERATOR_PACKAGE_H
#define GENERATOR_PACKAGE_H

#include "foundation/object.h"

#include <string>
#include <vector>
#include <set>
using namespace std;

namespace Common {

    class DiffStream;

} // namespace Common

namespace MOF {

    class Package;
    class Class;
    class PrimitiveType;
    class StructureType;
    class EnumerationType;
    class CollectionType;
    class AliasType;

} // namespace MOF

namespace Generator {

    class Package : public Object {
    public:
	Package(MOF::Package * package, const string & model_name);

	virtual void generate(const string & output_dir, const string & include_dir, int depth);

    protected:
	~Package();

	virtual void load_details();

	virtual void generate_makefile(Common::DiffStream & mam);
	virtual void generate_cvsignore(Common::DiffStream & ign);

	//
	//
	//

	MOF::Package * mPackage;
	string mModelName;

	struct PackageDetails {
	    MOF::Package * package;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	struct ClassDetails {
	    MOF::Class * klass;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	    bool is_abstract;
	};

	struct PrimitiveTypeDetails {
	    MOF::PrimitiveType * primitive_type;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	struct StructureTypeDetails {
	    MOF::StructureType * structure_type;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	struct EnumerationTypeDetails {
	    MOF::EnumerationType * enumeration_type;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	struct CollectionTypeDetails {
	    MOF::CollectionType * collection_type;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	struct AliasTypeDetails {
	    MOF::AliasType * alias_type;
	    string name;
	    string path;
	    string qualified_name;
	    string qualified_path;
	};

	PackageDetails mPackageDetails;

	vector<PackageDetails> mSubPackageDetails;
	vector<ClassDetails> mClassDetails;
	vector<PrimitiveTypeDetails> mPrimitiveTypeDetails;
	vector<StructureTypeDetails> mStructureTypeDetails;
	vector<EnumerationTypeDetails> mEnumerationTypeDetails;
	vector<CollectionTypeDetails> mCollectionTypeDetails;
	vector<AliasTypeDetails> mAliasTypeDetails;

	set<string> mBuiltSources;
	set<string> mBuiltHeaders;
	set<string> mSubdirectories;

    private:
	bool mDetailsLoaded;
    };

} // namespace Generator

#endif // GENERATOR_PACKAGE_H
