// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GTK_CANVAS_H
#define GTK_CANVAS_H

#include "toolkit/canvas.h"
#include "widget.h"

#include "drawing/rectangle.h"
#include "drawing/point.h"
#include "drawing/vector.h"

#include <stack>
#include <vector>
using namespace std;

namespace GTK {

    class Canvas : public Toolkit::Canvas, public Widget {
    public:
	Canvas(int width, int height);
	~Canvas();

	void refresh();
	void damage(const Drawing::Rectangle & rect);
	void set_cursor(Toolkit::Cursor cursor);

	//

	void set_color(const string & color_string);
	void set_line_width(int width);
	void set_line_style(Drawing::LineStyle line_style);

	void draw_point(const Drawing::Point & point);
	void draw_rectangle(const Drawing::Rectangle & rectangle, bool filled);
	void draw_line(const Drawing::Point & p1, const Drawing::Point & p2);
	void draw_oval(const Drawing::Rectangle & rectangle, bool filled);
	void draw_polygon(const vector<Drawing::Point> & points, bool filled);
	void draw_lines(const vector<Drawing::Point> & points);
	void draw_text(Drawing::Text * text, const Drawing::Point & point);
	void draw_image(Drawing::Image * image, const Drawing::Point & point);

	void push_clip(const Drawing::Rectangle & rectangle);
	void pop_clip();
	const Drawing::Rectangle & get_clip() const;

	const Drawing::Vector & get_size() const;

    protected:

    private:
	bool expose_event(const GdkEventExpose * event);
	bool configure_event(const GdkEventConfigure * event);
	bool motion_notify_event(const GdkEventMotion * event);
	bool enter_notify_event(const GdkEventCrossing * event);
	bool leave_notify_event(const GdkEventCrossing * event);
	bool button_press_event(const GdkEventButton * event);
	bool button_release_event(const GdkEventButton * event);
	bool key_press_event(const GdkEventKey * event);
	bool key_release_event(const GdkEventKey * event);
	bool timeout();

	static gint static_expose_event(GtkWidget * widget, GdkEventExpose * event,
					Canvas * canvas);
	static gint static_configure_event(GtkWidget * widget, GdkEventConfigure * event,
					Canvas * canvas);
	static gint static_motion_notify_event(GtkWidget * widget, GdkEventMotion * event,
                                               Canvas * canvas);
	static gint static_enter_notify_event(GtkWidget * widget, GdkEventCrossing * event,
                                              Canvas * canvas);
	static gint static_leave_notify_event(GtkWidget * widget, GdkEventCrossing * event,
                                              Canvas * canvas);
	static gint static_button_press_event(GtkWidget * widget, GdkEventButton * event,
                                              Canvas * canvas);
	static gint static_button_release_event(GtkWidget * widget, GdkEventButton * event,
						Canvas * canvas);
	static gint static_key_press_event(GtkWidget * widget, GdkEventKey * event,
                                           Canvas * canvas);
	static gint static_key_release_event(GtkWidget * widget, GdkEventKey * event,
					    Canvas * canvas);

	static gboolean static_timeout(gpointer data);

	enum {
	    HID_EXPOSE,
	    HID_CONFIGURE,
	    HID_MOTION_NOTIFY,
	    HID_ENTER_NOTIFY,
	    HID_LEAVE_NOTIFY,
	    HID_BUTTON_PRESS,
	    HID_BUTTON_RELEASE,
	    HID_KEY_PRESS,
	    HID_KEY_RELEASE,
	    HID_LAST
	};

	guint mHandlers[HID_LAST];

	stack<Drawing::Rectangle> mClipStack;
	Drawing::Vector mSize;

	unsigned int mDownButton;
	unsigned int mTimeoutId;

	int mTest;

	GtkWidget * mCanvas;

	vector<GdkPoint> mPointBuffer;

	static const unsigned int SCROLL_TIMEOUT;
	static const int SCROLL_THRESHOLD;
    };

} // namespace GTK

#endif // GTK_CANVAS_H
