// Dachshund,			      -*- mode: C++; c-basic-offset: 4; -*-
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "factory.h"

#include "foundation/macros.h"

namespace GTK {

    Factory::Factory() : Toolkit::Factory() {
    }

    Factory::~Factory() {
    }

    FileSelection *
    Factory::create_file_selection(const string & title, const string & path) const {
	return new FileSelection(title, path);
    }

    Menu *
    Factory::create_menu() const {
	return new Menu(false);
    }

    Palette *
    Factory::create_palette() const {
	return new Palette();
    }

    //------------------------------------------------------------

    Tag *
    Factory::create_tag(Toolkit::Document * document, const char * name) const {
	// Just create a plain tag
	return new Tag(document, name);
    }

    Tag *
    Factory::create_block(Toolkit::Document * document, const char * name, int level) const {
	Tag * tag = new Tag(document, name);
	tag->set_is_block(true);
	tag->set_level(level);
	return tag;
    }
    
    Tag *
    Factory::create_field(Toolkit::Document * document, const char * name, int level) const {
	Tag * tag = new Tag(document, name);
	tag->set_is_field(true);
	tag->set_level(level);
	return tag;
    }

    //------------------------------------------------------------

    Mark *
    Factory::create_mark(const char * name, Toolkit::Mark * location) const {
        return new Mark(name, location);
    }

    Mark *
    Factory::create_block_mark(const char * name, Toolkit::Mark * location,
			       Toolkit::Tag * block, int level, Object * object) const {
	Mark * mark = new Mark(name, location, false);
	mark->set_is_block_mark(true);
	mark->set_associated_block(block);
	mark->set_associated_level(level);
	mark->set_associated_object(object);
	return mark;
    }

    //------------------------------------------------------------

    Canvas *
    Factory::create_canvas(int width, int height) const {
        return new Canvas(width, height);
    }



    //================================================================================

    MessageBox *
    Factory::create_message_box(const string & message,
				Toolkit::MessageBoxKind kind,
                                Toolkit::Widget * parent) const {
        return new MessageBox(message, kind, parent);
    }

    Box *
    Factory::create_box(Toolkit::OrientationKind orientation) const {
        return new Box(orientation);
    }

    //
    //
    //

    Factory2::Factory2() : Drawing::Factory() {
    }

    Factory2::~Factory2() {
    }

    Image *
    Factory2::create_image(const string & file_name) const {
	return new Image(file_name);
    }

    Text *
    Factory2::create_text() const {
	return new Text();
    }

} // namespace GTK
