// Dachshund,			      -*- mode: C++; c-basic-offset: 4; -*-
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GTK_FACTORY_H
#define GTK_FACTORY_H

#include "toolkit/factory.h"
#include "drawing/factory.h"

#include "box.h"
#include "file_selection.h"
#include "menu.h"
#include "palette.h"
#include "tag.h"
#include "mark.h"
#include "canvas.h"
#include "document.h"
#include "message_box.h"

#include "image.h"
#include "text.h"

namespace GTK {

    class Factory : public Toolkit::Factory {
    public:
	Factory();
	~Factory();

	FileSelection * create_file_selection(const string & title, const string & path = "") const;

	Menu * create_menu() const;

	Palette * create_palette() const;

	Tag * create_tag(Toolkit::Document * document, const char * name) const;
	Tag * create_block(Toolkit::Document * document, const char * name, int level) const;
	Tag * create_field(Toolkit::Document * document, const char * name, int level) const;

	// General purpose mark
	Mark * create_mark(const char * name, Toolkit::Mark * location) const;

	// Block mark used to relate document to model
	Mark * create_block_mark(const char * name, Toolkit::Mark * location,
                                 Toolkit::Tag * block, int level, Object * object) const;

	Canvas * create_canvas(int width, int height) const;

	MessageBox * create_message_box(const string & message,
					Toolkit::MessageBoxKind kind,
					Toolkit::Widget * parent = 0) const;

        Box * create_box(Toolkit::OrientationKind orientation) const;

    private:
    };

    class Factory2 : public Drawing::Factory {
    public:
	Factory2();
	~Factory2();

	Image * create_image(const string & file_name) const;
	Text * create_text() const;

    private:
    };

} // namespace GTK

#endif // GTK_FACTORY_H
