// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef GTK_GTK_CANVAS_H
#define GTK_GTK_CANVAS_H

#include <gtk/gtkdrawingarea.h>

#ifdef __cplusplus
extern "C" {
#endif /* __cplusplus */


#define GTK_TYPE_CANVAS            (gtk_canvas_get_type ())
#define GTK_CANVAS(obj)            (GTK_CHECK_CAST ((obj), GTK_TYPE_CANVAS, GtkCanvas))
#define GTK_CANVAS_CLASS(klass)    (GTK_CHECK_CLASS_CAST ((klass), GTK_TYPE_CANVAS, GtkCanvasClass))
#define GTK_IS_CANVAS(obj)         (GTK_CHECK_TYPE ((obj), GTK_TYPE_CANVAS))
#define GTK_IS_CANVAS_CLASS(klass) (GTK_CHECK_CLASS_TYPE ((klass), GTK_TYPE_CANVAS))
#define GTK_CANVAS_GET_CLASS(obj)  (GTK_CHECK_GET_CLASS ((obj), GTK_TYPE_CANVAS, GtkCanvasClass))

typedef struct _GtkCanvas       GtkCanvas;
typedef struct _GtkCanvasClass  GtkCanvasClass;

struct _GtkCanvas {
    GtkDrawingArea drawing_area;

    /* private */

    GdkGC * gc;
    PangoLayout * layout;
    GdkRectangle damage;
    guint idle_id;
    gboolean grabbed;

    GHashTable * color_table;

    gchar * current_color_string;
    GdkLineStyle current_line_style;

    int width, height;
};

struct _GtkCanvasClass {
    GtkDrawingAreaClass parent_class;

    /* virtuals */
};


GtkType gtk_canvas_get_type(void) G_GNUC_CONST;
GtkWidget * gtk_canvas_new(int width, int height);

void gtk_canvas_damage(GtkCanvas * canvas, const GdkRectangle * area);
void gtk_canvas_set_color(GtkCanvas * canvas, const gchar * color_string);
#if 1
void gtk_canvas_set_line_width(GtkCanvas * canvas, int line_width);
#endif
void gtk_canvas_set_line_style(GtkCanvas * canvas, GdkLineStyle line_style);
void gtk_canvas_draw_polygon(GtkCanvas * canvas, const GdkPoint * points, int num, int filled);
void gtk_canvas_draw_lines(GtkCanvas * canvas, const GdkPoint * points, int num);
void gtk_canvas_draw_pixbuf(GtkCanvas * canvas, GdkPixbuf * pixuf, int x, int y);
void gtk_canvas_draw_layout(GtkCanvas * canvas, PangoLayout * layout, int x, int y);
void gtk_canvas_draw_point(GtkCanvas * canvas, int x, int y);
void gtk_canvas_draw_rectangle(GtkCanvas * canvas, const GdkRectangle * rect, int filled);
void gtk_canvas_draw_line(GtkCanvas * canvas, int x0, int y0, int x1, int y1);
void gtk_canvas_draw_oval(GtkCanvas * canvas, int x, int y, int w, int h, int filled);
void gtk_canvas_set_clip(GtkCanvas * canvas, const GdkRectangle * rect);
void gtk_canvas_unset_clip(GtkCanvas * canvas);
void gtk_canvas_set_width(GtkCanvas * canvas, int width);
void gtk_canvas_set_height(GtkCanvas * canvas, int height);
void gtk_canvas_set_dimensions(GtkCanvas * canvas, int width, int height);

#ifdef __cplusplus
}
#endif /* __cplusplus */

#endif /* GTK_GTK_CANVAS_H */
