// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "image.h"

#include "default.xpm"

#include "foundation/macros.h"
#include "foundation/standard_exceptions.h"

#include <gtk/gtkstock.h>
#include <gtk/gtkiconfactory.h>
#include <gtk/gtkwidget.h>

namespace GTK {

    GtkWidget * Image::mWidget = 0;

    map<string, string> Image::mTranslationMap;

    void
    Image::set_widget(GtkWidget * widget) {
	if(!mWidget) {
	    mWidget = widget;

	    // a good opportunity to populate the map

	    //mTranslationMap["palette/cursor.png"] = "gtk-cursor";

	    mTranslationMap["toolbar/copy.png"] = "gtk-copy";
	    mTranslationMap["toolbar/cut.png"] = "gtk-cut";
	    mTranslationMap["toolbar/new.png"] = "gtk-new";
	    mTranslationMap["toolbar/no.png"] = "gtk-no";
	    mTranslationMap["toolbar/open.png"] = "gtk-open";
	    mTranslationMap["toolbar/print.png"] = "gtk-print";
	    mTranslationMap["toolbar/paste.png"] = "gtk-paste";
	    mTranslationMap["toolbar/properties.png"] = "gtk-properties";
	    mTranslationMap["toolbar/quit.png"] = "gtk-quit";
	    mTranslationMap["toolbar/redo.png"] = "gtk-redo";
	    mTranslationMap["toolbar/save.png"] = "gtk-save";
	    mTranslationMap["toolbar/save-as.png"] = "gtk-save-as";
	    mTranslationMap["toolbar/undo.png"] = "gtk-undo";
	    mTranslationMap["toolbar/zoom-100.png"] = "gtk-zoom-100";
	    mTranslationMap["toolbar/zoom-in.png"] = "gtk-zoom-in";
	    mTranslationMap["toolbar/zoom-out.png"] = "gtk-zoom-out";
	}
	else {
	    ASSERT_NOT_REACHED();
	}
    }

    //

    Image::Image(const string & file_name) : Drawing::Image() {
	map<string, string>::iterator iter = mTranslationMap.find(file_name);

	ASSERT(mWidget);		// the widget for creating stock icons against
					// has not been set yet

	try {
	    if(iter != mTranslationMap.end()) {
		try {
		    load_stock_icon(iter->second);
		}
		catch(Exception &) {
		    load_icon_from_file(file_name);
		}
	    }
	    else {
		load_icon_from_file(file_name);
	    }
	}
	catch(Exception &) {
	    load_default_icon();
	}

	ASSERT(mPixbuf);

#if 0
	if(size != UNSCALED) {
	    GtkIconSize icon_size;

	    switch(size) {
	    case SMALL:
		icon_size = GTK_ICON_SIZE_BUTTON;
		break;
	    case MEDIUM:
		icon_size = GTK_ICON_SIZE_SMALL_TOOLBAR;
		break;
	    case LARGE:
		icon_size = GTK_ICON_SIZE_LARGE_TOOLBAR;
		break;
	    default:
		ASSERT_NOT_REACHED();
	    }

	    scale_icon(mPixbuf, icon_size);
	}
#endif
    }

    Image::~Image() {
	if(mPixbuf) {
	    g_object_unref(G_OBJECT(mPixbuf));
	}
    }

    int
    Image::get_w() const {
	return gdk_pixbuf_get_width(mPixbuf);
    }

    int
    Image::get_h() const {
	return gdk_pixbuf_get_height(mPixbuf);
    }

    void
    Image::load_stock_icon(const string & icon_name) {
	GtkIconSet * icon_set = gtk_icon_factory_lookup_default(icon_name.c_str());

	GtkIconSize * sizes = 0;
	int n_sizes = 0;
	GtkIconSize size = GTK_ICON_SIZE_INVALID;
	const GtkIconSize DESIRED_SIZE = GTK_ICON_SIZE_LARGE_TOOLBAR;

#if 0
	GTK_ICON_SIZE_INVALID,
	GTK_ICON_SIZE_MENU,
	GTK_ICON_SIZE_SMALL_TOOLBAR,
	GTK_ICON_SIZE_LARGE_TOOLBAR,
	GTK_ICON_SIZE_BUTTON,
	GTK_ICON_SIZE_DND,
	GTK_ICON_SIZE_DIALOG
#endif

	gtk_icon_set_get_sizes(icon_set, &sizes, &n_sizes);

	ASSERT(n_sizes > 0);

	for(int i = 0; i < n_sizes; ++i) {
	    size = sizes[i];

	    if(size == DESIRED_SIZE) {
		break;
	    }
	}

	g_free(sizes);

	mPixbuf = gtk_widget_render_icon(mWidget, icon_name.c_str(), size, 0);

	if(!mPixbuf) {
	    throw OddityException("The pixbuf didn't render...");
	}

	if(size != DESIRED_SIZE) {
	    scale(DESIRED_SIZE);
	}

	ASSERT(mPixbuf);
    }

    void
    Image::load_icon_from_file(const string & file_name) {
	string full_path = get_path_to_image(file_name);
	mPixbuf = gdk_pixbuf_new_from_file(full_path.c_str(), 0);

	ASSERT(mPixbuf);
    }

    void
    Image::load_default_icon() {
	mPixbuf = gdk_pixbuf_new_from_xpm_data(default_xpm);

	ASSERT(mPixbuf);
    }

    void
    Image::scale(GtkIconSize size) {
	gint w, h;

	gtk_icon_size_lookup(size, &w, &h);

	GdkPixbuf * pixbuf = gdk_pixbuf_scale_simple(mPixbuf, w, h, GDK_INTERP_BILINEAR);

	g_object_unref(G_OBJECT(mPixbuf));

	mPixbuf = pixbuf;
    }

} // namespace GTK
