// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif // HAVE_CONFIG_H

#include "log.h"

#include "foundation/macros.h"

#include <gtk/gtkscrolledwindow.h>
#include <gtk/gtktextview.h>

namespace GTK {

    Log::Log() : Toolkit::Log(), Widget() {
	mBaseWidget = gtk_scrolled_window_new(0, 0);
	gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(mBaseWidget),
				       GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	mTextBuffer = gtk_text_buffer_new(0);
	mTextView = gtk_text_view_new_with_buffer(mTextBuffer);

	gtk_text_view_set_editable(GTK_TEXT_VIEW(mTextView), FALSE);

	gtk_container_add(GTK_CONTAINER(mBaseWidget), mTextView);

	gtk_widget_show(mTextView);

	post_init();
    }

    Log::~Log() {
    }

    void
    Log::log(const string & message, Toolkit::LogLevel level) {
	// TODO use the log level to determine the color of the text

	//switch(level) {


	GtkTextIter iter;
	gtk_text_buffer_get_end_iter(mTextBuffer, &iter);

	gtk_text_buffer_insert(mTextBuffer, &iter, message.c_str(), message.size());
	gtk_text_buffer_insert(mTextBuffer, &iter, "\n", 1);

	// scroll to the end

	GtkAdjustment * vadjustment =
	    gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(mBaseWidget));

	RETURN_IF_FAIL(vadjustment);

	double old_value = gtk_adjustment_get_value(vadjustment);
	double new_value = vadjustment->upper;

	if(new_value != old_value) {
	    gtk_adjustment_set_value(vadjustment, new_value);
	}
    }

    void
    Log::clear() {
	GtkTextIter start;
	GtkTextIter end;

	gtk_text_buffer_get_start_iter(mTextBuffer, &start);
	gtk_text_buffer_get_end_iter(mTextBuffer, &end);

	gtk_text_buffer_delete(mTextBuffer, &start, &end);
    }

} // namespace GTK
