// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h" 

#endif // HAVE_CONFIG_H

#include "mark.h"
#include "tag.h"
#include "document.h"

#include "foundation/macros.h"

#include <gtk/gtktextmark.h>
#include <gtk/gtkenums.h>

#include <stdio.h>


namespace GTK {

  static void destroy_notify(void * data) {
    // Unref the data when the owning object is destroyed
    Object * obj = (Object *) data;
    obj->unref();
  }


  Mark::Mark(const char * name, GtkTextMark * gtkmark) 
    : Toolkit::Mark(name), mGtkMark(gtkmark) {
    //TRACE(2, "Create mark " << name);
  }


  Mark::Mark(const char * name, Toolkit::Mark * toolkit_location, bool left_gravity) 
    : Toolkit::Mark(name) {
    //TRACE(2, "Create mark " << name);

    GtkTextMark * location = static_cast<GTK::Mark *>(toolkit_location)->get_gtkmark();
    assert(location);
    GtkTextIter iter;
    GtkTextBuffer * buffer = gtk_text_mark_get_buffer(location);
    gtk_text_buffer_get_iter_at_mark(buffer, & iter, location);
    mGtkMark = gtk_text_buffer_create_mark(buffer, name, & iter, left_gravity);

    // DEBUG feature
    gtk_text_mark_set_visible(mGtkMark, 1);

    // Set data in the Mark to allow relating back to this Mark
    this->ref();
    g_object_set_data_full(G_OBJECT(mGtkMark), "this", dynamic_cast<Toolkit::Mark *>(this), 
			   & destroy_notify);
  }


  Mark::~Mark() {
    //FIXUP
    //TRACE(2, "");
    //if(mGtkMark) {
    //  g_object_unref(G_OBJECT(mGtkMark));
    //}
  }


  void
  Mark::relocate_to_mark(Toolkit::Mark * toolkit_location) {
    GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mGtkMark);
    GtkTextMark * location = static_cast<GTK::Mark *>(toolkit_location)->get_gtkmark();
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_mark(buffer, & iter, location);
    gtk_text_buffer_move_mark(buffer, mGtkMark, & iter);
  }
  
  void
  Mark::relocate_to_tag_end(Toolkit::Tag * toolkit_tag) {
    GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mGtkMark);
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_mark(buffer, & iter, mGtkMark);
    GtkTextTag * tag = static_cast<GTK::Tag *>(toolkit_tag)->get_gtktag();
    assert(gtk_text_iter_forward_to_tag_toggle(& iter, tag));
    assert(gtk_text_iter_ends_tag(& iter, tag));
    gtk_text_buffer_move_mark(buffer, mGtkMark, & iter);
  }

  bool
  Mark::at_tag_start(Toolkit::Tag * toolkit_tag) {
    GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mGtkMark);
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_mark(buffer, & iter, mGtkMark);
    GtkTextTag * tag = static_cast<GTK::Tag *>(toolkit_tag)->get_gtktag();
    return gtk_text_iter_begins_tag(& iter, tag);
  }

  bool
  Mark::at_tag_end(Toolkit::Tag * toolkit_tag) {
    GtkTextBuffer * buffer = gtk_text_mark_get_buffer(mGtkMark);
    GtkTextIter iter;
    gtk_text_buffer_get_iter_at_mark(buffer, & iter, mGtkMark);
    GtkTextTag * tag = static_cast<GTK::Tag *>(toolkit_tag)->get_gtktag();
    return gtk_text_iter_ends_tag(& iter, tag);
  }

} // namespace GTK
