// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "message_box.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <gtk/gtkmessagedialog.h>
#include <gtk/gtkwindow.h>

#include <list>
using namespace std;

namespace GTK {

    MessageBox::MessageBox(const string & message,
			   Toolkit::MessageBoxKind kind,
			   Toolkit::Widget * parent) : Toolkit::MessageBox(), Widget() {
	GtkWindow * window = 0;

	if(parent) {
	    GtkWidget * parent_ = known_cast<GTK::Widget *>(parent)->get_base_widget();
	    window = GTK_WINDOW(gtk_widget_get_toplevel(parent_));
	}

	GtkMessageType message_type;
	GtkButtonsType buttons;

	if(kind == Toolkit::MBK_QUESTION) {
	    message_type = GTK_MESSAGE_QUESTION;
	    buttons = GTK_BUTTONS_YES_NO;
	}
	else if(kind == Toolkit::MBK_INFORMATION) {
	    message_type = GTK_MESSAGE_INFO;
	    buttons = GTK_BUTTONS_OK;
	}
	else if(kind == Toolkit::MBK_WARNING) {
	    message_type = GTK_MESSAGE_WARNING;
	    buttons = GTK_BUTTONS_OK;
	}
	else if(kind == Toolkit::MBK_ERROR) {
	    message_type = GTK_MESSAGE_ERROR;
	    buttons = GTK_BUTTONS_OK;
	}
	else {
	    ASSERT_NOT_REACHED();
	}

	mBaseWidget = gtk_message_dialog_new(window,
					     GTK_DIALOG_MODAL,
					     message_type,
					     buttons,
					     message.c_str());

	post_init();
    }

    MessageBox::~MessageBox() {
    }

    Toolkit::ResponseKind
    MessageBox::go() {
	gint response = gtk_dialog_run(GTK_DIALOG(mBaseWidget));

	if(response == GTK_RESPONSE_YES) {
	    return Toolkit::RK_YES;
	}
	else if(response == GTK_RESPONSE_NO) {
	    return Toolkit::RK_NO;
	}
	else if(response == GTK_RESPONSE_OK) {
	    return Toolkit::RK_OK;
	}
	else if(response == GTK_RESPONSE_CANCEL) {
	    return Toolkit::RK_CANCEL;
	}
	else {
	    ASSERT_NOT_REACHED();
	}
    }

} // namespace GTK

