// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "text.h"

#include "foundation/macros.h"

namespace GTK {

    GtkWidget * Text::mWidget = 0;

    void
    Text::set_widget(GtkWidget * widget) {
	if(!mWidget) {
	    mWidget = widget;
	}
	else {
	    ASSERT_NOT_REACHED();
	}
    }

    Text::Text() : Drawing::Text() {
	ASSERT(mWidget);		// the widget for creating layouts against
	// has not been set yet


	mLayout = gtk_widget_create_pango_layout(mWidget, 0);
    }

    Text::~Text() {
	g_object_unref(G_OBJECT(mLayout));
    }

    void
    Text::set_text(const string & text) {
	pango_layout_set_text(mLayout, text.c_str(), text.size());
    }

    void
    Text::set_wrap_mode(WrapMode wrap_mode) {
	PangoWrapMode pango_wrap_mode;

	switch(wrap_mode) {
	case WM_WORD:
	    pango_wrap_mode = PANGO_WRAP_WORD;
	    break;
	case WM_CHAR:
	    pango_wrap_mode = PANGO_WRAP_CHAR;
	    break;
	default:
	    ASSERT_NOT_REACHED();
	}

	pango_layout_set_wrap(mLayout, pango_wrap_mode);
    }

    void
    Text::set_alignment(Alignment alignment) {
	PangoAlignment pango_alignment;

	switch(alignment) {
	case A_LEFT:
	    pango_alignment = PANGO_ALIGN_LEFT;
	    break;
	case A_CENTER:
	    pango_alignment = PANGO_ALIGN_CENTER;
	    break;
	case A_RIGHT:
	    pango_alignment = PANGO_ALIGN_RIGHT;
	    break;
	default:
	    ASSERT_NOT_REACHED();
	}

	pango_layout_set_alignment(mLayout, pango_alignment);
    }

    int
    Text::set_width(int width) {
	// calling pango_layout_set_width with width == -1 causes
	// the width to be unlimitd

	pango_layout_set_width(mLayout, (width - 1) * PANGO_SCALE);

	PangoRectangle ink_rect, logical_rect;
	pango_layout_get_extents(mLayout, &ink_rect, &logical_rect);

	int height = (logical_rect.y + logical_rect.height) / PANGO_SCALE + 1;

	return height;
    }

    void
    Text::set_font(const string & font_name) {
	string font_string = "Sans 12";

	PangoFontDescription * font_desc = pango_font_description_from_string(font_string.c_str());

	pango_layout_set_font_description(mLayout, font_desc);
    }

} // namespace GTK
