// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "widget.h"

#include "foundation/macros.h"

#include "toolkit/mouse_event.h"

namespace {

    void
    destroy_notify(void * vp) {
        // Destroy callback for data attached to widget
        if (vp) {
            Object * obj = static_cast<Object *>(vp);
            obj->unref();
        }
    }

} // namespace {anonymous}

namespace GTK {

    Widget::Widget() : Toolkit::Widget(),
	mBaseWidget(0),
	mDestroyHandler(0) {
    }

    Widget::~Widget() {
	if(mBaseWidget) {
	    g_signal_handler_disconnect(G_OBJECT(mBaseWidget), mDestroyHandler);

	    // FIXME I don't know what I'm doing here...

	    if(GTK_OBJECT_FLOATING(GTK_OBJECT(mBaseWidget))) {
		// if the object is still floating then we destroy it
		//MESSAGE("The base widget is still floating, so I will destroy it");
		gtk_object_destroy(GTK_OBJECT(mBaseWidget));
	    }
	    else {
		//MESSAGE("The base widget is around, but not floating, destroying it anyway");
		gtk_object_destroy(GTK_OBJECT(mBaseWidget));
	    }
	}
	else {
	    MESSAGE("The base widget is gone");
	}
    }

    void
    Widget::show() {
	ASSERT(mBaseWidget);

	gtk_widget_show(mBaseWidget);
    }

    void
    Widget::hide() {
	ASSERT(mBaseWidget);

	gtk_widget_hide(mBaseWidget);
    }


    void
    Widget::attach_object(Object * object) {
        g_object_set_data_full(G_OBJECT(mBaseWidget), "object", object, & ::destroy_notify);
    }

    Object *
    Widget::get_object() {
        void * vp = g_object_get_data(G_OBJECT(mBaseWidget), "object");
        return static_cast<Object *>(vp);
    }

    void
    Widget::post_init() {
	ASSERT(mBaseWidget);

	//MESSAGE("Post init called for: " << static_cast<void *>(mBaseWidget));

	mDestroyHandler = g_signal_connect(G_OBJECT(mBaseWidget), "destroy",
					   G_CALLBACK(&Widget::static_destroy_callback), this);
    }

    int
    Widget::translate_modifiers(int i) {
	int r = Toolkit::MT_NONE;

	if(i & GDK_SHIFT_MASK) r |= Toolkit::MT_SHIFT;
	if(i & GDK_CONTROL_MASK) r |= Toolkit::MT_CONTROL;
	if(i & GDK_BUTTON1_MASK) r |= Toolkit::MT_BUTTON1;
	if(i & GDK_BUTTON2_MASK) r |= Toolkit::MT_BUTTON2;
	if(i & GDK_BUTTON3_MASK) r |= Toolkit::MT_BUTTON3;

	return r;
    }

    int
    Widget::translate_presses(int i) {
	if(i == GDK_BUTTON_PRESS) {
	    return 1;
	}
	else if(i == GDK_2BUTTON_PRESS) {
	    return 2;
	}
	else if(i == GDK_3BUTTON_PRESS) {
	    return 3;
	}
	else {
	    return 0;
	}
    }

    void
    Widget::static_destroy_callback(GtkWidget * widget, Widget * _widget) {
	//MESSAGE("Widget::static_destroy_callback");
	_widget->destroy_callback(widget);
    }

    void
    Widget::destroy_callback(GtkWidget * widget) {
	//MESSAGE("Widget::destroy_callback");

	ASSERT(mBaseWidget == widget);

	//MESSAGE("Base widget was destroyed");

	mBaseWidget = 0;
	mDestroyHandler = 0;
    }

} // namespace GTK
