// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_ASSOCIATION_END_H
#define MOF_ASSOCIATION_END_H

#include "typed_element.h"

#include "multiplicity_type.h"
#include "aggregation_kind.h"
#include "boolean.h"

#include <iosfwd>
using namespace std;

namespace MOF {

    using namespace Common;

    class Model;
    class AssociationEnd;
    class Reference;

    //

    class AssociationEnd : public TypedElement {
    public:
	AssociationEnd();
	~AssociationEnd();

	const MultiplicityType * get_multiplicity() const { return mMultiplicity; }
	void set_multiplicity(const MultiplicityType * multiplicity);

	const AggregationKind * get_aggregation() const { return mAggregation; }
	void set_aggregation(const AggregationKind * aggregation);

	const Boolean * is_navigable() const { return mNavigable; }
	void set_navigable(const Boolean * navigable);

	const Boolean * is_changeable() const { return mChangeable; }
	void set_changeable(const Boolean * changeable);

	AssociationEnd * get_other_end();
	void set_other_end(AssociationEnd * other_end);

	// internal

	void internal_add_referent(Reference * referent) throw();
	void internal_remove_referent(Reference * referent) throw();

	void internal_add_referrer(Reference * referrer) throw();
	void internal_remove_referrer(Reference * referrer) throw();

    protected:
	// reflective

	void ref_delete_impl(set<string> & visited);

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();

	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;
	const string & ref_get_type() const;

    private:
	MultiplicityType * mMultiplicity;
	AggregationKind * mAggregation;
	Boolean * mNavigable;
	Boolean * mChangeable;

	set<Reference *> mReferents;
	set<Reference *> mReferrers;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_ASSOCIATION_END_H
