// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "attribute.h"

#include "model.h"

#include "foundation/commands.h"
#include "reflective/changes.h"

namespace MOF {

    const string Attribute::TYPE = "Attribute";

    //

    Attribute::Attribute() : StructuralFeature(),
        mDerived(0) {
    }

    Attribute::~Attribute() {
    }

    void
    Attribute::set_derived(const Boolean * derived) {
	begin_transaction();

	//

	Boolean * old_value = mDerived ? new Boolean(*mDerived) : 0;
	Boolean * new_value = derived ? new Boolean(*derived) : 0;

	//

	Command * command = command_flip_argument(this, &Attribute::set_derived,
						  new_value, old_value);
	log_command(command);
	command->unref();

	// make the changes

	if(mDerived) {
	    mDerived->unref();
	}

	mDerived = new_value ? new Boolean(*new_value) : 0;

	// changes made

	ChangeEvent * change = change_attribute_changed(this, "derived", new_value, old_value);
	log_change(change);
	change->unref();

	//

	if(new_value) {
	    new_value->unref();
	}

	if(old_value) {
	    old_value->unref();
	}

	//

	end_transaction();
    }

    bool
    Attribute::ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    return class_name == TYPE || StructuralFeature::ref_is_a_type_of(class_name);
	}
	else {
	    return false;
	}
    }

    const string &
    Attribute::ref_get_type() const {
	return TYPE;
    }

    void
    Attribute::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_attribute(this);

	    StructuralFeature::ref_implant_impl(visited);
	}
    }

    void
    Attribute::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_attribute(this);

	    StructuralFeature::ref_uproot_impl(visited);
	}
    }

} // namespace MOF
