// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "behavioral_feature.h"

#include "model.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

namespace MOF {

    const string BehavioralFeature::TYPE = "BehavioralFeature";

    //

    BehavioralFeature::BehavioralFeature() : Feature(), Namespace() {
    }

    BehavioralFeature::~BehavioralFeature() {
    }

    void
    BehavioralFeature::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    Namespace::ref_delete_impl(visited);
	    Feature::ref_delete_impl(visited);
	}
    }

    bool
    BehavioralFeature::ref_is_a_type_of_impl(set<string> & visited,
						 const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    return class_name == TYPE ||
		Namespace::ref_is_a_type_of(class_name) ||
		Feature::ref_is_a_type_of(class_name);
	}
	else {
	    return false;
	}
    }

    const string &
    BehavioralFeature::ref_get_type() const {
	return TYPE;
    }

    void
    BehavioralFeature::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_behavioral_feature(this);

	    Feature::ref_implant_impl(visited);
	    Namespace::ref_implant_impl(visited);
	}
    }

    void
    BehavioralFeature::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_behavioral_feature(this);

	    Feature::ref_uproot_impl(visited);
	    Namespace::ref_uproot_impl(visited);
	}
    }

} // namespace MOF
