// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_GENERALIZABLE_ELEMENT_H
#define MOF_GENERALIZABLE_ELEMENT_H

#include "namespace.h"

#include "boolean.h"
#include "visibility_kind.h"

#include <list>
#include <set>
using namespace std;

namespace MOF {

    using namespace Common;

    class Model;

    //

    class GeneralizableElement : public Namespace {
    public:
	const list<GeneralizableElement *> & get_supertypes() { return mSupertypes; }
	void add_supertype(GeneralizableElement * supertype, int index = -1);
	void remove_supertype(GeneralizableElement * supertype);
	void set_supertypes(const list<GeneralizableElement *> & supertypes);
	void modify_supertype(GeneralizableElement * old_supertype,
			      GeneralizableElement * new_supertype);

#if 0
	list<GeneralizableElement *> get_all_supertypes();
#endif

	const VisibilityKind * get_visibility() const { return mVisibility; }
	void set_visibility(const VisibilityKind * visibility);

	const Boolean * is_abstract() const { return mAbstract; }
	void set_abstract(const Boolean * abstract);

	const Boolean * is_root() const { return mRoot; }
	void set_root(const Boolean * root);

	const Boolean * is_leaf() const { return mLeaf; }
	void set_leaf(const Boolean * leaf);

#if 0
	ModelElement * lookup_element_extended(const string & name);
	set<ModelElement *> lookup_element_by_type_extended(const type_info & type,
							    bool include_subtypes);
#endif

	// internal

	void internal_add_subtype(GeneralizableElement * subtype) throw();
	void internal_remove_subtype(GeneralizableElement * subtype) throw();

    protected:
	GeneralizableElement();
	~GeneralizableElement();

	// internal

	// reflective

	void ref_delete_impl(set<string> & visited);

	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;
	const string & ref_get_type() const;

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();

    private:
	list<GeneralizableElement *> mSupertypes;
	set<GeneralizableElement *> mSubtypes;

	VisibilityKind * mVisibility;
	Boolean * mAbstract;
	Boolean * mRoot;
	Boolean * mLeaf;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_GENERALIZABLE_ELEMENT_H
