// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "model.h"

#include "association_end.h"
#include "association.h"
#include "attribute.h"
#include "behavioral_feature.h"
#include "class.h"
#include "classifier.h"
#include "collection_type.h"
#include "constant.h"
#include "constraint.h"
#include "data_type.h"
#include "exception.h"
#include "feature.h"
#include "generalizable_element.h"
#include "import.h"
#include "model_element.h"
#include "namespace.h"
#include "operation.h"
#include "package.h"
#include "parameter.h"
#include "reference.h"
#include "structural_feature.h"
#include "tag.h"
#include "alias_type.h"
#include "typed_element.h"
#include "primitive_type.h"
#include "structure_type.h"
#include "structure_field.h"
#include "enumeration_type.h"

#include "foundation/macros.h"

namespace MOF {

    Model::Model() : REF::Context() {
    }

    Model::~Model() {
    }

    REF::Class *
    Model::ref_create_class(const string & class_name) {
	if(class_name == "association_end") {
	    return new AssociationEnd();
	}
	else if(class_name == "association") {
	    return new Association();
	}
	else if(class_name == "attribute") {
	    return new Attribute();
	}
	else if(class_name == "collection_type") {
	    return new CollectionType();
	}
	else if(class_name == "class") {
	    return new Class();
	}
	else if(class_name == "constant") {
	    return new Constant();
	}
	else if(class_name == "constraint") {
	    return new Constraint();
	}
	else if(class_name == "data_type") {
	    return new DataType();
	}
	else if(class_name == "exception") {
	    return new Exception();
	}
	else if(class_name == "import") {
	    return new Import();
	}
	else if(class_name == "operation") {
	    return new Operation();
	}
	else if(class_name == "package") {
	    return new Package();
	}
	else if(class_name == "parameter") {
	    return new Parameter();
	}
	else if(class_name == "reference") {
	    return new Reference();
	}
	else if(class_name == "tag") {
	    return new Tag();
	}
	else if(class_name == "alias_type") {
	    return new AliasType();
	}
	else if(class_name == "primitive_type") {
	    return new PrimitiveType();
	}
	else if(class_name == "enumeratinon_type") {
	    return new EnumerationType();
	}
	else if(class_name == "structure_type") {
	    return new StructureType();
	}
	else if(class_name == "structure_field") {
	    return new StructureField();
	}
	else {
	    ERROR("No such class_name: " << class_name);
	    throw;	// FIXME
	}
    }

    REF::Attribute *
    Model::ref_create_attribute(const string & attribute_name) {
	if(attribute_name == "name") {
	    return new Name();
	}

	NYI();

	throw;		// FIXME
    }

    void
    Model::internal_add_model_element(ModelElement * element) {
	mAllOfTypeModelElement.insert(element);
    }

    void
    Model::internal_add_behavioral_feature(BehavioralFeature * element) {
	mAllOfTypeBehavioralFeature.insert(element);
    }

    void
    Model::internal_add_classifier(Classifier * element) {
	mAllOfTypeClassifier.insert(element);
    }

    void
    Model::internal_add_class(Class * element) {
	mAllOfTypeClass.insert(element);
    }

    void
    Model::internal_add_feature(Feature * element) {
	mAllOfTypeFeature.insert(element);
    }

    void
    Model::internal_add_typed_element(TypedElement * element) {
	mAllOfTypeTypedElement.insert(element);
    }

    void
    Model::internal_add_association_end(AssociationEnd * element) {
	mAllOfTypeAssociationEnd.insert(element);
    }

    void
    Model::internal_add_association(Association * element) {
	mAllOfTypeAssociation.insert(element);
    }

    void
    Model::internal_add_generalizable_element(GeneralizableElement * element) {
	mAllOfTypeGeneralizableElement.insert(element);
    }

    void
    Model::internal_add_namespace(Namespace * element) {
	mAllOfTypeNamespace.insert(element);
    }

    void
    Model::internal_add_structural_feature(StructuralFeature * element) {
	mAllOfTypeStructuralFeature.insert(element);
    }

    void
    Model::internal_add_attribute(Attribute * element) {
	mAllOfTypeAttribute.insert(element);
    }

    void
    Model::internal_add_constant(Constant * element) {
	mAllOfTypeConstant.insert(element);
    }

    void
    Model::internal_add_constraint(Constraint * element) {
	mAllOfTypeConstraint.insert(element);
    }

    void
    Model::internal_add_data_type(DataType * element) {
	mAllOfTypeDataType.insert(element);
    }

    void
    Model::internal_add_exception(Exception * element) {
	mAllOfTypeException.insert(element);
    }

    void
    Model::internal_add_import(Import * element) {
	mAllOfTypeImport.insert(element);
    }

    void
    Model::internal_add_operation(Operation * element) {
	mAllOfTypeOperation.insert(element);
    }

    void
    Model::internal_add_package(Package * element) {
	mAllOfTypePackage.insert(element);
    }

    void
    Model::internal_add_parameter(Parameter * element) {
	mAllOfTypeParameter.insert(element);
    }

    void
    Model::internal_add_reference(Reference * element) {
	mAllOfTypeReference.insert(element);
    }

    void
    Model::internal_add_tag(Tag * element) {
	mAllOfTypeTag.insert(element);
    }

    void
    Model::internal_add_alias_type(AliasType * element) {
	mAllOfTypeAliasType.insert(element);
    }

    void
    Model::internal_add_primitive_type(PrimitiveType * element) {
	mAllOfTypePrimitiveType.insert(element);
    }

    void
    Model::internal_add_enumeration_type(EnumerationType * element) {
	mAllOfTypeEnumerationType.insert(element);
    }

    void
    Model::internal_add_structure_type(StructureType * element) {
	mAllOfTypeStructureType.insert(element);
    }

    void
    Model::internal_add_structure_field(StructureField * element) {
	mAllOfTypeStructureField.insert(element);
    }

    void
    Model::internal_add_collection_type(CollectionType * element) {
	mAllOfTypeCollectionType.insert(element);
    }

    //
    //
    //

    void
    Model::internal_remove_model_element(ModelElement * element) {
	set<ModelElement *>::iterator iter = mAllOfTypeModelElement.find(element);

	ASSERT(iter != mAllOfTypeModelElement.end());

	mAllOfTypeModelElement.erase(iter);
    }

    void
    Model::internal_remove_behavioral_feature(BehavioralFeature * element) {
	set<BehavioralFeature *>::iterator iter = mAllOfTypeBehavioralFeature.find(element);

	ASSERT(iter != mAllOfTypeBehavioralFeature.end());

	mAllOfTypeBehavioralFeature.erase(iter);
    }

    void
    Model::internal_remove_classifier(Classifier * element) {
	set<Classifier *>::iterator iter = mAllOfTypeClassifier.find(element);

	ASSERT(iter != mAllOfTypeClassifier.end());

	mAllOfTypeClassifier.erase(iter);
    }

    void
    Model::internal_remove_class(Class * element) {
	set<Class *>::iterator iter = mAllOfTypeClass.find(element);

	ASSERT(iter != mAllOfTypeClass.end());

	mAllOfTypeClass.erase(iter);
    }

    void
    Model::internal_remove_feature(Feature * element) {
	set<Feature *>::iterator iter = mAllOfTypeFeature.find(element);

	ASSERT(iter != mAllOfTypeFeature.end());

	mAllOfTypeFeature.erase(iter);
    }

    void
    Model::internal_remove_typed_element(TypedElement * element) {
	/*
	set<TypedElement *>::iterator iter = mAllOfTypeTypedElement.find(element);

	ASSERT(iter != mAllOfTypeTypedElement.end());

	mAllOfTypeTypedElement.erase(iter);
	*/
    }

    void
    Model::internal_remove_association_end(AssociationEnd * element) {
	set<AssociationEnd *>::iterator iter = mAllOfTypeAssociationEnd.find(element);

	ASSERT(iter != mAllOfTypeAssociationEnd.end());

	mAllOfTypeAssociationEnd.erase(iter);
    }

    void
    Model::internal_remove_association(Association * element) {
	set<Association *>::iterator iter = mAllOfTypeAssociation.find(element);

	ASSERT(iter != mAllOfTypeAssociation.end());

	mAllOfTypeAssociation.erase(iter);
    }

    void
    Model::internal_remove_generalizable_element(GeneralizableElement * element) {
	set<GeneralizableElement *>::iterator iter = mAllOfTypeGeneralizableElement.find(element);

	ASSERT(iter != mAllOfTypeGeneralizableElement.end());

	mAllOfTypeGeneralizableElement.erase(iter);
    }

    void
    Model::internal_remove_namespace(Namespace * element) {
	set<Namespace *>::iterator iter = mAllOfTypeNamespace.find(element);

	ASSERT(iter != mAllOfTypeNamespace.end());

	mAllOfTypeNamespace.erase(iter);
    }

    void
    Model::internal_remove_structural_feature(StructuralFeature * element) {
	set<StructuralFeature *>::iterator iter = mAllOfTypeStructuralFeature.find(element);

	ASSERT(iter != mAllOfTypeStructuralFeature.end());

	mAllOfTypeStructuralFeature.erase(iter);
    }

    void
    Model::internal_remove_attribute(Attribute * element) {
	set<Attribute *>::iterator iter = mAllOfTypeAttribute.find(element);

	ASSERT(iter != mAllOfTypeAttribute.end());

	mAllOfTypeAttribute.erase(iter);
    }

    void
    Model::internal_remove_constant(Constant * element) {
	set<Constant *>::iterator iter = mAllOfTypeConstant.find(element);

	ASSERT(iter != mAllOfTypeConstant.end());

	mAllOfTypeConstant.erase(iter);
    }

    void
    Model::internal_remove_constraint(Constraint * element) {
	set<Constraint *>::iterator iter = mAllOfTypeConstraint.find(element);

	ASSERT(iter != mAllOfTypeConstraint.end());

	mAllOfTypeConstraint.erase(iter);
    }

    void
    Model::internal_remove_data_type(DataType * element) {
	set<DataType *>::iterator iter = mAllOfTypeDataType.find(element);

	ASSERT(iter != mAllOfTypeDataType.end());

	mAllOfTypeDataType.erase(iter);
    }

    void
    Model::internal_remove_exception(Exception * element) {
	set<Exception *>::iterator iter = mAllOfTypeException.find(element);

	ASSERT(iter != mAllOfTypeException.end());

	mAllOfTypeException.erase(iter);
    }

    void
    Model::internal_remove_import(Import * element) {
	set<Import *>::iterator iter = mAllOfTypeImport.find(element);

	ASSERT(iter != mAllOfTypeImport.end());

	mAllOfTypeImport.erase(iter);
    }

    void
    Model::internal_remove_operation(Operation * element) {
	set<Operation *>::iterator iter = mAllOfTypeOperation.find(element);

	ASSERT(iter != mAllOfTypeOperation.end());

	mAllOfTypeOperation.erase(iter);
    }

    void
    Model::internal_remove_package(Package * element) {
	set<Package *>::iterator iter = mAllOfTypePackage.find(element);

	ASSERT(iter != mAllOfTypePackage.end());

	mAllOfTypePackage.erase(iter);
    }

    void
    Model::internal_remove_parameter(Parameter * element) {
	set<Parameter *>::iterator iter = mAllOfTypeParameter.find(element);

	ASSERT(iter != mAllOfTypeParameter.end());

	mAllOfTypeParameter.erase(iter);
    }

    void
    Model::internal_remove_reference(Reference * element) {
	set<Reference *>::iterator iter = mAllOfTypeReference.find(element);

	ASSERT(iter != mAllOfTypeReference.end());

	mAllOfTypeReference.erase(iter);
    }

    void
    Model::internal_remove_tag(Tag * element) {
	set<Tag *>::iterator iter = mAllOfTypeTag.find(element);

	ASSERT(iter != mAllOfTypeTag.end());

	mAllOfTypeTag.erase(iter);
    }

    void
    Model::internal_remove_alias_type(AliasType * element) {
	set<AliasType *>::iterator iter = mAllOfTypeAliasType.find(element);

	ASSERT(iter != mAllOfTypeAliasType.end());

	mAllOfTypeAliasType.erase(iter);
    }

    void
    Model::internal_remove_primitive_type(PrimitiveType * element) {
	set<PrimitiveType *>::iterator iter = mAllOfTypePrimitiveType.find(element);

	ASSERT(iter != mAllOfTypePrimitiveType.end());

	mAllOfTypePrimitiveType.erase(iter);
    }

    void
    Model::internal_remove_structure_type(StructureType * element) {
	set<StructureType *>::iterator iter = mAllOfTypeStructureType.find(element);

	ASSERT(iter != mAllOfTypeStructureType.end());

	mAllOfTypeStructureType.erase(iter);
    }

    void
    Model::internal_remove_structure_field(StructureField * element) {
	set<StructureField *>::iterator iter = mAllOfTypeStructureField.find(element);

	ASSERT(iter != mAllOfTypeStructureField.end());

	mAllOfTypeStructureField.erase(iter);
    }

    void
    Model::internal_remove_enumeration_type(EnumerationType * element) {
	set<EnumerationType *>::iterator iter = mAllOfTypeEnumerationType.find(element);

	ASSERT(iter != mAllOfTypeEnumerationType.end());

	mAllOfTypeEnumerationType.erase(iter);
    }

    void
    Model::internal_remove_collection_type(CollectionType * element) {
	set<CollectionType *>::iterator iter = mAllOfTypeCollectionType.find(element);

	ASSERT(iter != mAllOfTypeCollectionType.end());

	mAllOfTypeCollectionType.erase(iter);
    }

} // namespace MOF
