// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_MODEL_ELEMENT_H
#define MOF_MODEL_ELEMENT_H

#include "reflective/class.h"

#include "name.h"

#include <set>
#include <list>
#include <string>
#include <iosfwd>
using namespace std;

namespace Common { }

namespace MOF {

    using namespace Common;

    class Model;

    class Boolean;

    class Constraint;
    class Tag;
    class Namespace;

    //

    class ModelElement : public REF::Class {
    public:
	void set_name(const Name * name);
	const Name * get_name() const { return mName; }
#if 0
	bool has_name(const Name * name) const { return *mName == *name; }
#endif

	void set_annotation(const Name * annotation);
	const Name * get_annotation() const { return mAnnotation; }

	const Boolean * is_frozen() const { return mFrozen; }
	void set_frozen(const Boolean * frozen);

	const Boolean * is_visible() const { return mVisible; }
	void set_visible(const Boolean * visible);

	//

	Namespace * get_container() { return mContainer; }
	void set_container(Namespace * container);

	const set<ModelElement *> & get_providers() { return mProviders; }
	void add_provider(ModelElement * provider);
	void remove_provider(ModelElement * provider);
	void set_providers(const set<ModelElement *> & providers);

	const list<Tag *> & get_tags() { return mTags; }
	void add_tag(Tag * tag, int index = -1);
	void remove_tag(Tag * tag);
	void set_tags(const list<Tag *> & tags);

	const set<Constraint *> & get_constraints() { return mConstraints; }
	void add_constraint(Constraint * constraint);
	void remove_constraint(Constraint * constraint);
	void set_constraints(const set<Constraint *> & constraints);

	//

	string get_qualified_name() const;

	// internal

	void internal_set_container(Namespace * container) throw();

	void internal_add_dependent(ModelElement * dependent) throw();
	void internal_remove_dependent(ModelElement * dependent) throw();

	void internal_add_tag(Tag * tag) throw();
	void internal_remove_tag(Tag * tag) throw();

	void internal_add_constraint(Constraint * constraint) throw();
	void internal_remove_constraint(Constraint * constraint) throw();

    protected:
	ModelElement();
	~ModelElement();

	// internal reflective

	void ref_set_context_impl(set<string> & visited, REF::Context * context);

	void ref_implant_impl(set<string> & visited) throw();

	void ref_uproot_impl(set<string> & visited) throw();

	//

	const string & ref_get_type() const;

	bool ref_is_a_type_of_impl(set<string> & visited,
				       const string & class_name) const;

	void ref_delete_impl(set<string> & visited);

	//

	bool ref_add_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role,
				   REF::Class * position);

	bool ref_remove_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role);

	bool ref_get_role_impl(set<string> & visited,
				   REF::Class * & klass,
				   const string & role_name);

	bool ref_set_role_impl(set<string> & visited,
				   const string & role_name,
				   REF::Class * role);

	bool ref_get_role_iterator_impl(set<string> & visited,
					REF::Iterator * & iterator,
					const string & role_name);

	bool ref_create_class_impl(set<string> & visitd,
				   REF::Class * & role,
				   const string & role_name) const;

	bool ref_get_attribute_impl(set<string> & visited,
				    const REF::Attribute * & attribute,
				    const string & attribute_name) const;

	bool ref_set_attribute_impl(set<string> & visited,
					const string & attribute_name,
					const REF::Attribute * attribute);

	bool ref_get_attribute_set_impl(set<string> & visited,
					const vector<const REF::Attribute *> * & attribute_set,
					const string & attribute_name) const;

	bool ref_set_attribute_set_impl(set<string> & visited,
					const string & attribute_name,
					const vector<const REF::Attribute *> * attribute_set);

	bool ref_create_attribute_impl(set<string> & visited,
					   REF::Attribute * & attribute,
					   const string & attribute_name) const;


    private:
	Name * mName;
	Name * mAnnotation;

	Boolean * mFrozen;
	Boolean * mVisible;

	Namespace * mContainer;
	set<ModelElement *> mProviders;
	set<ModelElement *> mDependents;
	list<Tag *> mTags;
	set<Constraint *> mConstraints;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_MODEL_ELEMENT_H
