// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // CONFIG_H

#include "package.h"

#include "class.h"
#include "data_type.h"
#include "association.h"
#include "exception.h"
#include "constraint.h"
#include "constant.h"
#include "tag.h"
#include "model.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <typeinfo>
using namespace std;

namespace MOF {

    const string Package::TYPE = "Package";

    //

    Package::Package() : GeneralizableElement() {
    }

    Package::~Package() {
    }

#if 0
    void
    Package::externalize(FormatType format) {
    }

    void
    Package::internalize(FormatType formate, istream & stream) {
    }
#endif

    //

    bool
    Package::ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const {
	return class_name == TYPE || GeneralizableElement::ref_is_a_type_of(class_name);
    }

    const string &
    Package::ref_get_type() const {
	return TYPE;
    }

    void
    Package::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_add_package(this);

	    GeneralizableElement::ref_implant_impl(visited);
	}
    }

    void
    Package::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();

	    ASSERT(context);

	    Model * model = known_cast<Model *>(context);

	    model->internal_remove_package(this);

	    GeneralizableElement::ref_implant_impl(visited);
	}
    }

} // namespace MOF
