// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_REFERENCE_H
#define MOF_REFERENCE_H

#include "structural_feature.h"

namespace MOF {

    class AssociationEnd;

    using namespace Common;

    const string REFERENCE_MULTIPLICITY_MUST_MATCH_END =
	"org.omg.mof:constraint.model.reference.reference_multiplicity_must_match_end";
    const string REFERENCE_MUST_BE_INSTANCE_SCOPED =
	"org.omg.mof:constraint.model.reference.reference_must_be_instance_scoped";
    const string CHANGEABLE_REFERENCE_MUST_HAVE_CHANGEABLE_END =
	"org.omg.mof:constraint.model.reference.changeable_reference_must_have_changeable_end";
    const string REFERENCE_TYPE_MUST_MATCH_END_TYPE =
	"org.omg.mof:constraint.model.reference.reference_type_must_match_end_type";
    const string REFERENCED_END_MUST_BE_NAVIGABLE =
	"org.omg.mof:constraint.model.reference.referenced_end_must_be_navigable";
    const string CONTAINER_MUST_MATCH_EXPOSED_TYPE =
	"org.omg.mof:constraint.model.reference.container_must_match_exposed_type";
    const string REFERENCED_END_MUST_BE_VISIBLE =
	"org.omg.mof:constraint.model.reference.referenced_end_must_be_visible";

    class Model;
    class Reference;

    //

    class Reference : public StructuralFeature {
    public:
	Reference();
	~Reference();

	void set_referenced_end(AssociationEnd * reference_end);
	AssociationEnd * get_referenced_end() { return mReferencedEnd; }

	void set_exposed_end(AssociationEnd * exposed_end);
	AssociationEnd * get_exposed_end() { return mExposedEnd; }

    protected:
	// reflective

	bool ref_is_a_type_of_impl(set<string> & visited, const string & class_name) const;
	const string & ref_get_type() const;

	void ref_delete_impl(set<string> & visited);

	void ref_implant_impl(set<string> & visited) throw();
	void ref_uproot_impl(set<string> & visited) throw();

    private:
	AssociationEnd * mReferencedEnd;
	AssociationEnd * mExposedEnd;

	static const string TYPE;
    };

} // namespace MOF

#endif // MOF_REFERENCE_H
