// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef MOF_WRITER_H
#define MOF_WRITER_H

#include "foundation/object.h"

#include "xml/stream.h"
using namespace XML;

#include <iosfwd>
#include <typeinfo>
#include <map>
using namespace std;

namespace MOF {

    // concrete types

    class Association;
    class AssociationEnd;
    class Attribute;
    class Class;
    class Constant;
    class Constraint;
    class DataType;
    class Exception;
    class Import;
    class Operation;
    class Package;
    class Parameter;
    class Reference;
    class Tag;
    class AliasType;

    // abstract types

    class BehavioralFeature;
    class Classifier;
    class Feature;
    class GeneralizableElement;
    class ModelElement;
    class Namespace;
    class StructuralFeature;
    class TypedElement;

    class Writer : public Object {
    public:
	explicit Writer(ModelElement * root = 0);
	~Writer();

	virtual void write(ostream & ost);

    protected:
	void invoke_handler(ModelElement * model_element, bool reference);

	// start the entry, but uncasted

	void a_association(ModelElement * model_element, bool reference);
	void a_association_end(ModelElement * model_element, bool reference);
	void a_attribute(ModelElement * model_element, bool reference);
	void a_class(ModelElement * model_element, bool reference);
	void a_constant(ModelElement * model_element, bool reference);
	void a_constraint(ModelElement * model_element, bool reference);
	void a_data_type(ModelElement * model_element, bool reference);
	void a_exception(ModelElement * model_element, bool reference);
	void a_import(ModelElement * model_element, bool reference);
	void a_operation(ModelElement * model_element, bool reference);
	void a_package(ModelElement * model_element, bool reference);
	void a_parameter(ModelElement * model_element, bool reference);
	void a_reference(ModelElement * model_element, bool reference);
	void a_tag(ModelElement * model_element, bool reference);
	void a_alias_type(ModelElement * model_element, bool reference);

	// start the entry

	void b_association(Association * association, bool reference);
	void b_association_end(AssociationEnd * association_end, bool reference);
	void b_attribute(Attribute * attribute, bool reference);
	void b_class(Class * klass, bool reference);
	void b_constant(Constant * constant, bool reference);
	void b_constraint(Constraint * constraint, bool reference);
	void b_data_type(DataType * data_type, bool reference);
	void b_exception(Exception * exception, bool reference);
	void b_import(Import * import, bool reference);
	void b_operation(Operation * operation, bool reference);
	void b_package(Package * package, bool reference);
	void b_parameter(Parameter * parameter, bool reference);
	void b_reference(Reference * reference, bool reference);
	void b_tag(Tag * tag, bool reference);
	void b_alias_type(AliasType * alias_type, bool reference);

	// fill in the guts (concrete types)

	void c_association(Association * association);
	void c_association_end(AssociationEnd * association_end);
	void c_attribute(Attribute * attribute);
	void c_class(Class * klass);
	void c_constant(Constant * constant);
	void c_constraint(Constraint * constraint);
	void c_data_type(DataType * data_type);
	void c_exception(Exception * exception);
	void c_import(Import * import);
	void c_operation(Operation * operation);
	void c_package(Package * package);
	void c_parameter(Parameter * parameter);
	void c_reference(Reference * reference);
	void c_tag(Tag * tag);
	void c_alias_type(AliasType * alias_type);

	// fill in the guts (abstract types)

	void d_behavioral_feature(BehavioralFeature * behavioral_feature);
	void d_classifier(Classifier * classifier);
	void d_feature(Feature * feature);
	void d_generalizable_element(GeneralizableElement * generalizable_element);
	void d_model_element(ModelElement * model_element);
	void d_namespace(Namespace * namspace);
	void d_structural_feature(StructuralFeature * structural_feature);
	void d_typed_element(TypedElement * typed_element);

    private:
	ModelElement * mRoot;

	typedef const type_info * TypeInfo;
	typedef void (Writer::*Method)(ModelElement *, bool);

	struct Compare { bool operator() (TypeInfo a, TypeInfo b) const { return a->before(*b); } };

	map<TypeInfo, Method, Compare> mMap;
	Stream * mStream;
    };

    ostream & operator << (ostream & ost, Writer & writer);

} // namespace MOF

#endif // MOF_WRITER_H
