#ifndef OCL_LEXER_H
#define OCL_LEXER_H

#include <iosfwd>
#include <deque>
#include <stack>
#include <string>
using namespace std;

#include "foundation/exception.h"

namespace OCL {

    enum TokenKind {
	TOKEN_EOF,

	TOKEN_LEFT_ROUND_BRACKET,
	TOKEN_RIGHT_ROUND_BRACKET,

	TOKEN_LEFT_SQUARE_BRACKET,
	TOKEN_RIGHT_SQUARE_BRACKET,

	TOKEN_LEFT_CURLY_BRACKET,
	TOKEN_RIGHT_CURLY_BRACKET,

	TOKEN_COLON,
	TOKEN_DOUBLE_COLON,

	TOKEN_COMMA,

	TOKEN_EQUAL,
	TOKEN_NOT_EQUAL,
	TOKEN_LESS_THAN,
	TOKEN_GREATER_THAN,
	TOKEN_LESS_EQUAL,
	TOKEN_GREATER_EQUAL,
	TOKEN_RIGHT_ARROW,
	TOKEN_DOT_DOT,
	TOKEN_DOT,
	TOKEN_HASH,
	TOKEN_SEMICOLON,
	TOKEN_BAR,
	TOKEN_AT,
	TOKEN_PLUS,
	TOKEN_MINUS,
	TOKEN_MULTIPLY,
	TOKEN_DIVIDE,

	TOKEN_NAME,
	TOKEN_NUMBER,
	TOKEN_STRING,

	TOKEN_LITERAL_CONTEXT,
	TOKEN_LITERAL_INV,
	TOKEN_LITERAL_PRE,
	TOKEN_LITERAL_POST,
	TOKEN_LITERAL_DEF,
	TOKEN_LITERAL_LET,
	TOKEN_LITERAL_IN,
	TOKEN_LITERAL_ENUM,
	TOKEN_LITERAL_AND,
	TOKEN_LITERAL_OR,
	TOKEN_LITERAL_XOR,
	TOKEN_LITERAL_IMPLIES,
	TOKEN_LITERAL_SET,
	TOKEN_LITERAL_BAG,
	TOKEN_LITERAL_SEQUENCE,
	TOKEN_LITERAL_COLLECTION,

	TOKEN_LITERAL_NOT,

	TOKEN_LITERAL_IF,
	TOKEN_LITERAL_THEN,
	TOKEN_LITERAL_ELSE,
	TOKEN_LITERAL_ENDIF,
    };

    ostream & operator << (ostream & ost, TokenKind token_kind);

    class LexicalException : public Exception {
    protected:
	explicit LexicalException(const string & details);
	~LexicalException();
    };

    class ExpectedToken : public LexicalException {
    public:
	explicit ExpectedToken(const string & details);
	~ExpectedToken();
    };

    class BadToken : public LexicalException {
    public:
	explicit BadToken(const string & details);
	~BadToken();
    };

    class Lexer {
    public:
	explicit Lexer(istream & input);
	~Lexer();

	TokenKind lookahead(int amount = 1);
	void match(TokenKind token);
	const string & value();

	void push();
	void pop();
	void restore();

    protected:
	void shift();
	void fetch();

	static bool is_ident(char c);	// is legal as subsequent character in identifier

    private:
	istream & mInput;

	int mOffset;
	deque<pair<TokenKind, string> > mDeque;
	stack<int> mMarks;
    };

} // namespace OCL

#endif // OCL_LEXER_H
