#ifndef OCL_PARSER_H
#define OCL_PARSER_H

#include "lexer.h"

namespace OCL {

    class ParserException : public Exception {
    protected:
	explicit ParserException(const string & details);
	~ParserException();
    };

    class SyntacticException : public ParserException {
    public:
	explicit SyntacticException(const string & details);
	~SyntacticException();
    };

    class Parser : private Lexer {
    public:
	explicit Parser(istream & input);
	~Parser();

	void parse();

    private:
	void path_name();
	void constraint();
	void context_declaration();
	void let_expression();
	void stereotype();
	void ocl_expression();
	void operation_context();
	void classifier_context();
	void operation_name();
	void formal_parameter_list();
	void return_type();
	void type_specifier();
	void simple_type_specifier();
	void collection_type();
	void collection_kind();
	void expression();
	void logical_expression();
	void if_expression();
	void relational_expression();
	void logical_operator();
	void additive_expression();
	void relational_operator();
	void multiplicative_expression();
	void add_operator();
	void unary_expression();
	void multiply_operator();
	void unary_operator();
	void postfix_expression();
	void primary_expression();
	void property_call();
	void literal_collection();
	void literal();
	void property_call_parameters();
	void declarator();
	void actual_parameter_list();
	void enum_literal();
	void collection_item();
	void time_expression();
	void qualifiers();

	int mGuessing;
	int mDepth;
    };

} // namespace OCL

#endif // OCL_PARSER_H
