// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

// C++ Implementations

#include "foundation/object.h"
#include "foundation/macros.h"

#include "reflective/class.h"
#include "reflective/context.h"
#include "reflective/changes.h"

#include "toolkit/widget.h"
#include "toolkit/tree.h"
#include "toolkit/document.h"
#include "toolkit/tag.h"
#include "toolkit/mark.h"
#include "toolkit/mouse_event.h"

#include "drawing/image.h"

// Python Interfaces

#include "python/foundation/object.h"

#include "python/reflective/class.h"
#include "python/reflective/context.h"
#include "python/reflective/changes.h"

#include "python/toolkit/widget.h"
#include "python/toolkit/tree.h"
#include "python/toolkit/document.h"
#include "python/toolkit/tag.h"
#include "python/toolkit/mark.h"
#include "python/toolkit/mouse_event.h"

#include "python/drawing/image.h"

// standard header files

#include <sstream>
using namespace std;

namespace Python {

    bool convert_to_object(Object *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            return true;
        }
        else if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &reflectiveClassType) {
            ClassReference * rp = (ClassReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &reflectiveContextType) {
            ContextReference * rp = (ContextReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &toolkitWidgetType) {
            WidgetReference * rp = (WidgetReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &toolkitTreeType) {
            TreeReference * rp = (TreeReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &toolkitDocumentType) {
            DocumentReference * rp = (DocumentReference *)obj;
            result = rp->get();
            return true;
        }
        else if (obj->ob_type == &toolkitTagType) {
            TagReference * rp = (TagReference *)obj;
            result = rp->get();
            return true;
        }
        /* FIXUP is this required?
           if (obj->ob_type == &toolkitMouseEventType) {
           MouseEventReference * rp = (MouseEventReference *)obj;
           result = rp->get();
           return true;
        }
        }*/
        else if (obj->ob_type == &drawingImageType) {
            ImageReference * rp = (ImageReference *)obj;
            result = rp->get();
            return true;
        }
        else {
        // Unsupported type
            TRACE(1, "Unsupported type");

            ostringstream ost;
            ost << obj->ob_type->tp_name << " not convertable to Object type";

            PyErr_SetString(PyExc_TypeError, ost.str().c_str());
            return false;
        }
    }


    bool convert_to_class(REF::Class *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<REF::Class *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &reflectiveClassType) {
            ClassReference * rp = (ClassReference *)obj;
            result = dynamic_cast<REF::Class *>(rp->get());
            if (result)
                return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Class type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_context(REF::Context *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<REF::Context *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &reflectiveContextType) {
            ContextReference * rp = (ContextReference *)obj;
            result = rp->get();
            if (result)
                return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Context type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_widget(Toolkit::Widget *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::Widget *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitWidgetType) {
            WidgetReference * rp = (WidgetReference *)obj;
            result = rp->get();
            return true;
        }
        if (obj->ob_type == &toolkitTreeType) {
            TreeReference * rp = (TreeReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Widget type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_tree(Toolkit::Tree *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::Tree *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitWidgetType) {
            WidgetReference * rp = (WidgetReference *)obj;
            result = dynamic_cast<Toolkit::Tree *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitTreeType) {
            TreeReference * rp = (TreeReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Tree type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_document(Toolkit::Document *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::Document *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitDocumentType) {
            DocumentReference * rp = (DocumentReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Document type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_tag(Toolkit::Tag *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::Tag *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitTagType) {
            TagReference * rp = (TagReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Document type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_mark(Toolkit::Mark *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::Mark *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitMarkType) {
            MarkReference * rp = (MarkReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to Mark type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_mouse_event(const Toolkit::MouseEvent *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Toolkit::MouseEvent *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &toolkitMouseEventType) {
            MouseEventReference * rp = (MouseEventReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to MouseEvent type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }


    bool convert_to_image(Drawing::Image *& result, PyObject * obj) {
        if (obj == 0)
            return false;

        if (obj == Py_None) {
            result = 0;
            return true;
        }
        if (obj->ob_type == &foundationObjectType) {
            ObjectReference * rp = (ObjectReference *)obj;
            result = dynamic_cast<Drawing::Image *>(rp->get());
            if (result)
                return true;
        }
        if (obj->ob_type == &drawingImageType) {
            ImageReference * rp = (ImageReference *)obj;
            result = rp->get();
            return true;
        }

        // Unsupported type
        TRACE(1, "Unsupported type");

        ostringstream ost;
        ost << obj->ob_type->tp_name << " not convertable to MouseEvent type";

        PyErr_SetString(PyExc_TypeError, ost.str().c_str());
        return false;
    }

} // namespace Python
