// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

// C++ Implementations

#include "foundation/object.h"
#include "foundation/macros.h"

#include "reflective/class.h"
#include "reflective/context.h"

#include "toolkit/widget.h"
#include "toolkit/tree.h"
#include "toolkit/document.h"
#include "toolkit/tag.h"
#include "toolkit/mouse_event.h"
#include "drawing/image.h"

// Python Interfaces

//#include "python/reflective/class.h"
//#include "python/reflective/context.h"
//#include "python/toolkit/widget.h"
//#include "python/toolkit/tree.h"
//#include "python/toolkit/document.h"
//#include "python/toolkit/tag.h"
//#include "python/toolkit/mouse_event.h"
//#include "python/drawing/image.h"

#include "python/foundation/object.h"

#include <map>
using namespace std;

//================================================================================

namespace {
  using namespace Python;

  typedef map<Object *, PyObject *> ReferenceMap;
  ReferenceMap reference_table;



  //--------------------------------------------------------------------------------

  struct PyMethodDef object_methods[ ] = {
  
    { NULL, NULL }
  };



  PyObject *
  object_getattr(PyObject * self, char * name) {

    // Look for a method
    return Py_FindMethod(object_methods, self, name);
  }


  int
  object_setattr(PyObject * self, char * name, PyObject * value) {
    PyErr_SetString(PyExc_NameError, "No attributes to be set");
    return -1;
  }


  void
  object_dealloc(PyObject * self) {
    if (PyObject_TypeCheck(self, & foundationObjectType)) {
      ObjectReference * rp = (ObjectReference *) self;
      if (rp->mObject) {
	reference_table.erase(rp->mObject);
	rp->mObject->unref();
      }
    }
    PyMem_DEL(self);
  }


}


//================================================================================

namespace Python {

  //================================================================================

  PyTypeObject foundationObjectType = {
    PyObject_HEAD_INIT(&PyType_Type)		// Initialisation
    0,						// ob_size	: Usually 0
    "Object",				        // tp_name	: Type Name
    sizeof(ObjectReference),			// tp_basicsize : Object size
    0,						// tp_itemsize  : Usually 0
  
    // Standard methods
    (destructor) object_dealloc,		// tp_dealloc	: refcount = 0
    (printfunc) 0,				// tp_print	: print x
    (getattrfunc) object_getattr,		// tp_getattr	: x.attr
    (setattrfunc) object_setattr,		// tp_setattr	: a.attr = v
    (cmpfunc) 0,				// tp_compare	: x > y
    (reprfunc) 0,				// tp_repr	: repr(x)

    // Type categories
    0,						// tp_as_number
    0,						// tp_as_sequence
    0,						// tp_as_mapping

    (hashfunc) 0,				// tp_hash
    (ternaryfunc) 0,				// tp_call
    (reprfunc) 0,				// tp_str
    (getattrofunc) 0,				// tp_getatto
    (setattrofunc) 0,				// tp_setatto

    0,						// tp_as_buffer
    Py_TPFLAGS_BASETYPE,			// tp_flags
    0,						// tp_doc
  };


  //--------------------------------------------------------------------------------

  PyObject *
  object_reference_new(Object * object) {

    // Check for NULL
    if (object == 0)
      return Py_BuildValue("");


    PyObject * result = 0;
    ReferenceMap::iterator iter = reference_table.find(object);
    if (iter == reference_table.end()) {

      // Create a new reference
      ObjectReference * self = PyObject_NEW(ObjectReference, & foundationObjectType);
      if (! self)
	return NULL;
    
      object->ref();		 // take joint ownership of the object
      self->mObject = object;
      result = (PyObject *) self;
      reference_table[object] = result;
    }
    else {
      TRACE(2, "Reusing an existing reference");
      // Use an existing reference
      result = iter->second;
      Py_INCREF(result);
    }
    return result;
  }

  PyObject *
  object_reference_new(PyObject * self, PyObject * args) {
    MESSAGE("Create a new Object");
    NYI();
    return NULL;
  }


}
