// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef PYTHON_PYTHON_H
#define PYTHON_PYTHON_H

#include <Python.h>

namespace Python {


  class Interpreter {
    // This singleton class represents the python interpreter instance
  public:
    static Interpreter * instance();

  protected:
    Interpreter();

    static Interpreter * mInstance;
  };


  //--------------------------------------------------------------------------------


  class Module {
    // This class is used to gain access to a python module in C++
  public:
    Module(const char * name);
    ~Module();
    const char *	get_name() const { return mName; }
    PyObject *		get_self() const { return mObject; }
    PyObject *		get_item(const char * name) const;

  protected:
    const char *	mName;
    PyObject *		mObject;
    PyObject *		mDictionary;
  };


  class Callable {
    // This class is used to represent a python function in C++
  public:
    Callable(const Module & module, const char * name);
    ~Callable();
    const char *	get_name() const { return mName; }
    PyObject *		call(PyObject * args);

  protected:
    const char *	mName;
    PyObject *		mFunction;
  };

}

#endif
