// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "foundation/object.h"
#include "foundation/macros.h"
#include "foundation/exception.h"

#include "toolkit/factory.h"
#include "toolkit/tag.h"

// Python interfaces

#include "python/common/convert.h"
#include "python/foundation/object.h"
#include "python/toolkit/tag.h"

#include <map>
using namespace std;

//================================================================================

namespace {
  using namespace Python;


  typedef map<Object *, PyObject *> ReferenceMap;
  ReferenceMap reference_table;



  PyObject *
  wrapped_tag_get_name(PyObject * self, PyObject * args) {
    // const string & get_name() const;
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tag.get_name"))
	return NULL;

      // Finally call the method
      const string & name = rp->mTag->get_name();
      return Py_BuildValue("s", name.c_str());
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  //--------------------------------------------------------------------------------


  PyObject *
  wrapped_tag_is_sensitive_change(PyObject * self, PyObject * args) {
    //bool	is_sensitive_change() const;
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tag.is_sensitive_change"))
	return NULL;

      // Finally call the method
      int result = rp->mTag->is_sensitive_change();
      return Py_BuildValue("i", result);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  PyObject *
  wrapped_tag_is_sensitive_apply(PyObject * self, PyObject * args) {
    //bool	is_sensitive_apply()  const;
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tag.is_sensitive_apply"))
	return NULL;

      // Finally call the method
      int result = rp->mTag->is_sensitive_apply();
      return Py_BuildValue("i", result);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  PyObject *
  wrapped_tag_is_sensitive_entry(PyObject * self, PyObject * args) {
    //bool	is_sensitive_entry()  const;
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      if (!PyArg_ParseTuple(args, ":Tag.is_sensitive_entry"))
	return NULL;

      // Finally call the method
      int result = rp->mTag->is_sensitive_entry();
      return Py_BuildValue("i", result);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }


  //--------------------------------------------------------------------------------


  PyObject *
  wrapped_tag_set_sensitive_change(PyObject * self, PyObject * args) {
    //void	set_sensitive_change(bool s)
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      int state = 0;
      if (!PyArg_ParseTuple(args, "i:Tag.set_sensitive_change", & state))
	return NULL;

      // Finally call the method
      rp->mTag->set_sensitive_change(state);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  PyObject *
  wrapped_tag_set_sensitive_apply(PyObject * self, PyObject * args) {
    //void	set_sensitive_apply(bool s)
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      int state = 0;
      if (!PyArg_ParseTuple(args, "i:Tag.set_sensitive_apply", & state))
	return NULL;

      // Finally call the method
      rp->mTag->set_sensitive_apply(state);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  PyObject *
  wrapped_tag_set_sensitive_entry(PyObject * self, PyObject * args) {
    //void	set_sensitive_entry(bool s)
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      int state = 0;
      if (!PyArg_ParseTuple(args, "i:Tag.set_sensitive_entry", & state))
	return NULL;

      // Finally call the method
      rp->mTag->set_sensitive_entry(state);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }

  //--------------------------------------------------------------------------------

  PyObject *
  wrapped_tag_set_attribute(PyObject * self, PyObject * args) {
    //virtual bool set_attribute(const char * name, const char * value)
    assert(PyObject_TypeCheck(self, & toolkitTagType));
    TagReference * rp = (TagReference *) self;
    try {
      // Extract the arguments
      const char * name = 0;
      const char * value = 0;
      if (!PyArg_ParseTuple(args, "ss:Tag.set_attribute", & name, & value))
	return NULL;

      // Finally call the method
      rp->mTag->set_attribute(name, value);
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
    // Return None
    return Py_BuildValue("");
  }


  //--------------------------------------------------------------------------------


  struct PyMethodDef toolkit_tag_methods[ ] = {
/*
    { "begin_transaction",	wrapped_tag_begin_transaction, METH_VARARGS },
    { "end_transaction",	wrapped_tag_end_transaction, METH_VARARGS },
*/
    { "get_name",		wrapped_tag_get_name,        METH_VARARGS },

    { "is_sensitive_change",	wrapped_tag_is_sensitive_change,  METH_VARARGS },
    { "is_sensitive_apply",	wrapped_tag_is_sensitive_apply,   METH_VARARGS },
    { "is_sensitive_entry",	wrapped_tag_is_sensitive_entry,   METH_VARARGS },

    { "set_sensitive_change",	wrapped_tag_set_sensitive_change, METH_VARARGS },
    { "set_sensitive_apply",	wrapped_tag_set_sensitive_apply,  METH_VARARGS },
    { "set_sensitive_entry",	wrapped_tag_set_sensitive_entry,  METH_VARARGS },

    { "set_attribute",		wrapped_tag_set_attribute,        METH_VARARGS },
    
    { NULL, NULL }
  };



  PyObject *
  tag_getattr(PyObject * self, char * name) {

    // Look for a method
    return Py_FindMethod(toolkit_tag_methods, self, name);
  }


  int
  tag_setattr(PyObject * self, char * name, PyObject * value) {
    PyErr_SetString(PyExc_NameError, "No attributes to be set");
    return -1;
  }


  void
  tag_dealloc(PyObject * self) {

    if (PyObject_TypeCheck(self, & toolkitTagType)) {
      TagReference * rp = (TagReference *) self;
      if (rp->mTag) {
	reference_table.erase(rp->mTag);
	rp->mTag->unref();
      }
    }
    PyMem_DEL(self);
  }

}

//================================================================================

namespace Python {


  PyTypeObject toolkitTagType = {
    PyObject_HEAD_INIT(&PyType_Type)		// Initialisation
    0,						// ob_size	: Usually 0
    "toolkit.Tag",				// tp_name	: Type Name
    sizeof(TagReference),			// tp_basicsize : Object size
    0,						// tp_itemsize  : Usually 0
    
    // Standard methods
    (destructor) tag_dealloc,			// tp_dealloc	: refcount = 0
    (printfunc) 0,				// tp_print	: print x
    (getattrfunc) tag_getattr,			// tp_getattr	: x.attr
    (setattrfunc) tag_setattr,			// tp_setattr	: a.attr = v
    (cmpfunc) 0,				// tp_compare	: x > y
    (reprfunc) 0,				// tp_repr	: repr(x)

    // Type categories
    0,						// tp_as_number
    0,						// tp_as_sequence
    0,						// tp_as_mapping

    (hashfunc) 0,				// tp_hash
    (ternaryfunc) 0,				// tp_call
    (reprfunc) 0,				// tp_str
    (getattrofunc) 0,				// tp_getatto
    (setattrofunc) 0,				// tp_setatto
    
    0,						// tp_as_buffer
    0,						// tp_flags
    0,						// tp_doc
    
    (traverseproc) 0,				// tp_traverse
    (inquiry) 0,				// tp_clear
    (richcmpfunc) 0,				// tp_richcompare
    0,						// tp_weaklistoffset
    (getiterfunc) 0,				// tp_iter
    (iternextfunc) 0,				// tp_iternext
    0, 						// tp_methods
    0, 						// tp_members
    0,						// tp_getset
    0,						// tp_base

  };


  //--------------------------------------------------------------------------------


  PyObject *
  tag_reference_new(Toolkit::Tag * tag) {

    PyObject * result = 0;
    ReferenceMap::iterator iter = reference_table.find(tag);
    if (iter == reference_table.end()) {

      // Create a new reference
      TagReference * self = PyObject_NEW(TagReference, & toolkitTagType);
      if (! self)
	return NULL;
    
      tag->ref();		 // take joint ownership of the object
      self->mTag = tag;
      result = (PyObject *) self;
      reference_table[tag] = result;
    }
    else {
      //TRACE(2, "Reusing an existing reference");
      // Use an existing reference
      result = iter->second;
      Py_INCREF(result);
    }
    return result;
  }


  PyObject *
  tag_reference_new_tag(PyObject * self, PyObject * args) {
    //TRACE(1, "Create a new Toolkit.Tag");
    try {
      // Extract the arguments
      const char * name;
      PyObject * ref_document = 0;
      if (!PyArg_ParseTuple(args, "Os:Tag", & ref_document, & name))
	return NULL;

      Toolkit::Document * document = 0;
      if (!convert_to_document(document, ref_document))
	return NULL;

      // Finally call the method
      Toolkit::Factory * factory = Toolkit::Factory::instance();
      Toolkit::Tag * tag = factory->create_tag(document, name);
      PyObject * result = tag_reference_new(tag);
      tag->unref(); // release our copy
      return result;
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  PyObject *
  tag_reference_new_block(PyObject * self, PyObject * args) {
    //TRACE(1, "Create a new Toolkit.Block");
    try {
      // Extract the arguments
      const char * name;
      PyObject * ref_document = 0;
      int level = 0;
      if (!PyArg_ParseTuple(args, "Osi:Block", & ref_document, & name, & level))
	return NULL;

      Toolkit::Document * document = 0;
      if (!convert_to_document(document, ref_document))
	return NULL;

      // Finally call the method
      Toolkit::Factory * factory = Toolkit::Factory::instance();
      Toolkit::Tag * tag = factory->create_block(document, name, level);
      PyObject * result = tag_reference_new(tag);
      tag->unref(); // release our copy
      return result;
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

  PyObject *
  tag_reference_new_field(PyObject * self, PyObject * args) {
    //TRACE(1, "Create a new Toolkit.Field");
    try {
      // Extract the arguments
      const char * name;
      PyObject * ref_document = 0;
      int level = 0;
      if (!PyArg_ParseTuple(args, "Osi:Field", & ref_document, & name, & level))
	return NULL;

      Toolkit::Document * document = 0;
      if (!convert_to_document(document, ref_document))
	return NULL;

      // Finally call the method
      Toolkit::Factory * factory = Toolkit::Factory::instance();
      Toolkit::Tag * tag = factory->create_field(document, name, level);
      PyObject * result = tag_reference_new(tag);
      tag->unref(); // release our copy
      return result;
    }
    catch (Exception & e) {
      string error = "Toolkit Exception " + e.get_name() + ": " + e.get_details();
      PyErr_SetString(PyExc_RuntimeError, error.c_str());
      return NULL;
    }
    catch (...) {
      PyErr_SetString(PyExc_RuntimeError, "Undefined C++ exception caught");
      return NULL;      
    }
  }

} // namespace Python
