// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef REFLECTIVE_CLASS_H
#define REFLECTIVE_CLASS_H

#include "foundation/object.h"

#include "common/uuid.h"

#include <string>
#include <set>
#include <vector>
using namespace std;

namespace REF {

    class Context;
    class Attribute;
    class Iterator;

    class Class : public Object {
    public:
	Class(const Common::UUID & uuid = Common::UUID::create());
	Class(const Class & klass);

	//

	const Common::UUID & ref_get_uuid() const { return mUUID; }

	//

	Context * ref_get_context() { return mContext; }

	//

	virtual const string & ref_get_type() const = 0;

	//
	// operations that traverse the hierarchy using the _impl operations below
	//

	// misc

	bool ref_is_a_type_of(const string & class_name) const;
	void ref_delete();

	// unary-role

	void ref_set_role(const string & role_name, Class * role);
	Class * ref_get_role(const string & role_name);

	// multi-role

	void ref_add_role(const string & role_name, Class * role, Class * position = 0);
	void ref_remove_role(const string & role_name, Class * role);
	Iterator * ref_get_role_iterator(const string & role_name);

	// create a class based on the role name

	Class * ref_create_class(const string & role_name) const;

	// unary-attribute

	const Attribute * ref_get_attribute(const string & attribute_name) const;
	void ref_set_attribute(const string & attribute_name, const Attribute * attribute);

	// multi-attribute

	const vector<const Attribute *> & ref_get_attribute_set(const string & attribute_name)const;
	void ref_set_attribute_set(const string & attribute_name,
				   const vector<const Attribute *> & attribute_set);

	// create an attribute based on the attribute name

	Attribute * ref_create_attribute(const string & attribute_name) const;

	//
	// internal functions
	//

	void internal_ref_set_context(Context * context);
	void internal_ref_implant();
	void internal_ref_uproot();

    protected:
	~Class();

	// implementations

	virtual bool ref_is_a_type_of_impl(set<string> & visited,
					   const string & class_name) const = 0;

	virtual void ref_delete_impl(set<string> & visited) = 0;

	//

	virtual bool ref_add_role_impl(set<string> & visited,
				       const string & role_name,
				       Class * role,
				       Class * position) = 0;

	virtual bool ref_remove_role_impl(set<string> & visited,
					  const string & role_name,
					  Class * role) = 0;

	virtual bool ref_get_role_iterator_impl(set<string> & visited,
						Iterator * & iterator,
						const string & role_name) = 0;

	virtual bool ref_set_role_impl(set<string> & visited,
				       const string & role_name,
				       Class * role) = 0;

	virtual bool ref_get_role_impl(set<string> & visited,
				       Class * & role,
				       const string & role_name) = 0;

	virtual bool ref_create_class_impl(set<string> & visitd,
					   Class * & role,
					   const string & role_name) const = 0;

	virtual bool ref_get_attribute_impl(set<string> & visited,
					    const Attribute * & attribute,
					    const string & attribute_name) const = 0;

	virtual bool ref_set_attribute_impl(set<string> & visited,
					    const string & attribute_name,
					    const Attribute * attribute) = 0;

	virtual bool ref_get_attribute_set_impl(set<string> & visited,
						const vector<const Attribute *> * & attribute_set,
						const string & attribute_name) const = 0;

	virtual bool ref_set_attribute_set_impl(set<string> & visited,
						const string & attribute_name,
						const vector<const Attribute *> * attribute_set)= 0;

	virtual bool ref_create_attribute_impl(set<string> & visited,
					       Attribute * & attribute,
					       const string & attribute_name) const = 0;

	//
	// internal implementations
	//

	virtual void ref_set_context_impl(set<string> & visited, Context * context) = 0;

	virtual void ref_implant_impl(set<string> & visited) throw() = 0;

	virtual void ref_uproot_impl(set<string> & visited) throw() = 0;

    private:
	Context * mContext;

	Common::UUID mUUID;
    };

} // namespace REF

#endif // REFLECTIVE_CLASS_H
