// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef REFLECTIVE_LIST_ITERATOR_H
#define REFLECTIVE_LIST_ITERATOR_H

#include "iterator.h"

#ifdef DEBUG
#include "foundation/macros.h"
#endif // DEBUG

#include <list>
using namespace std;

namespace REF {

    template<class T>
    class ListIterator : public Iterator {
    public:
	ListIterator(const list<T *> & l) : mList(l) {
	    rewind();
	}

	Class * get() {
#ifdef DEBUG
	    ASSERT(is_valid());
#endif // DEBUG

	    return *mIter;
	}

	void increment() {
#ifdef DEBUG
	    ASSERT(is_valid());
#endif // DEBUG

	    ++mIter;
	    ++mIndex;
	}

	void rewind() {
	    mIter = mList.begin();
	    mIndex = 0;
	}

	bool is_valid() const {
	    return mIter != mList.end();
	}

	int get_index() const {
	    return mIndex;
	}

    protected:
	~ListIterator() {
	}

    private:
	const list<T *> & mList;
	typename list<T *>::const_iterator mIter;
	int mIndex;
    };

    template<class T>
    ListIterator<T> * iterator(const list<T *> & l) {
	return new ListIterator<T>(l);
    }

} // namespace REF

#endif // REFLECTIVE_LIST_ITERATOR_H
