// Dachshund
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef REFLECTIVE_POINTER_ITERATOR_H
#define REFLECTIVE_POINTER_ITERATOR_H

#include "iterator.h"

#ifdef DEBUG
#include "foundation/macros.h"
#endif // DEBUG

namespace REF {

    // NOTE that if the pointer is null then the iterator will
    // never be valid, this is to be consistent with other iterators.
    // i.e., PointerIterator can be thought of like a list of size 0 or 1
    // where the size is 0 if the pointer is null

    template<class T>
    class PointerIterator : public Iterator {
    public:
	PointerIterator(T * t) : mT(t) {
	    rewind();
	}

	Class * get() {
#ifdef DEBUG
	    ASSERT(is_valid());
#endif // DEBUG

	    return mT;
	}

	void increment() {
#ifdef DEBUG
	    ASSERT(is_valid());
#endif // DEBUG

	    ++mIndex;
	}

	void rewind() {
	    mIndex = 0;
	}

	bool is_valid() const {
	    return mIndex == 0 && mT;
	}

	int get_index() const {
	    return mIndex;
	}

    protected:
	~PointerIterator() {
	}

    private:
	T * mT;
	int mIndex;
    };

    template<class T>
    PointerIterator<T> * iterator(T * t) {
	return new PointerIterator<T>(t);
    }

} // namespace REF

#endif // REFLECTIVE_POINTER_ITERATOR_H
