// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_CANVAS_H
#define TOOLKIT_CANVAS_H

#include "widget.h"
#include "drawing/drawable.h"

#include "cursor.h"

#include <list>
using namespace std;

namespace Toolkit {

    class MouseListener;
    class CanvasListener;
    class KeyListener;
    class MouseEvent;
    class CanvasEvent;
    class KeyEvent;

    class Canvas : virtual public Widget, public Drawing::Drawable {
    public:
	Canvas();
	~Canvas();

	virtual void refresh() = 0;
	virtual void damage(const Drawing::Rectangle & rect) = 0;
	virtual void set_cursor(Cursor cursor) = 0;

	//

	void add(MouseListener * mouse_listener);
	void remove(const MouseListener * mouse_listener);

	void add(CanvasListener * canvas_listener);
	void remove(const CanvasListener * canvas_listener);

	void add(KeyListener * key_listener);
	void remove(const KeyListener * key_listener);

	// TODO add a thing for editing text

    protected:
	bool mouse_entered(const MouseEvent * mouse_event);
	bool mouse_exited(const MouseEvent * mouse_event);
	bool mouse_pressed(const MouseEvent * mouse_event);
	bool mouse_released(const MouseEvent * mouse_event);
	bool mouse_moved(const MouseEvent * mouse_event);
	bool mouse_dragged(const MouseEvent * mouse_event);

	bool exposed(const CanvasEvent * canvas_event);

	bool key_pressed(const KeyEvent * key_event);
	bool key_released(const KeyEvent * key_event);

    private:
	list<MouseListener *> mMouseListeners;
	list<CanvasListener *> mCanvasListeners;
	list<KeyListener *> mKeyListeners;
    };

} // namespace Toolkit

#endif // TOOLKIT_CANVAS_H
