// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_FACTORY_H
#define TOOLKIT_FACTORY_H

#include "foundation/object.h"

#include "message_box_kind.h"
#include "orientation_kind.h"

#include <string>
using namespace std;

namespace Toolkit {

    class Widget;
    class Document;
    class FileSelection;
    class Menu;
    class Palette;
    class Tag;
    class Mark;
    class Canvas;
    class Widget;
    class MessageBox;
    class Box;

    class Factory : public Object {
    public:
	static void instance(Factory * instance);
	static Factory * instance();
	static void shutdown();

	virtual FileSelection * create_file_selection(const string & title,
						      const string & path = "") const = 0;

	virtual Menu * create_menu() const = 0;

	virtual Palette * create_palette() const = 0;

	virtual Tag * create_tag(Document * document, const char * name) const = 0;
	virtual Tag * create_block(Document * document, const char * name, int level) const = 0;
	virtual Tag * create_field(Document * document, const char * name, int level) const = 0;

	// General purpose mark
	virtual Mark * create_mark(const char * name, Mark * location) const = 0;

	// Block mark used to relate document to model
	virtual Mark * create_block_mark(const char * name, Mark * location,
					 Tag * block, int level, Object * object) const = 0;

	virtual Canvas * create_canvas(int canvas_width, int canvas_height) const = 0;

	virtual MessageBox * create_message_box(const string & message,
						MessageBoxKind kind,
						Widget * parent = 0) const = 0;

        virtual Box * create_box(OrientationKind orientation) const = 0;

    protected:
	Factory();
	~Factory();

    private:
	static Factory * mInstance;
    };

} // namespace Toolkit

#endif // TOOLKIT_FACTORY_H
