// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "settings.h"

#include "settings_event.h"
#include "settings_listener.h"

#include "foundation/macros.h"

#include <algorithm>
#include <functional>
using namespace std;

namespace Toolkit {

    Settings * Settings::mInstance = 0;

    Settings *
    Settings::instance() {
	ASSERT(mInstance);

	return mInstance;
    }

    void
    Settings::instance(Settings * instance) {
	ASSERT(instance);
	ASSERT(!mInstance);

	mInstance = instance;
    }

    void
    Settings::shutdown() {
	if(mInstance) {
	    delete mInstance;
	    mInstance = 0;
	}
    }

    Settings::Settings() : Object() {
    }

    Settings::~Settings() {
    }

    void
    Settings::add(SettingsListener * settings_listener) {
	mSettingsListeners.push_back(settings_listener);
    }

    void
    Settings::remove(SettingsListener * settings_listener) {
	mSettingsListeners.remove(settings_listener);
    }

    void
    Settings::value_changed(const string & key) {
	SettingsEvent settings_event(this, key);

	for_each(mSettingsListeners.begin(),
		mSettingsListeners.end(),
		bind2nd(mem_fun(&SettingsListener::value_changed), &settings_event));
    }

} // namespace Toolkit
