// Dachshund							-*- C++ -*-
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_TAG_H
#define TOOLKIT_TAG_H

#include "foundation/object.h"

#include <string>
using namespace std;


// Components of a Document, see document.h

namespace Toolkit {

  class Document;

  class Tag : public Object {
  public:
    Tag(const char * name);
    ~Tag();

    const string & get_name() const { return mName; }

    // Tag type used when locating fields and blocks within document
    bool	is_block() const { return mIsBlock; }
    bool	is_field() const { return mIsField; }
    int		get_level() const { return mLevel; }

    void	set_is_block(bool s) { mIsBlock = s; }
    void	set_is_field(bool s) { mIsField = s; }
    void	set_level(int level) { mLevel = level; }

    // These sensitivities control the issue of calls to DocumentMaintainer methods
    bool	is_sensitive_change() const { return mSensitiveChange; }
    bool	is_sensitive_apply()  const { return mSensitiveApply; }
    bool	is_sensitive_entry()  const { return mSensitiveEntry; }

    void	set_sensitive_change(bool s) { mSensitiveChange = s; }
    void	set_sensitive_apply(bool s)  { mSensitiveApply  = s; }
    void	set_sensitive_entry(bool s)  { mSensitiveEntry  = s; }


    // Set the tag attributes
    virtual bool set_attribute(const char * name, const char * value) = 0;


  protected:
    string	mName;
    bool	mIsBlock;
    bool	mIsField;
    int		mLevel;
    bool	mSensitiveChange;
    bool	mSensitiveApply;
    bool	mSensitiveEntry;
  };



} // namespace Toolkit

#endif // TOOLKIT_TAG_H
