// Dachshund
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef TOOLKIT_TREE_H
#define TOOLKIT_TREE_H

#include "widget.h"

#include <list>
#include <string>
using namespace std;

namespace Drawing {

    class Image;
    class Point;

} // namespace Drawing

namespace Toolkit {

    class TreeMaintainer;
    class MouseListener;
    class MouseEvent;

    class Tree : public virtual Widget {
    public:
	Tree();
	~Tree();

	virtual void add(Object * node, Drawing::Image * image, const string & label, 
			 bool expanded,
			 Object * parent, Object * position = 0) = 0;

	virtual void remove(Object * node) = 0;		// must have no children
	virtual void clear() = 0;

	virtual Object * get_root() = 0;
	virtual Object * get_parent(Object * node) = 0;

	virtual bool contains(Object * node) const = 0;	// is the item somewhere in the tree ?

	virtual void set_image(Object * node, Drawing::Image * image) = 0;
	virtual void set_label(Object * node, const string & label) = 0;

	virtual Object * point_to_node(const Drawing::Point & point) = 0;

	//

	void set_maintainer(TreeMaintainer * maintainer);
	TreeMaintainer * get_maintainer() { return mTreeMaintainer; }

	void add(MouseListener * mouse_listener);
	void remove(const MouseListener * mouse_listener);

    protected:
	bool mouse_entered(const MouseEvent * mouse_event);
	bool mouse_exited(const MouseEvent * mouse_event);
	bool mouse_pressed(const MouseEvent * mouse_event);
	bool mouse_released(const MouseEvent * mouse_event);
	bool mouse_moved(const MouseEvent * mouse_event);
	bool mouse_dragged(const MouseEvent * mouse_event);

    private:
	TreeMaintainer * mTreeMaintainer;

	list<MouseListener *> mMouseListeners;
    };

} // namespace Toolkit

#endif // TOOLKIT_TREE_H
