#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "class_diagram_palette_builder.h"

#include "uml/diagram/class_presentation.h"
#include "uml/diagram/package_presentation.h"
#include "uml/diagram/association_presentation.h"
#include "uml/diagram/dependency_presentation.h"
#include "uml/diagram/generalization_presentation.h"

#include "diagram/mode.h"
#include "diagram/command_set_mode.h"
#include "diagram/mode_select.h"

#include "toolkit/factory.h"

#include "drawing/factory.h"
#include "drawing/image.h"

#include "mode_create_poly_edge.h"
#include "mode_create_node.h"

namespace UMLExt {

    void
    build_class_diagram_palette(Toolkit::Palette * palette) {
        MESSAGE("Gets here!");

        Drawing::Factory * factory = Drawing::Factory::instance();

        Diagram::Mode * mode;
        Command * command;
        Drawing::Image * image;

        mode = new Diagram::ModeSelect();
        mode->set_sticky(true);
        mode->set_replaceable(false);
        image = factory->create_image("palette/cursor.png");
        command = new Diagram::CommandSetMode(mode, "Select");

        palette->add_element(command, image);

        command->execute();         // the default mode

        command->unref();
        image->unref();

        //

        mode = new UMLExt::ModeCreateNode<UML::ClassPresentation>();
        image = factory->create_image("palette/class.png");
        command = new Diagram::CommandSetMode(mode, "Create Class");

        palette->add_element(command, image);

        command->unref();
        image->unref();

        //

        mode = new UMLExt::ModeCreateNode<UML::PackagePresentation>();
        image = factory->create_image("palette/package.png");
        command = new Diagram::CommandSetMode(mode, "Create Package");

        palette->add_element(command, image);

        command->unref();
        image->unref();

        //

        mode = new UMLExt::ModeCreatePolyEdge<UML::AssociationPresentation>();
        image = factory->create_image("palette/aggregation.png");
        command = new Diagram::CommandSetMode(mode, "Create Association");

        palette->add_element(command, image);

        command->unref();
        image->unref();

        //

        mode = new UMLExt::ModeCreatePolyEdge<UML::GeneralizationPresentation>();
        image = factory->create_image("palette/generalization.png");
        command = new Diagram::CommandSetMode(mode, "Create Generalization");

        palette->add_element(command, image);

        command->unref();
        image->unref();

        //

        mode = new UMLExt::ModeCreatePolyEdge<UML::DependencyPresentation>();
        image = factory->create_image("palette/dependency.png");
        command = new Diagram::CommandSetMode(mode, "Create Dependency");

        palette->add_element(command, image);

        command->unref();
        image->unref();
    }

} // namespace UMLExt
