// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_gimlet.h"

#include "foundation/macros.h"

#include "drawing/drawable.h"

#include <cmath>
using namespace std;

namespace UMLExt {

    FigGimlet::FigGimlet(GimletKind kind) : Diagram::FigPoly(),
        mKind(kind) {
	set_fill_color("#ffffff");
	set_line_color("#000000");
    }

    FigGimlet::~FigGimlet() {
    }

    void
    FigGimlet::paint(Drawing::Drawable * drawable) const {
	if(!mPoints.empty()) {
            drawable->set_line_style(get_line_style());

	    if(is_filled()) {
		drawable->set_color(get_fill_color());
		drawable->draw_polygon(mPoints, true);

		drawable->set_color(get_line_color());
		drawable->draw_polygon(mPoints, false);
	    }
	    else {
		drawable->set_color(get_line_color());
		drawable->draw_lines(mPoints);
	    }
	}
    }

    void
    FigGimlet::set_control_points(const Drawing::Point & source,
				  const Drawing::Point & destination) {
	mSource = source;
	mDestination = destination;

	update_points();
    }

    void
    FigGimlet::set_kind(GimletKind kind) {
	if(mKind != kind) {
	    mKind = kind;
	    update_points();
	}
    }

    void
    FigGimlet::update_points() {
	damage();

	if(mKind == GK_NULL) {
	    mPoints.clear();
	}
	else if(mKind == GK_ARROW) {
	    mPoints.resize(3);

	    const double SPREAD = 18.0 * M_PI / 180.0;
	    const double LENGTH = 18.0;

	    double angle = atan2((double)mDestination.get_y() - mSource.get_y(),
				 (double)mDestination.get_x() - mSource.get_x());

	    mPoints[0].set_x(mDestination.get_x() - (int)rint(LENGTH * cos(angle - SPREAD)));
	    mPoints[0].set_y(mDestination.get_y() - (int)rint(LENGTH * sin(angle - SPREAD)));

	    mPoints[1] = mDestination;

	    mPoints[2].set_x(mDestination.get_x() - (int)rint(LENGTH * cos(angle + SPREAD)));
	    mPoints[2].set_y(mDestination.get_y() - (int)rint(LENGTH * sin(angle + SPREAD)));
	}
	else if(mKind == GK_DIAMOND) {
	    mPoints.resize(4);

	    const double SPREAD = 22.0 * M_PI / 180.0;
	    const double LENGTH = 22.0;

	    double angle = atan2((double)mDestination.get_y() - mSource.get_y(),
				 (double)mDestination.get_x() - mSource.get_x());

	    mPoints[0] = mDestination;

	    mPoints[1].set_x(mDestination.get_x() - (int)rint(LENGTH / 2.0 * cos(angle - SPREAD)));
	    mPoints[1].set_y(mDestination.get_y() - (int)rint(LENGTH / 2.0 * sin(angle - SPREAD)));

	    mPoints[2].set_x(mDestination.get_x() - (int)rint(LENGTH * cos(angle) * cos(SPREAD)));
	    mPoints[2].set_y(mDestination.get_y() - (int)rint(LENGTH * sin(angle) * cos(SPREAD)));

	    mPoints[3].set_x(mDestination.get_x() - (int)rint(LENGTH / 2.0 * cos(angle + SPREAD)));
	    mPoints[3].set_y(mDestination.get_y() - (int)rint(LENGTH / 2.0 * sin(angle + SPREAD)));
	}
	else {
	    ASSERT_NOT_REACHED();
	}

	update_bounds_to_points();

	damage();
    }

} // namespace UMLExt
