// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_node.h"

#include "colors.h"

#include "uml/foundation/data_types/name.h"
#include "uml/diagram/node_element.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <sstream>
using namespace std;

namespace UMLExt {

    FigNode::FigNode(UML::PresentationElement * pe) : Diagram::FigNode(pe) {
	set_fill_color(NODE_FILL_COLOR);
	set_line_color(NODE_LINE_COLOR);
    }

    FigNode::~FigNode() {
    }

    void
    FigNode::begin_interaction() {
        Diagram::FigNode::begin_interaction();
    }

    void
    FigNode::end_interaction() {
	begin_transaction();

	UML::NodeElement * ne = known_cast<UML::NodeElement *>(get_owner());

	ostringstream ost;

	ost << get_bounds();

	UML::Name * geometry = new UML::Name();
	geometry->ref_set_value(ost.str());
	ne->set_geometry(geometry);
	geometry->unref();

        Diagram::FigNode::end_interaction();

	end_transaction();
    }

    void
    FigNode::update_to_owner() {
        Diagram::FigNode::update_to_owner();

        UML::NodeElement * ne = known_cast<UML::NodeElement *>(get_owner());

        const UML::Name * geometry = ne->get_geometry();

        if(geometry) {
            Drawing::Rectangle rect;
            istringstream ist(geometry->get_value());

            ist >> rect;

            rebound(rect);
        }
        else {
	    ERROR("No geometry in node element");
        }
    }

} // namespace UMLExt
