// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_note.h"

#include "drawing/drawable.h"

#include "foundation/macros.h"

namespace UMLExt {

    FigNote::FigNote(UML::CommentPresentation * cp) : FigNode(cp) {
        mBaseFig = new Diagram::FigPolyLine();
        mBaseFig->set_filled(true);
        mBaseFig->set_fill_color(get_fill_color());

        mChildren.push_front(mBaseFig);

        //

        mOutlineFig = new Diagram::FigPolyLine();
        mOutlineFig->set_filled(false);
        mOutlineFig->set_line_color(get_line_color());

        mChildren.push_front(mOutlineFig);

        //

        mFlapFig = new Diagram::FigPolyLine();
        mFlapFig->set_filled(false);
        mFlapFig->set_line_color(get_line_color());

        //

        mBodyFig = new Diagram::FigText();

        mChildren.push_front(mBodyFig);
    }

    FigNote::~FigNote() {
    }

    void
    FigNote::update_to_owner() {
        FigNode::update_to_owner();

        // TODO
    }

    void
    FigNote::layout() {
	//   1                  2   3
	// 1 +------------------+
	//   |                  |\		//
	//   |                  | \		//
	//   |                  |  \		//
	// 2 |                  +---+
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	// 3 +----------------------+

	const double FLAP_FRACTION = 1.0 / 5.0;

	int w = get_bounds().get_w();
	int x1 = get_bounds().get_x();
	int x2 = x1 + int((1.0 - FLAP_FRACTION) * double(w - 1) + 0.5);
	int x3 = get_bounds().get_xw();

	int h = get_bounds().get_h();
	int y1 = get_bounds().get_y();
	int y2 = y1 + int(FLAP_FRACTION * double(h - 1) + 0.5);
	int y3 = get_bounds().get_yh();

	vector<Drawing::Point> points(5);

	points[0].set(x1, y1);
	points[1].set(x2, y1);
	points[2].set(x3, y2);
	points[3].set(x3, y3);
	points[4].set(x1, y3);

        mBaseFig->set_points(points);
        mOutlineFig->set_points(points);

	points.resize(3);
	points[0].set(x2, y1);
	points[1].set(x2, y2);
	points[2].set(x3, y2);

        mFlapFig->set_points(points);
    }

} // namespace UMLExt
