// Dachshund,
// Copyright (C) 2002 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "fig_package.h"

#include "uml/foundation/data_types/name.h"
#include "uml/model_management/package.h"

#include "foundation/macros.h"
#include "foundation/utility.h"

#include <sstream>
using namespace std;

namespace UMLExt {

    FigPackage::FigPackage(UML::PackagePresentation * pp) : FigNode(pp) {
	mLittleBaseFig = new Diagram::FigRect();
	mLittleBaseFig->set_filled(true);
	mLittleBaseFig->set_fill_color(get_fill_color());

	mChildren.push_front(mLittleBaseFig);

	//

	mBigBaseFig = new Diagram::FigRect();
	mBigBaseFig->set_filled(true);
	mBigBaseFig->set_fill_color(get_fill_color());

	mChildren.push_front(mBigBaseFig);

	//

	mLittleOutlineFig = new Diagram::FigRect();
	mLittleOutlineFig->set_filled(false);
	mLittleOutlineFig->set_line_color(get_line_color());

	mChildren.push_front(mLittleOutlineFig);

	//

	mBigOutlineFig = new Diagram::FigRect();
	mBigOutlineFig->set_filled(false);
	mBigOutlineFig->set_line_color(get_line_color());

	mChildren.push_front(mBigOutlineFig);

	//

	mNameFig = new Diagram::FigText();

	mChildren.push_front(mNameFig);
    }

    FigPackage::~FigPackage() {
    }

    void
    FigPackage::begin_interaction() {
	FigNode::begin_interaction();
    }

    void
    FigPackage::end_interaction() {
	begin_transaction();

	UML::PackagePresentation * cp = known_cast<UML::PackagePresentation *>(get_owner());

	ostringstream ost;

	ost << get_bounds();

	UML::Name * geometry = new UML::Name();
	geometry->ref_set_value(ost.str());
	cp->set_geometry(geometry);
	geometry->unref();

	FigNode::end_interaction();

	end_transaction();
    }

    void
    FigPackage::update_to_owner() {
	UML::PackagePresentation * pp = known_cast<UML::PackagePresentation *>(get_owner());

	const UML::Name * geometry = pp->get_geometry();

	if(geometry) {
	    Drawing::Rectangle rect;
	    istringstream ist(geometry->get_value());

	    ist >> rect;

	    rebound(rect);
	}
	else {
	    ERROR("No geometry in presentation element");
	}

	//

	UML::Package * p = known_cast<UML::Package *>(pp->get_subject());

	if(p->get_name()) {
	    mNameFig->set_text(p->get_name()->get_value());
	}
	else {
	    mNameFig->set_text("<unnamed>");
	}
    }

    void
    FigPackage::layout() {
	//   1        2             3
	//
	// 1 +--------+
	//   |        |
	//   |        |
	// 2 +--------+-------------+
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	//   |                      |
	// 3 +----------------------+

	const double H_FACTOR = 3.0 / 12.0;
	const double V_FACTOR = 2.0 / 12.0;

	int w = get_bounds().get_w();
	int x1 = get_bounds().get_x();
	int x2 = x1 + int(H_FACTOR * double(w - 1) + 0.5);
	int x3 = get_bounds().get_xw();

	int h = get_bounds().get_h();
	int y1 = get_bounds().get_y();
	int y2 = y1 + int(V_FACTOR * double(h - 1) + 0.5);
	int y3 = get_bounds().get_yh();

	//
	//
	//

	mLittleBaseFig->rebound(Drawing::Rectangle(x1, y1, x2 - x1 + 1, y2 - y1 + 1));
	mLittleOutlineFig->rebound(mLittleBaseFig->get_bounds());
	mBigBaseFig->rebound(Drawing::Rectangle(x1, y2, x3 - x1 + 1, y3 - y2 + 1));
	mBigOutlineFig->rebound(mBigBaseFig->get_bounds());

	mNameFig->translate(Drawing::Vector(x1 + 1 - mNameFig->get_bounds().get_x(),
					    y2 + 1 - mNameFig->get_bounds().get_y()));
	mNameFig->set_width(mBounds.get_w() - 2);
    }

} // namespace UMLExt
