// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "class_presentation.h"

#include "association_presentation.h"
#include "generalization_presentation.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string ClassPresentation::TYPE = "ClassPresentation";
    
    ClassPresentation::ClassPresentation() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    ClassPresentation::ClassPresentation(const ClassPresentation & class_presentation) {
	ASSERT_NOT_REACHED();
    }
    
    ClassPresentation::~ClassPresentation() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    ClassPresentation::internal_add_association_source(AssociationPresentation * association_source) {
	association_source->ref();
	
	int index = mAssociation_source.size();
	
	mAssociation_source.push_back(association_source);
	
	ChangeEvent * change = change_role_added(this, "association_source",
						 association_source, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ClassPresentation::internal_remove_association_source(AssociationPresentation * association_source) {
	mAssociation_source.remove(association_source);
	
	ChangeEvent * change = change_role_removed(this, "association_source",
						   association_source, -1, false);
	log_change(change);
	change->unref();
	
	association_source->unref();
    }
    
    void // BBB
    ClassPresentation::set_association_source(const list<AssociationPresentation *> & association_source) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<AssociationPresentation *> before;
	set<AssociationPresentation *> after;
	
	before.insert(mAssociation_source.begin(), mAssociation_source.end());
	after.insert(association_source.begin(), association_source.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mAssociation_source.size());
	RETURN_IF_FAIL(after.size() == association_source.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<AssociationPresentation *> added;
	vector<AssociationPresentation *> removed;
	
	set_difference(association_source.begin(), association_source.end(),
		       mAssociation_source.begin(), mAssociation_source.end(),
		       back_inserter(added));
	
	set_difference(mAssociation_source.begin(), mAssociation_source.end(),
		       association_source.begin(), association_source.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ClassPresentation::set_association_source,
						  association_source, mAssociation_source);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&AssociationPresentation::internal_set_source), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&AssociationPresentation::internal_set_source),
			 static_cast<ClassPresentation *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mAssociation_source = association_source;
	
	// changes made
	
	for(vector<AssociationPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    AssociationPresentation * association_source = *iter;
	    ChangeEvent * change = change_role_added(this, "association_source",
						     association_source, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<AssociationPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    AssociationPresentation * association_source = *iter;
	    ChangeEvent * change = change_role_removed(this, "association_source",
						       association_source, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ClassPresentation::add_association_source(AssociationPresentation * association_source, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mAssociation_source.size()));
	
	if(index < 0) {
	    index = mAssociation_source.size();
	}
	
	list<AssociationPresentation *>::iterator iter = mAssociation_source.begin();
	
	for(int i = 0; i < static_cast<int>(mAssociation_source.size()); ++i) {
	    RETURN_IF_FAIL(*iter != association_source);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::add_association_source,
						&ClassPresentation::remove_association_source,
						association_source, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "association_source", association_source, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association_source->ref();
	
	mAssociation_source.insert(iter, association_source);
	association_source->internal_set_source(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ClassPresentation::remove_association_source(AssociationPresentation * association_source) {
	// work out the index
	
	int index = 0;
	
	list<AssociationPresentation *>::iterator iter = mAssociation_source.begin();
	
	for(; iter != mAssociation_source.end(); ++iter, ++index) {
	    if(*iter == association_source) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mAssociation_source.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::remove_association_source,
						&ClassPresentation::add_association_source,
						association_source, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "association_source", association_source, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association_source->internal_set_source(static_cast<ClassPresentation *>(0));
	mAssociation_source.erase(iter);
	
	association_source->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ClassPresentation::internal_add_association_destination(AssociationPresentation * association_destination) {
	association_destination->ref();
	
	int index = mAssociation_destination.size();
	
	mAssociation_destination.push_back(association_destination);
	
	ChangeEvent * change = change_role_added(this, "association_destination",
						 association_destination, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ClassPresentation::internal_remove_association_destination(AssociationPresentation * association_destination) {
	mAssociation_destination.remove(association_destination);
	
	ChangeEvent * change = change_role_removed(this, "association_destination",
						   association_destination, -1, false);
	log_change(change);
	change->unref();
	
	association_destination->unref();
    }
    
    void // BBB
    ClassPresentation::set_association_destination(const list<AssociationPresentation *> & association_destination) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<AssociationPresentation *> before;
	set<AssociationPresentation *> after;
	
	before.insert(mAssociation_destination.begin(), mAssociation_destination.end());
	after.insert(association_destination.begin(), association_destination.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mAssociation_destination.size());
	RETURN_IF_FAIL(after.size() == association_destination.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<AssociationPresentation *> added;
	vector<AssociationPresentation *> removed;
	
	set_difference(association_destination.begin(), association_destination.end(),
		       mAssociation_destination.begin(), mAssociation_destination.end(),
		       back_inserter(added));
	
	set_difference(mAssociation_destination.begin(), mAssociation_destination.end(),
		       association_destination.begin(), association_destination.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ClassPresentation::set_association_destination,
						  association_destination, mAssociation_destination);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&AssociationPresentation::internal_set_destination), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&AssociationPresentation::internal_set_destination),
			 static_cast<ClassPresentation *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mAssociation_destination = association_destination;
	
	// changes made
	
	for(vector<AssociationPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    AssociationPresentation * association_destination = *iter;
	    ChangeEvent * change = change_role_added(this, "association_destination",
						     association_destination, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<AssociationPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    AssociationPresentation * association_destination = *iter;
	    ChangeEvent * change = change_role_removed(this, "association_destination",
						       association_destination, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ClassPresentation::add_association_destination(AssociationPresentation * association_destination, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mAssociation_destination.size()));
	
	if(index < 0) {
	    index = mAssociation_destination.size();
	}
	
	list<AssociationPresentation *>::iterator iter = mAssociation_destination.begin();
	
	for(int i = 0; i < static_cast<int>(mAssociation_destination.size()); ++i) {
	    RETURN_IF_FAIL(*iter != association_destination);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::add_association_destination,
						&ClassPresentation::remove_association_destination,
						association_destination, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "association_destination", association_destination, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association_destination->ref();
	
	mAssociation_destination.insert(iter, association_destination);
	association_destination->internal_set_destination(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ClassPresentation::remove_association_destination(AssociationPresentation * association_destination) {
	// work out the index
	
	int index = 0;
	
	list<AssociationPresentation *>::iterator iter = mAssociation_destination.begin();
	
	for(; iter != mAssociation_destination.end(); ++iter, ++index) {
	    if(*iter == association_destination) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mAssociation_destination.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::remove_association_destination,
						&ClassPresentation::add_association_destination,
						association_destination, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "association_destination", association_destination, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	association_destination->internal_set_destination(static_cast<ClassPresentation *>(0));
	mAssociation_destination.erase(iter);
	
	association_destination->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ClassPresentation::internal_add_generalization(GeneralizationPresentation * generalization) {
	generalization->ref();
	
	int index = mGeneralization.size();
	
	mGeneralization.push_back(generalization);
	
	ChangeEvent * change = change_role_added(this, "generalization",
						 generalization, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ClassPresentation::internal_remove_generalization(GeneralizationPresentation * generalization) {
	mGeneralization.remove(generalization);
	
	ChangeEvent * change = change_role_removed(this, "generalization",
						   generalization, -1, false);
	log_change(change);
	change->unref();
	
	generalization->unref();
    }
    
    void // BBB
    ClassPresentation::set_generalization(const list<GeneralizationPresentation *> & generalization) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<GeneralizationPresentation *> before;
	set<GeneralizationPresentation *> after;
	
	before.insert(mGeneralization.begin(), mGeneralization.end());
	after.insert(generalization.begin(), generalization.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mGeneralization.size());
	RETURN_IF_FAIL(after.size() == generalization.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<GeneralizationPresentation *> added;
	vector<GeneralizationPresentation *> removed;
	
	set_difference(generalization.begin(), generalization.end(),
		       mGeneralization.begin(), mGeneralization.end(),
		       back_inserter(added));
	
	set_difference(mGeneralization.begin(), mGeneralization.end(),
		       generalization.begin(), generalization.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ClassPresentation::set_generalization,
						  generalization, mGeneralization);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&GeneralizationPresentation::internal_set_subtype), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&GeneralizationPresentation::internal_set_subtype),
			 static_cast<ClassPresentation *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mGeneralization = generalization;
	
	// changes made
	
	for(vector<GeneralizationPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    GeneralizationPresentation * generalization = *iter;
	    ChangeEvent * change = change_role_added(this, "generalization",
						     generalization, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<GeneralizationPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    GeneralizationPresentation * generalization = *iter;
	    ChangeEvent * change = change_role_removed(this, "generalization",
						       generalization, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ClassPresentation::add_generalization(GeneralizationPresentation * generalization, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mGeneralization.size()));
	
	if(index < 0) {
	    index = mGeneralization.size();
	}
	
	list<GeneralizationPresentation *>::iterator iter = mGeneralization.begin();
	
	for(int i = 0; i < static_cast<int>(mGeneralization.size()); ++i) {
	    RETURN_IF_FAIL(*iter != generalization);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::add_generalization,
						&ClassPresentation::remove_generalization,
						generalization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "generalization", generalization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	generalization->ref();
	
	mGeneralization.insert(iter, generalization);
	generalization->internal_set_subtype(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ClassPresentation::remove_generalization(GeneralizationPresentation * generalization) {
	// work out the index
	
	int index = 0;
	
	list<GeneralizationPresentation *>::iterator iter = mGeneralization.begin();
	
	for(; iter != mGeneralization.end(); ++iter, ++index) {
	    if(*iter == generalization) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mGeneralization.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::remove_generalization,
						&ClassPresentation::add_generalization,
						generalization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "generalization", generalization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	generalization->internal_set_subtype(static_cast<ClassPresentation *>(0));
	mGeneralization.erase(iter);
	
	generalization->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    ClassPresentation::internal_add_specialization(GeneralizationPresentation * specialization) {
	specialization->ref();
	
	int index = mSpecialization.size();
	
	mSpecialization.push_back(specialization);
	
	ChangeEvent * change = change_role_added(this, "specialization",
						 specialization, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    ClassPresentation::internal_remove_specialization(GeneralizationPresentation * specialization) {
	mSpecialization.remove(specialization);
	
	ChangeEvent * change = change_role_removed(this, "specialization",
						   specialization, -1, false);
	log_change(change);
	change->unref();
	
	specialization->unref();
    }
    
    void // BBB
    ClassPresentation::set_specialization(const list<GeneralizationPresentation *> & specialization) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<GeneralizationPresentation *> before;
	set<GeneralizationPresentation *> after;
	
	before.insert(mSpecialization.begin(), mSpecialization.end());
	after.insert(specialization.begin(), specialization.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSpecialization.size());
	RETURN_IF_FAIL(after.size() == specialization.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<GeneralizationPresentation *> added;
	vector<GeneralizationPresentation *> removed;
	
	set_difference(specialization.begin(), specialization.end(),
		       mSpecialization.begin(), mSpecialization.end(),
		       back_inserter(added));
	
	set_difference(mSpecialization.begin(), mSpecialization.end(),
		       specialization.begin(), specialization.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &ClassPresentation::set_specialization,
						  specialization, mSpecialization);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&GeneralizationPresentation::internal_set_supertype), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&GeneralizationPresentation::internal_set_supertype),
			 static_cast<ClassPresentation *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSpecialization = specialization;
	
	// changes made
	
	for(vector<GeneralizationPresentation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    GeneralizationPresentation * specialization = *iter;
	    ChangeEvent * change = change_role_added(this, "specialization",
						     specialization, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<GeneralizationPresentation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    GeneralizationPresentation * specialization = *iter;
	    ChangeEvent * change = change_role_removed(this, "specialization",
						       specialization, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    ClassPresentation::add_specialization(GeneralizationPresentation * specialization, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSpecialization.size()));
	
	if(index < 0) {
	    index = mSpecialization.size();
	}
	
	list<GeneralizationPresentation *>::iterator iter = mSpecialization.begin();
	
	for(int i = 0; i < static_cast<int>(mSpecialization.size()); ++i) {
	    RETURN_IF_FAIL(*iter != specialization);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::add_specialization,
						&ClassPresentation::remove_specialization,
						specialization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "specialization", specialization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specialization->ref();
	
	mSpecialization.insert(iter, specialization);
	specialization->internal_set_supertype(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    ClassPresentation::remove_specialization(GeneralizationPresentation * specialization) {
	// work out the index
	
	int index = 0;
	
	list<GeneralizationPresentation *>::iterator iter = mSpecialization.begin();
	
	for(; iter != mSpecialization.end(); ++iter, ++index) {
	    if(*iter == specialization) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSpecialization.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&ClassPresentation::remove_specialization,
						&ClassPresentation::add_specialization,
						specialization, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "specialization", specialization, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specialization->internal_set_supertype(static_cast<ClassPresentation *>(0));
	mSpecialization.erase(iter);
	
	specialization->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    ClassPresentation::ref_set_context_impl(set<string> & visited,
					    REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    NodeElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    ClassPresentation::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_class_presentation(this);
	    
	    NodeElement::ref_implant_impl(visited);
	}
    }
    
    void
    ClassPresentation::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    NodeElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_class_presentation(this);
	}
    }
    
    bool
    ClassPresentation::ref_add_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role,
					 REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "association_source") {
		AssociationPresentation * association_source = dynamic_cast<AssociationPresentation *>(role);
		
		if(!association_source) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_association_source(association_source, index);
		return true;
	    }
	    
	    if(role_name == "association_destination") {
		AssociationPresentation * association_destination = dynamic_cast<AssociationPresentation *>(role);
		
		if(!association_destination) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_association_destination(association_destination, index);
		return true;
	    }
	    
	    if(role_name == "generalization") {
		GeneralizationPresentation * generalization = dynamic_cast<GeneralizationPresentation *>(role);
		
		if(!generalization) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_generalization(generalization, index);
		return true;
	    }
	    
	    if(role_name == "specialization") {
		GeneralizationPresentation * specialization = dynamic_cast<GeneralizationPresentation *>(role);
		
		if(!specialization) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_specialization(specialization, index);
		return true;
	    }
	    
	    if(NodeElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_remove_role_impl(set<string> & visited,
					    const string & role_name,
					    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_get_role_impl(set<string> & visited,
					 REF::Class * & role,
					 const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_set_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_create_class_impl(set<string> & visited,
					     REF::Class * & role,
					     const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "association_source") {
		role = new AssociationPresentation();
		return true;
	    }
	    
	    if(role_name == "association_destination") {
		role = new AssociationPresentation();
		return true;
	    }
	    
	    if(role_name == "generalization") {
		role = new GeneralizationPresentation();
		return true;
	    }
	    
	    if(role_name == "specialization") {
		role = new GeneralizationPresentation();
		return true;
	    }
	    
	    if(NodeElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_get_role_iterator_impl(set<string> & visited,
						  REF::Iterator * & iterator,
						  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "association_source") {
		iterator = REF::iterator<AssociationPresentation>(get_association_source());
		return true;
	    }
	    
	    if(role_name == "association_destination") {
		iterator = REF::iterator<AssociationPresentation>(get_association_destination());
		return true;
	    }
	    
	    if(role_name == "generalization") {
		iterator = REF::iterator<GeneralizationPresentation>(get_generalization());
		return true;
	    }
	    
	    if(role_name == "specialization") {
		iterator = REF::iterator<GeneralizationPresentation>(get_specialization());
		return true;
	    }
	    
	    if(NodeElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_get_attribute_impl(set<string> & visited,
					      const REF::Attribute * & attribute,
					      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_set_attribute_impl(set<string> & visited,
					      const string & attribute_name,
					      const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_get_attribute_set_impl(set<string> & visited,
						  const vector<const REF::Attribute *> * & attribute_set,
						  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_set_attribute_set_impl(set<string> & visited,
						  const string & attribute_name,
						  const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_create_attribute_impl(set<string> & visited,
						 REF::Attribute * & attribute,
						 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(NodeElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    ClassPresentation::ref_is_a_type_of_impl(set<string> & visited,
					     const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "ClassPresentation") {
		return true;
	    }
	    
	    if(NodeElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    ClassPresentation::ref_get_type() const {
	return TYPE;
    }
    
    void
    ClassPresentation::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<AssociationPresentation *> association_source;
	    set_association_source(association_source);
	    
	    list<AssociationPresentation *> association_destination;
	    set_association_destination(association_destination);
	    
	    list<GeneralizationPresentation *> generalization;
	    set_generalization(generalization);
	    
	    list<GeneralizationPresentation *> specialization;
	    set_specialization(specialization);
	    
	    NodeElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
