// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "dependency_presentation.h"

#include "presentation_element.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string DependencyPresentation::TYPE = "DependencyPresentation";
    
    DependencyPresentation::DependencyPresentation() {
	mSupplier = 0;
	mClient = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    DependencyPresentation::DependencyPresentation(const DependencyPresentation & dependency_presentation) {
	ASSERT_NOT_REACHED();
    }
    
    DependencyPresentation::~DependencyPresentation() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // MMM
    DependencyPresentation::internal_set_supplier(PresentationElement * supplier) {
	ChangeEvent * change = change_role_changed(this, "supplier",
						   supplier, mSupplier, false);
	log_change(change);
	
	change->unref();
	
	if(mSupplier) {
	    mSupplier->unref();
	}
	
	mSupplier = supplier;
	
	if(mSupplier) {
	    mSupplier->ref();
	}
    }
    
    void // LLL
    DependencyPresentation::set_supplier(PresentationElement * supplier) {
	if(mSupplier != supplier) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(supplier) {
		ASSERT(!supplier->ref_get_context() ||
		       supplier->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &DependencyPresentation::set_supplier,
						      supplier, mSupplier);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "supplier", supplier,
						       mSupplier, false);
	    log_change(change);
	    change->unref();
	    
	    if(mSupplier) {
		mSupplier->internal_remove_supplier_dependency(this);
		mSupplier->unref();
	    }
	    
	    mSupplier = supplier;
	    
	    if(mSupplier) {
		mSupplier->internal_add_supplier_dependency(this);
		mSupplier->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // MMM
    DependencyPresentation::internal_set_client(PresentationElement * client) {
	ChangeEvent * change = change_role_changed(this, "client",
						   client, mClient, false);
	log_change(change);
	
	change->unref();
	
	if(mClient) {
	    mClient->unref();
	}
	
	mClient = client;
	
	if(mClient) {
	    mClient->ref();
	}
    }
    
    void // LLL
    DependencyPresentation::set_client(PresentationElement * client) {
	if(mClient != client) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(client) {
		ASSERT(!client->ref_get_context() ||
		       client->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &DependencyPresentation::set_client,
						      client, mClient);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "client", client,
						       mClient, false);
	    log_change(change);
	    change->unref();
	    
	    if(mClient) {
		mClient->internal_remove_client_dependency(this);
		mClient->unref();
	    }
	    
	    mClient = client;
	    
	    if(mClient) {
		mClient->internal_add_client_dependency(this);
		mClient->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void
    DependencyPresentation::ref_set_context_impl(set<string> & visited,
						 REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    EdgeElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    DependencyPresentation::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_dependency_presentation(this);
	    
	    EdgeElement::ref_implant_impl(visited);
	}
    }
    
    void
    DependencyPresentation::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    EdgeElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_dependency_presentation(this);
	}
    }
    
    bool
    DependencyPresentation::ref_add_role_impl(set<string> & visited,
					      const string & role_name,
					      REF::Class * role,
					      REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_remove_role_impl(set<string> & visited,
						 const string & role_name,
						 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_get_role_impl(set<string> & visited,
					      REF::Class * & role,
					      const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		role = get_supplier();
		return true;
	    }
	    
	    if(role_name == "client") {
		role = get_client();
		return true;
	    }
	    
	    if(EdgeElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_set_role_impl(set<string> & visited,
					      const string & role_name,
					      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		PresentationElement * supplier = dynamic_cast<PresentationElement *>(role);
		
		if(!supplier) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_supplier(supplier);
		return true;
	    }
	    
	    if(role_name == "client") {
		PresentationElement * client = dynamic_cast<PresentationElement *>(role);
		
		if(!client) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_client(client);
		return true;
	    }
	    
	    if(EdgeElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_create_class_impl(set<string> & visited,
						  REF::Class * & role,
						  const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		role = new PresentationElement();
		return true;
	    }
	    
	    if(role_name == "client") {
		role = new PresentationElement();
		return true;
	    }
	    
	    if(EdgeElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_get_role_iterator_impl(set<string> & visited,
						       REF::Iterator * & iterator,
						       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "supplier") {
		iterator = REF::iterator<PresentationElement>(get_supplier());
		return true;
	    }
	    
	    if(role_name == "client") {
		iterator = REF::iterator<PresentationElement>(get_client());
		return true;
	    }
	    
	    if(EdgeElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_get_attribute_impl(set<string> & visited,
						   const REF::Attribute * & attribute,
						   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_set_attribute_impl(set<string> & visited,
						   const string & attribute_name,
						   const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_get_attribute_set_impl(set<string> & visited,
						       const vector<const REF::Attribute *> * & attribute_set,
						       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_set_attribute_set_impl(set<string> & visited,
						       const string & attribute_name,
						       const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_create_attribute_impl(set<string> & visited,
						      REF::Attribute * & attribute,
						      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(EdgeElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    DependencyPresentation::ref_is_a_type_of_impl(set<string> & visited,
						  const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "DependencyPresentation") {
		return true;
	    }
	    
	    if(EdgeElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    DependencyPresentation::ref_get_type() const {
	return TYPE;
    }
    
    void
    DependencyPresentation::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    PresentationElement * supplier = 0;
	    set_supplier(supplier);
	    
	    PresentationElement * client = 0;
	    set_client(client);
	    
	    EdgeElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
