// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "diagram.h"

#include "graph_element.h"
#include "uml/foundation/data_types/name.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Diagram::TYPE = "Diagram";
    
    Diagram::Diagram() {
	mName = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Diagram::Diagram(const Diagram & diagram) {
	ASSERT_NOT_REACHED();
    }
    
    Diagram::~Diagram() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mName) {
	    mName->unref();
	}
    }
    
    void // AAA
    Diagram::set_name(const Name * name) {
	begin_transaction();
	
	//
	
	Name * old_value = mName ? new Name(*mName) : 0;
	Name * new_value = name ? new Name(*name) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &Diagram::set_name,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mName) {
	    mName->unref();
	}
	
	mName = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "name", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // EEE
    Diagram::internal_add_element(GraphElement * element) {
	element->ref();
	
	int index = mElement.size();
	
	mElement.push_back(element);
	
	ChangeEvent * change = change_role_added(this, "element",
						 element, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Diagram::internal_remove_element(GraphElement * element) {
	mElement.remove(element);
	
	ChangeEvent * change = change_role_removed(this, "element",
						   element, -1, true);
	log_change(change);
	change->unref();
	
	element->unref();
    }
    
    void // BBB
    Diagram::set_element(const list<GraphElement *> & element) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<GraphElement *> before;
	set<GraphElement *> after;
	
	before.insert(mElement.begin(), mElement.end());
	after.insert(element.begin(), element.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mElement.size());
	RETURN_IF_FAIL(after.size() == element.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<GraphElement *> added;
	vector<GraphElement *> removed;
	
	set_difference(element.begin(), element.end(),
		       mElement.begin(), mElement.end(),
		       back_inserter(added));
	
	set_difference(mElement.begin(), mElement.end(),
		       element.begin(), element.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Diagram::set_element,
						  element, mElement);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&GraphElement::internal_set_diagram), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&GraphElement::internal_set_diagram),
			 static_cast<Diagram *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mElement = element;
	
	// changes made
	
	for(vector<GraphElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    GraphElement * element = *iter;
	    ChangeEvent * change = change_role_added(this, "element",
						     element, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<GraphElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    GraphElement * element = *iter;
	    ChangeEvent * change = change_role_removed(this, "element",
						       element, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Diagram::add_element(GraphElement * element, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mElement.size()));
	
	if(index < 0) {
	    index = mElement.size();
	}
	
	list<GraphElement *>::iterator iter = mElement.begin();
	
	for(int i = 0; i < static_cast<int>(mElement.size()); ++i) {
	    RETURN_IF_FAIL(*iter != element);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Diagram::add_element,
						&Diagram::remove_element,
						element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "element", element, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	element->ref();
	
	mElement.insert(iter, element);
	element->internal_set_diagram(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Diagram::remove_element(GraphElement * element) {
	// work out the index
	
	int index = 0;
	
	list<GraphElement *>::iterator iter = mElement.begin();
	
	for(; iter != mElement.end(); ++iter, ++index) {
	    if(*iter == element) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mElement.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Diagram::remove_element,
						&Diagram::add_element,
						element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "element", element, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	element->internal_set_diagram(static_cast<Diagram *>(0));
	mElement.erase(iter);
	
	element->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Diagram::ref_set_context_impl(set<string> & visited,
				  REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Diagram::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_diagram(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Diagram::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_diagram(this);
	}
    }
    
    bool
    Diagram::ref_add_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role,
			       REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "element") {
		GraphElement * element = dynamic_cast<GraphElement *>(role);
		
		if(!element) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_element(element, index);
		return true;
	    }
	    
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_remove_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_get_role_impl(set<string> & visited,
			       REF::Class * & role,
			       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_set_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_create_class_impl(set<string> & visited,
				   REF::Class * & role,
				   const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "element") {
		role = new GraphElement();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_get_role_iterator_impl(set<string> & visited,
					REF::Iterator * & iterator,
					const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "element") {
		iterator = REF::iterator<GraphElement>(get_element());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_get_attribute_impl(set<string> & visited,
				    const REF::Attribute * & attribute,
				    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		attribute = get_name();
		return true;
	    }
	    
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_set_attribute_impl(set<string> & visited,
				    const string & attribute_name,
				    const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		const Name * name = dynamic_cast<const Name *>(attribute);
		
		if(!name) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_name(name);
		return true;
	    }
	    
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_get_attribute_set_impl(set<string> & visited,
					const vector<const REF::Attribute *> * & attribute_set,
					const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_set_attribute_set_impl(set<string> & visited,
					const string & attribute_name,
					const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_create_attribute_impl(set<string> & visited,
				       REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "name") {
		attribute = new Name();
		return true;
	    }
	    
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Diagram::ref_is_a_type_of_impl(set<string> & visited,
				   const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Diagram") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Diagram::ref_get_type() const {
	return TYPE;
    }
    
    void
    Diagram::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<GraphElement *> element_list = get_element();
	    for_each(element_list.begin(), element_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<GraphElement *> element;
	    set_element(element);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
