// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "feature.h"

#include "classifier.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Feature::TYPE = "Feature";
    
    Feature::Feature() {
	mOwner = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Feature::Feature(const Feature & feature) {
	ASSERT_NOT_REACHED();
    }
    
    Feature::~Feature() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // MMM
    Feature::internal_set_owner(Classifier * owner) {
	ChangeEvent * change = change_role_changed(this, "owner",
						   owner, mOwner, false);
	log_change(change);
	
	change->unref();
	
	if(mOwner) {
	    mOwner->unref();
	}
	
	mOwner = owner;
	
	if(mOwner) {
	    mOwner->ref();
	}
    }
    
    void // LLL
    Feature::set_owner(Classifier * owner) {
	if(mOwner != owner) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(owner) {
		ASSERT(!ref_get_context() || owner->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &Feature::set_owner,
						      owner, mOwner);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "owner", owner,
						       mOwner, false);
	    log_change(change);
	    change->unref();
	    
	    if(mOwner) {
		mOwner->internal_remove_feature(this);
		mOwner->unref();
	    }
	    
	    mOwner = owner;
	    
	    if(mOwner) {
		mOwner->internal_add_feature(this);
		mOwner->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void
    Feature::ref_set_context_impl(set<string> & visited,
				  REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Feature::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_feature(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Feature::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_feature(this);
	}
    }
    
    bool
    Feature::ref_add_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role,
			       REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_remove_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_get_role_impl(set<string> & visited,
			       REF::Class * & role,
			       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = get_owner();
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_set_role_impl(set<string> & visited,
			       const string & role_name,
			       REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		Classifier * owner = dynamic_cast<Classifier *>(role);
		
		if(!owner) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_owner(owner);
		return true;
	    }
	    
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_create_class_impl(set<string> & visited,
				   REF::Class * & role,
				   const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		role = new Classifier();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_get_role_iterator_impl(set<string> & visited,
					REF::Iterator * & iterator,
					const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "owner") {
		iterator = REF::iterator<Classifier>(get_owner());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_get_attribute_impl(set<string> & visited,
				    const REF::Attribute * & attribute,
				    const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_set_attribute_impl(set<string> & visited,
				    const string & attribute_name,
				    const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_get_attribute_set_impl(set<string> & visited,
					const vector<const REF::Attribute *> * & attribute_set,
					const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_set_attribute_set_impl(set<string> & visited,
					const string & attribute_name,
					const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_create_attribute_impl(set<string> & visited,
				       REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Feature::ref_is_a_type_of_impl(set<string> & visited,
				   const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Feature") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Feature::ref_get_type() const {
	return TYPE;
    }
    
    void
    Feature::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    Classifier * owner = 0;
	    set_owner(owner);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
