// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "generalization.h"

#include "generalizable_element.h"
#include "uml/foundation/data_types/name.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Generalization::TYPE = "Generalization";
    
    Generalization::Generalization() {
	mChild = 0;
	mParent = 0;
	mDiscriminator = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Generalization::Generalization(const Generalization & generalization) {
	ASSERT_NOT_REACHED();
    }
    
    Generalization::~Generalization() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
	if(mDiscriminator) {
	    mDiscriminator->unref();
	}
    }
    
    void // AAA
    Generalization::set_discriminator(const Name * discriminator) {
	begin_transaction();
	
	//
	
	Name * old_value = mDiscriminator ? new Name(*mDiscriminator) : 0;
	Name * new_value = discriminator ? new Name(*discriminator) : 0;
	
	//
	
	Command * command = command_flip_argument(this, &Generalization::set_discriminator,
						  new_value, old_value);
	log_command(command);
	command->unref();
	
	// make the changes
	
	if(mDiscriminator) {
	    mDiscriminator->unref();
	}
	
	mDiscriminator = new_value ? new Name(*new_value) : 0;
	
	// changes made
	
	ChangeEvent * change = change_attribute_changed(this, "discriminator", new_value, old_value);
	log_change(change);
	change->unref();
	
	//
	
	if(new_value) {
	    new_value->unref();
	}
	
	if(old_value) {
	    old_value->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // MMM
    Generalization::internal_set_child(GeneralizableElement * child) {
	ChangeEvent * change = change_role_changed(this, "child",
						   child, mChild, false);
	log_change(change);
	
	change->unref();
	
	if(mChild) {
	    mChild->unref();
	}
	
	mChild = child;
	
	if(mChild) {
	    mChild->ref();
	}
    }
    
    void // LLL
    Generalization::set_child(GeneralizableElement * child) {
	if(mChild != child) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(child) {
		ASSERT(!child->ref_get_context() ||
		       child->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &Generalization::set_child,
						      child, mChild);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "child", child,
						       mChild, false);
	    log_change(change);
	    change->unref();
	    
	    if(mChild) {
		mChild->internal_remove_generalization(this);
		mChild->unref();
	    }
	    
	    mChild = child;
	    
	    if(mChild) {
		mChild->internal_add_generalization(this);
		mChild->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // MMM
    Generalization::internal_set_parent(GeneralizableElement * parent) {
	ChangeEvent * change = change_role_changed(this, "parent",
						   parent, mParent, false);
	log_change(change);
	
	change->unref();
	
	if(mParent) {
	    mParent->unref();
	}
	
	mParent = parent;
	
	if(mParent) {
	    mParent->ref();
	}
    }
    
    void // LLL
    Generalization::set_parent(GeneralizableElement * parent) {
	if(mParent != parent) {
	    // neither end is an aggregate, so we must have the same context
	    // as each other
	    
	    if(parent) {
		ASSERT(!parent->ref_get_context() ||
		       parent->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &Generalization::set_parent,
						      parent, mParent);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "parent", parent,
						       mParent, false);
	    log_change(change);
	    change->unref();
	    
	    if(mParent) {
		mParent->internal_remove_specialization(this);
		mParent->unref();
	    }
	    
	    mParent = parent;
	    
	    if(mParent) {
		mParent->internal_add_specialization(this);
		mParent->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void
    Generalization::ref_set_context_impl(set<string> & visited,
					 REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    Relationship::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Generalization::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_generalization(this);
	    
	    Relationship::ref_implant_impl(visited);
	}
    }
    
    void
    Generalization::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    Relationship::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_generalization(this);
	}
    }
    
    bool
    Generalization::ref_add_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role,
				      REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_remove_role_impl(set<string> & visited,
					 const string & role_name,
					 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_get_role_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "child") {
		role = get_child();
		return true;
	    }
	    
	    if(role_name == "parent") {
		role = get_parent();
		return true;
	    }
	    
	    if(Relationship::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_set_role_impl(set<string> & visited,
				      const string & role_name,
				      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "child") {
		GeneralizableElement * child = dynamic_cast<GeneralizableElement *>(role);
		
		if(!child) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_child(child);
		return true;
	    }
	    
	    if(role_name == "parent") {
		GeneralizableElement * parent = dynamic_cast<GeneralizableElement *>(role);
		
		if(!parent) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_parent(parent);
		return true;
	    }
	    
	    if(Relationship::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_create_class_impl(set<string> & visited,
					  REF::Class * & role,
					  const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "child") {
		role = new GeneralizableElement();
		return true;
	    }
	    
	    if(role_name == "parent") {
		role = new GeneralizableElement();
		return true;
	    }
	    
	    if(Relationship::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_get_role_iterator_impl(set<string> & visited,
					       REF::Iterator * & iterator,
					       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "child") {
		iterator = REF::iterator<GeneralizableElement>(get_child());
		return true;
	    }
	    
	    if(role_name == "parent") {
		iterator = REF::iterator<GeneralizableElement>(get_parent());
		return true;
	    }
	    
	    if(Relationship::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_get_attribute_impl(set<string> & visited,
					   const REF::Attribute * & attribute,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "discriminator") {
		attribute = get_discriminator();
		return true;
	    }
	    
	    if(Relationship::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_set_attribute_impl(set<string> & visited,
					   const string & attribute_name,
					   const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "discriminator") {
		const Name * discriminator = dynamic_cast<const Name *>(attribute);
		
		if(!discriminator) {
		    WARNING("Wrong type for attribute: " + attribute->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for attribute");
		}
		
		set_discriminator(discriminator);
		return true;
	    }
	    
	    if(Relationship::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_get_attribute_set_impl(set<string> & visited,
					       const vector<const REF::Attribute *> * & attribute_set,
					       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_set_attribute_set_impl(set<string> & visited,
					       const string & attribute_name,
					       const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(Relationship::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_create_attribute_impl(set<string> & visited,
					      REF::Attribute * & attribute,
					      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(attribute_name == "discriminator") {
		attribute = new Name();
		return true;
	    }
	    
	    if(Relationship::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Generalization::ref_is_a_type_of_impl(set<string> & visited,
					  const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Generalization") {
		return true;
	    }
	    
	    if(Relationship::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Generalization::ref_get_type() const {
	return TYPE;
    }
    
    void
    Generalization::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement * child = 0;
	    set_child(child);
	    
	    GeneralizableElement * parent = 0;
	    set_parent(parent);
	    
	    Relationship::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
