// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "method.h"

#include "operation.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Method::TYPE = "Method";
    
    Method::Method() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Method::Method(const Method & method) {
	ASSERT_NOT_REACHED();
    }
    
    Method::~Method() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Method::internal_add_specification(Operation * specification) {
	specification->ref();
	
	int index = mSpecification.size();
	
	mSpecification.push_back(specification);
	
	ChangeEvent * change = change_role_added(this, "specification",
						 specification, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Method::internal_remove_specification(Operation * specification) {
	mSpecification.remove(specification);
	
	ChangeEvent * change = change_role_removed(this, "specification",
						   specification, -1, false);
	log_change(change);
	change->unref();
	
	specification->unref();
    }
    
    void // BBB
    Method::set_specification(const list<Operation *> & specification) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Operation *> before;
	set<Operation *> after;
	
	before.insert(mSpecification.begin(), mSpecification.end());
	after.insert(specification.begin(), specification.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mSpecification.size());
	RETURN_IF_FAIL(after.size() == specification.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Operation *> added;
	vector<Operation *> removed;
	
	set_difference(specification.begin(), specification.end(),
		       mSpecification.begin(), mSpecification.end(),
		       back_inserter(added));
	
	set_difference(mSpecification.begin(), mSpecification.end(),
		       specification.begin(), specification.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Method::set_specification,
						  specification, mSpecification);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Operation::internal_add_method), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Operation::internal_remove_method), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mSpecification = specification;
	
	// changes made
	
	for(vector<Operation *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Operation * specification = *iter;
	    ChangeEvent * change = change_role_added(this, "specification",
						     specification, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Operation *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Operation * specification = *iter;
	    ChangeEvent * change = change_role_removed(this, "specification",
						       specification, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Method::add_specification(Operation * specification, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mSpecification.size()));
	
	if(index < 0) {
	    index = mSpecification.size();
	}
	
	list<Operation *>::iterator iter = mSpecification.begin();
	
	for(int i = 0; i < static_cast<int>(mSpecification.size()); ++i) {
	    RETURN_IF_FAIL(*iter != specification);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Method::add_specification,
						&Method::remove_specification,
						specification, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "specification", specification, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specification->ref();
	
	mSpecification.insert(iter, specification);
	specification->internal_add_method(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Method::remove_specification(Operation * specification) {
	// work out the index
	
	int index = 0;
	
	list<Operation *>::iterator iter = mSpecification.begin();
	
	for(; iter != mSpecification.end(); ++iter, ++index) {
	    if(*iter == specification) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mSpecification.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Method::remove_specification,
						&Method::add_specification,
						specification, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "specification", specification, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	specification->internal_remove_method(this);
	mSpecification.erase(iter);
	
	specification->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Method::ref_set_context_impl(set<string> & visited,
				 REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    BehavioralFeature::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Method::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_method(this);
	    
	    BehavioralFeature::ref_implant_impl(visited);
	}
    }
    
    void
    Method::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    BehavioralFeature::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_method(this);
	}
    }
    
    bool
    Method::ref_add_role_impl(set<string> & visited,
			      const string & role_name,
			      REF::Class * role,
			      REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "specification") {
		Operation * specification = dynamic_cast<Operation *>(role);
		
		if(!specification) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_specification(specification, index);
		return true;
	    }
	    
	    if(BehavioralFeature::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_remove_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_get_role_impl(set<string> & visited,
			      REF::Class * & role,
			      const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_set_role_impl(set<string> & visited,
			      const string & role_name,
			      REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_create_class_impl(set<string> & visited,
				  REF::Class * & role,
				  const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "specification") {
		role = new Operation();
		return true;
	    }
	    
	    if(BehavioralFeature::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_get_role_iterator_impl(set<string> & visited,
				       REF::Iterator * & iterator,
				       const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "specification") {
		iterator = REF::iterator<Operation>(get_specification());
		return true;
	    }
	    
	    if(BehavioralFeature::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_get_attribute_impl(set<string> & visited,
				   const REF::Attribute * & attribute,
				   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_set_attribute_impl(set<string> & visited,
				   const string & attribute_name,
				   const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_get_attribute_set_impl(set<string> & visited,
				       const vector<const REF::Attribute *> * & attribute_set,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_set_attribute_set_impl(set<string> & visited,
				       const string & attribute_name,
				       const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_create_attribute_impl(set<string> & visited,
				      REF::Attribute * & attribute,
				      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Method::ref_is_a_type_of_impl(set<string> & visited,
				  const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Method") {
		return true;
	    }
	    
	    if(BehavioralFeature::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Method::ref_get_type() const {
	return TYPE;
    }
    
    void
    Method::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Operation *> specification;
	    set_specification(specification);
	    
	    BehavioralFeature::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
