// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "operation.h"

#include "method.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Operation::TYPE = "Operation";
    
    Operation::Operation() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Operation::Operation(const Operation & operation) {
	ASSERT_NOT_REACHED();
    }
    
    Operation::~Operation() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Operation::internal_add_method(Method * method) {
	method->ref();
	
	int index = mMethod.size();
	
	mMethod.push_back(method);
	
	ChangeEvent * change = change_role_added(this, "method",
						 method, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Operation::internal_remove_method(Method * method) {
	mMethod.remove(method);
	
	ChangeEvent * change = change_role_removed(this, "method",
						   method, -1, false);
	log_change(change);
	change->unref();
	
	method->unref();
    }
    
    void // BBB
    Operation::set_method(const list<Method *> & method) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Method *> before;
	set<Method *> after;
	
	before.insert(mMethod.begin(), mMethod.end());
	after.insert(method.begin(), method.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mMethod.size());
	RETURN_IF_FAIL(after.size() == method.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Method *> added;
	vector<Method *> removed;
	
	set_difference(method.begin(), method.end(),
		       mMethod.begin(), mMethod.end(),
		       back_inserter(added));
	
	set_difference(mMethod.begin(), mMethod.end(),
		       method.begin(), method.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Operation::set_method,
						  method, mMethod);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Method::internal_add_specification), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Method::internal_remove_specification), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mMethod = method;
	
	// changes made
	
	for(vector<Method *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Method * method = *iter;
	    ChangeEvent * change = change_role_added(this, "method",
						     method, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Method *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Method * method = *iter;
	    ChangeEvent * change = change_role_removed(this, "method",
						       method, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Operation::add_method(Method * method, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mMethod.size()));
	
	if(index < 0) {
	    index = mMethod.size();
	}
	
	list<Method *>::iterator iter = mMethod.begin();
	
	for(int i = 0; i < static_cast<int>(mMethod.size()); ++i) {
	    RETURN_IF_FAIL(*iter != method);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Operation::add_method,
						&Operation::remove_method,
						method, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "method", method, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	method->ref();
	
	mMethod.insert(iter, method);
	method->internal_add_specification(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Operation::remove_method(Method * method) {
	// work out the index
	
	int index = 0;
	
	list<Method *>::iterator iter = mMethod.begin();
	
	for(; iter != mMethod.end(); ++iter, ++index) {
	    if(*iter == method) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mMethod.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Operation::remove_method,
						&Operation::add_method,
						method, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "method", method, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	method->internal_remove_specification(this);
	mMethod.erase(iter);
	
	method->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Operation::ref_set_context_impl(set<string> & visited,
				    REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    BehavioralFeature::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Operation::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_operation(this);
	    
	    BehavioralFeature::ref_implant_impl(visited);
	}
    }
    
    void
    Operation::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    BehavioralFeature::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_operation(this);
	}
    }
    
    bool
    Operation::ref_add_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role,
				 REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "method") {
		Method * method = dynamic_cast<Method *>(role);
		
		if(!method) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_method(method, index);
		return true;
	    }
	    
	    if(BehavioralFeature::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_remove_role_impl(set<string> & visited,
				    const string & role_name,
				    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_get_role_impl(set<string> & visited,
				 REF::Class * & role,
				 const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_set_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_create_class_impl(set<string> & visited,
				     REF::Class * & role,
				     const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "method") {
		role = new Method();
		return true;
	    }
	    
	    if(BehavioralFeature::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_get_role_iterator_impl(set<string> & visited,
					  REF::Iterator * & iterator,
					  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "method") {
		iterator = REF::iterator<Method>(get_method());
		return true;
	    }
	    
	    if(BehavioralFeature::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_get_attribute_impl(set<string> & visited,
				      const REF::Attribute * & attribute,
				      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_set_attribute_impl(set<string> & visited,
				      const string & attribute_name,
				      const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_get_attribute_set_impl(set<string> & visited,
					  const vector<const REF::Attribute *> * & attribute_set,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_set_attribute_set_impl(set<string> & visited,
					  const string & attribute_name,
					  const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_create_attribute_impl(set<string> & visited,
					 REF::Attribute * & attribute,
					 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(BehavioralFeature::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Operation::ref_is_a_type_of_impl(set<string> & visited,
				     const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Operation") {
		return true;
	    }
	    
	    if(BehavioralFeature::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Operation::ref_get_type() const {
	return TYPE;
    }
    
    void
    Operation::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Method *> method;
	    set_method(method);
	    
	    BehavioralFeature::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
