// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "parameter.h"

#include "behavioral_feature.h"
#include "classifier.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Parameter::TYPE = "Parameter";
    
    Parameter::Parameter() {
	mBehavioralFeature = 0;
	mType = 0;
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Parameter::Parameter(const Parameter & parameter) {
	ASSERT_NOT_REACHED();
    }
    
    Parameter::~Parameter() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // MMM
    Parameter::internal_set_behavioral_feature(BehavioralFeature * behavioral_feature) {
	ChangeEvent * change = change_role_changed(this, "behavioral_feature",
						   behavioral_feature, mBehavioralFeature, false);
	log_change(change);
	
	change->unref();
	
	if(mBehavioralFeature) {
	    mBehavioralFeature->unref();
	}
	
	mBehavioralFeature = behavioral_feature;
	
	if(mBehavioralFeature) {
	    mBehavioralFeature->ref();
	}
    }
    
    void // LLL
    Parameter::set_behavioral_feature(BehavioralFeature * behavioral_feature) {
	if(mBehavioralFeature != behavioral_feature) {
	    // the other end is an aggregate, so we must either have
	    // no context or the same context that the other end has
	    
	    if(behavioral_feature) {
		ASSERT(!ref_get_context() || behavioral_feature->ref_get_context() == ref_get_context());
	    }
	    
	    begin_transaction();
	    
	    Command * command = command_flip_argument(this,
						      &Parameter::set_behavioral_feature,
						      behavioral_feature, mBehavioralFeature);
	    log_command(command);
	    command->unref();
	    
	    ChangeEvent * change = change_role_changed(this,
						       "behavioral_feature", behavioral_feature,
						       mBehavioralFeature, false);
	    log_change(change);
	    change->unref();
	    
	    if(mBehavioralFeature) {
		mBehavioralFeature->internal_remove_parameter(this);
		mBehavioralFeature->unref();
	    }
	    
	    mBehavioralFeature = behavioral_feature;
	    
	    if(mBehavioralFeature) {
		mBehavioralFeature->internal_add_parameter(this);
		mBehavioralFeature->ref();
	    }
	    
	    end_transaction();
	}
    }
    
    void // MMM
    Parameter::internal_set_type(Classifier * type) {
	ChangeEvent * change = change_role_changed(this, "type",
						   type, mType, false);
	log_change(change);
	
	change->unref();
	
	if(mType) {
	    mType->unref();
	}
	
	mType = type;
	
	if(mType) {
	    mType->ref();
	}
    }
    
    void
    Parameter::ref_set_context_impl(set<string> & visited,
				    REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Parameter::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_parameter(this);
	    
	    ModelElement::ref_implant_impl(visited);
	}
    }
    
    void
    Parameter::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    ModelElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_parameter(this);
	}
    }
    
    bool
    Parameter::ref_add_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role,
				 REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_remove_role_impl(set<string> & visited,
				    const string & role_name,
				    REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_get_role_impl(set<string> & visited,
				 REF::Class * & role,
				 const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "behavioral_feature") {
		role = get_behavioral_feature();
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_set_role_impl(set<string> & visited,
				 const string & role_name,
				 REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "behavioral_feature") {
		BehavioralFeature * behavioral_feature = dynamic_cast<BehavioralFeature *>(role);
		
		if(!behavioral_feature) {
		    WARNING("Wrong type for role: " + role->ref_get_type() +
			    ", in class: " + ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type());
		}
		
		set_behavioral_feature(behavioral_feature);
		return true;
	    }
	    
	    if(ModelElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_create_class_impl(set<string> & visited,
				     REF::Class * & role,
				     const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "behavioral_feature") {
		role = new BehavioralFeature();
		return true;
	    }
	    
	    if(ModelElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_get_role_iterator_impl(set<string> & visited,
					  REF::Iterator * & iterator,
					  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "behavioral_feature") {
		iterator = REF::iterator<BehavioralFeature>(get_behavioral_feature());
		return true;
	    }
	    
	    if(ModelElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_get_attribute_impl(set<string> & visited,
				      const REF::Attribute * & attribute,
				      const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_set_attribute_impl(set<string> & visited,
				      const string & attribute_name,
				      const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_get_attribute_set_impl(set<string> & visited,
					  const vector<const REF::Attribute *> * & attribute_set,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_set_attribute_set_impl(set<string> & visited,
					  const string & attribute_name,
					  const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_create_attribute_impl(set<string> & visited,
					 REF::Attribute * & attribute,
					 const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(ModelElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Parameter::ref_is_a_type_of_impl(set<string> & visited,
				     const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Parameter") {
		return true;
	    }
	    
	    if(ModelElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Parameter::ref_get_type() const {
	return TYPE;
    }
    
    void
    Parameter::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    BehavioralFeature * behavioral_feature = 0;
	    set_behavioral_feature(behavioral_feature);
	    
	    ModelElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
