// generated by Dachshund V0.1.0
//
// Dachshund,
// Copyright (C) 2001 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "stereotype.h"

#include "tag_definition.h"
#include "uml/foundation/core/constraint.h"
#include "uml/foundation/core/model_element.h"

#include "uml.h"

#include "reflective/changes.h"
#include "reflective/set_iterator.h"
#include "reflective/list_iterator.h"
#include "reflective/pointer_iterator.h"
#include "reflective/exceptions.h"

#include "foundation/macros.h"
#include "foundation/commands.h"
#include "foundation/utility.h"
#include "foundation/exception.h"
#include "foundation/algorithm.h"
#include "foundation/functional.h"

#include <vector>
#include <functional>
#include <algorithm>
using namespace std;

namespace UML {
    
    const string Stereotype::TYPE = "Stereotype";
    
    Stereotype::Stereotype() {
	
	TRACE(9, "Constructed: " << static_cast<Object *>(this));
    }
    
    Stereotype::Stereotype(const Stereotype & stereotype) {
	ASSERT_NOT_REACHED();
    }
    
    Stereotype::~Stereotype() {
	TRACE(9, "Destroyed: " << static_cast<Object *>(this));
	
    }
    
    void // EEE
    Stereotype::internal_add_extended_element(ModelElement * extended_element) {
	extended_element->ref();
	
	int index = mExtendedElement.size();
	
	mExtendedElement.push_back(extended_element);
	
	ChangeEvent * change = change_role_added(this, "extended_element",
						 extended_element, index, false);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Stereotype::internal_remove_extended_element(ModelElement * extended_element) {
	mExtendedElement.remove(extended_element);
	
	ChangeEvent * change = change_role_removed(this, "extended_element",
						   extended_element, -1, false);
	log_change(change);
	change->unref();
	
	extended_element->unref();
    }
    
    void // BBB
    Stereotype::set_extended_element(const list<ModelElement *> & extended_element) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<ModelElement *> before;
	set<ModelElement *> after;
	
	before.insert(mExtendedElement.begin(), mExtendedElement.end());
	after.insert(extended_element.begin(), extended_element.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mExtendedElement.size());
	RETURN_IF_FAIL(after.size() == extended_element.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<ModelElement *> added;
	vector<ModelElement *> removed;
	
	set_difference(extended_element.begin(), extended_element.end(),
		       mExtendedElement.begin(), mExtendedElement.end(),
		       back_inserter(added));
	
	set_difference(mExtendedElement.begin(), mExtendedElement.end(),
		       extended_element.begin(), extended_element.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Stereotype::set_extended_element,
						  extended_element, mExtendedElement);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&ModelElement::internal_add_stereotype), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&ModelElement::internal_remove_stereotype), this));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mExtendedElement = extended_element;
	
	// changes made
	
	for(vector<ModelElement *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    ModelElement * extended_element = *iter;
	    ChangeEvent * change = change_role_added(this, "extended_element",
						     extended_element, false);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<ModelElement *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    ModelElement * extended_element = *iter;
	    ChangeEvent * change = change_role_removed(this, "extended_element",
						       extended_element, false);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Stereotype::add_extended_element(ModelElement * extended_element, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mExtendedElement.size()));
	
	if(index < 0) {
	    index = mExtendedElement.size();
	}
	
	list<ModelElement *>::iterator iter = mExtendedElement.begin();
	
	for(int i = 0; i < static_cast<int>(mExtendedElement.size()); ++i) {
	    RETURN_IF_FAIL(*iter != extended_element);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::add_extended_element,
						&Stereotype::remove_extended_element,
						extended_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "extended_element", extended_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	extended_element->ref();
	
	mExtendedElement.insert(iter, extended_element);
	extended_element->internal_add_stereotype(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Stereotype::remove_extended_element(ModelElement * extended_element) {
	// work out the index
	
	int index = 0;
	
	list<ModelElement *>::iterator iter = mExtendedElement.begin();
	
	for(; iter != mExtendedElement.end(); ++iter, ++index) {
	    if(*iter == extended_element) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mExtendedElement.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::remove_extended_element,
						&Stereotype::add_extended_element,
						extended_element, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "extended_element", extended_element, false, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	extended_element->internal_remove_stereotype(this);
	mExtendedElement.erase(iter);
	
	extended_element->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    Stereotype::internal_add_stereotype_constraint(Constraint * stereotype_constraint) {
	stereotype_constraint->ref();
	
	int index = mStereotypeConstraint.size();
	
	mStereotypeConstraint.push_back(stereotype_constraint);
	
	ChangeEvent * change = change_role_added(this, "stereotype_constraint",
						 stereotype_constraint, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Stereotype::internal_remove_stereotype_constraint(Constraint * stereotype_constraint) {
	mStereotypeConstraint.remove(stereotype_constraint);
	
	ChangeEvent * change = change_role_removed(this, "stereotype_constraint",
						   stereotype_constraint, -1, true);
	log_change(change);
	change->unref();
	
	stereotype_constraint->unref();
    }
    
    void // BBB
    Stereotype::set_stereotype_constraint(const list<Constraint *> & stereotype_constraint) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<Constraint *> before;
	set<Constraint *> after;
	
	before.insert(mStereotypeConstraint.begin(), mStereotypeConstraint.end());
	after.insert(stereotype_constraint.begin(), stereotype_constraint.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mStereotypeConstraint.size());
	RETURN_IF_FAIL(after.size() == stereotype_constraint.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<Constraint *> added;
	vector<Constraint *> removed;
	
	set_difference(stereotype_constraint.begin(), stereotype_constraint.end(),
		       mStereotypeConstraint.begin(), mStereotypeConstraint.end(),
		       back_inserter(added));
	
	set_difference(mStereotypeConstraint.begin(), mStereotypeConstraint.end(),
		       stereotype_constraint.begin(), stereotype_constraint.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Stereotype::set_stereotype_constraint,
						  stereotype_constraint, mStereotypeConstraint);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&Constraint::internal_set_constrained_stereotype), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&Constraint::internal_set_constrained_stereotype),
			 static_cast<Stereotype *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mStereotypeConstraint = stereotype_constraint;
	
	// changes made
	
	for(vector<Constraint *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    Constraint * stereotype_constraint = *iter;
	    ChangeEvent * change = change_role_added(this, "stereotype_constraint",
						     stereotype_constraint, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<Constraint *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    Constraint * stereotype_constraint = *iter;
	    ChangeEvent * change = change_role_removed(this, "stereotype_constraint",
						       stereotype_constraint, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Stereotype::add_stereotype_constraint(Constraint * stereotype_constraint, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mStereotypeConstraint.size()));
	
	if(index < 0) {
	    index = mStereotypeConstraint.size();
	}
	
	list<Constraint *>::iterator iter = mStereotypeConstraint.begin();
	
	for(int i = 0; i < static_cast<int>(mStereotypeConstraint.size()); ++i) {
	    RETURN_IF_FAIL(*iter != stereotype_constraint);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::add_stereotype_constraint,
						&Stereotype::remove_stereotype_constraint,
						stereotype_constraint, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "stereotype_constraint", stereotype_constraint, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	stereotype_constraint->ref();
	
	mStereotypeConstraint.insert(iter, stereotype_constraint);
	stereotype_constraint->internal_set_constrained_stereotype(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Stereotype::remove_stereotype_constraint(Constraint * stereotype_constraint) {
	// work out the index
	
	int index = 0;
	
	list<Constraint *>::iterator iter = mStereotypeConstraint.begin();
	
	for(; iter != mStereotypeConstraint.end(); ++iter, ++index) {
	    if(*iter == stereotype_constraint) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mStereotypeConstraint.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::remove_stereotype_constraint,
						&Stereotype::add_stereotype_constraint,
						stereotype_constraint, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "stereotype_constraint", stereotype_constraint, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	stereotype_constraint->internal_set_constrained_stereotype(static_cast<Stereotype *>(0));
	mStereotypeConstraint.erase(iter);
	
	stereotype_constraint->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void // EEE
    Stereotype::internal_add_defined_tag(TagDefinition * defined_tag) {
	defined_tag->ref();
	
	int index = mDefinedTag.size();
	
	mDefinedTag.push_back(defined_tag);
	
	ChangeEvent * change = change_role_added(this, "defined_tag",
						 defined_tag, index, true);
	log_change(change);
	change->unref();
    }
    
    void // FFF
    Stereotype::internal_remove_defined_tag(TagDefinition * defined_tag) {
	mDefinedTag.remove(defined_tag);
	
	ChangeEvent * change = change_role_removed(this, "defined_tag",
						   defined_tag, -1, true);
	log_change(change);
	change->unref();
	
	defined_tag->unref();
    }
    
    void // BBB
    Stereotype::set_defined_tag(const list<TagDefinition *> & defined_tag) {
	// put the items into sets so that we can call set_difference
	// to work out what is actually being added and removed
	
	set<TagDefinition *> before;
	set<TagDefinition *> after;
	
	before.insert(mDefinedTag.begin(), mDefinedTag.end());
	after.insert(defined_tag.begin(), defined_tag.end());
	
	// check for duplicates
	
	RETURN_IF_FAIL(before.size() == mDefinedTag.size());
	RETURN_IF_FAIL(after.size() == defined_tag.size());
	
	// sort the stuff being added and the stuff being removed
	// into separate containers
	
	vector<TagDefinition *> added;
	vector<TagDefinition *> removed;
	
	set_difference(defined_tag.begin(), defined_tag.end(),
		       mDefinedTag.begin(), mDefinedTag.end(),
		       back_inserter(added));
	
	set_difference(mDefinedTag.begin(), mDefinedTag.end(),
		       defined_tag.begin(), defined_tag.end(),
		       back_inserter(removed));
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_argument(this,
						  &Stereotype::set_defined_tag,
						  defined_tag, mDefinedTag);
	log_command(command);
	command->unref();
	
	// make the changes
	
	for_each(added.begin(), added.end(),
		 bind2nd(mem_fun(&TagDefinition::internal_set_owner), this));
	for_each(removed.begin(), removed.end(),
		 bind2nd(mem_fun(&TagDefinition::internal_set_owner),
			 static_cast<Stereotype *>(0)));
	
	for_each(added.begin(), added.end(), mem_fun(&Object::ref));
	for_each(removed.begin(), removed.end(), mem_fun(&Object::unref));
	
	mDefinedTag = defined_tag;
	
	// changes made
	
	for(vector<TagDefinition *>::iterator iter = added.begin();
	    iter != added.end();
	    ++iter) {
	    TagDefinition * defined_tag = *iter;
	    ChangeEvent * change = change_role_added(this, "defined_tag",
						     defined_tag, true);
	    log_change(change);
	    change->unref();
	}
	
	for(vector<TagDefinition *>::iterator iter = removed.begin();
	    iter != removed.end();
	    ++iter) {
	    TagDefinition * defined_tag = *iter;
	    ChangeEvent * change = change_role_removed(this, "defined_tag",
						       defined_tag, true);
	    log_change(change);
	    change->unref();
	}
	
	//
	
	end_transaction();
    }
    
    void // CCC
    Stereotype::add_defined_tag(TagDefinition * defined_tag, int index) {
	// make sure the the index isn't out of range
	
	RETURN_IF_FAIL(index <= static_cast<int>(mDefinedTag.size()));
	
	if(index < 0) {
	    index = mDefinedTag.size();
	}
	
	list<TagDefinition *>::iterator iter = mDefinedTag.begin();
	
	for(int i = 0; i < static_cast<int>(mDefinedTag.size()); ++i) {
	    RETURN_IF_FAIL(*iter != defined_tag);
	    
	    if(i < index) {
		++iter;
	    }
	}
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::add_defined_tag,
						&Stereotype::remove_defined_tag,
						defined_tag, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_added(this, "defined_tag", defined_tag, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	defined_tag->ref();
	
	mDefinedTag.insert(iter, defined_tag);
	defined_tag->internal_set_owner(this);
	
	// changes made
	
	end_transaction();
    }
    
    void // DDD
    Stereotype::remove_defined_tag(TagDefinition * defined_tag) {
	// work out the index
	
	int index = 0;
	
	list<TagDefinition *>::iterator iter = mDefinedTag.begin();
	
	for(; iter != mDefinedTag.end(); ++iter, ++index) {
	    if(*iter == defined_tag) {
		break;
	    }
	}
	
	RETURN_IF_FAIL(iter != mDefinedTag.end());
	
	// transaction time
	
	begin_transaction();
	
	//
	
	Command * command = command_flip_method(this,
						&Stereotype::remove_defined_tag,
						&Stereotype::add_defined_tag,
						defined_tag, index);
	log_command(command);
	command->unref();
	
	ChangeEvent * change = change_role_removed(this, "defined_tag", defined_tag, true, index);
	log_change(change);
	change->unref();
	
	// make the changes
	
	defined_tag->internal_set_owner(static_cast<Stereotype *>(0));
	mDefinedTag.erase(iter);
	
	defined_tag->unref();
	
	// changes made
	
	end_transaction();
    }
    
    void
    Stereotype::ref_set_context_impl(set<string> & visited,
				     REF::Context * context) {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_set_context_impl(visited, context);
	}
    }
    
    void
    Stereotype::ref_implant_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_stereotype(this);
	    
	    GeneralizableElement::ref_implant_impl(visited);
	}
    }
    
    void
    Stereotype::ref_uproot_impl(set<string> & visited) throw() {
	if(visited.insert(TYPE).second) {
	    GeneralizableElement::ref_uproot_impl(visited);
	    
	    REF::Context * context = ref_get_context();
	    
	    ASSERT(context);
	    
	    UML * model = known_cast<UML *>(context);
	    
	    model->internal_add_stereotype(this);
	}
    }
    
    bool
    Stereotype::ref_add_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role,
				  REF::Class * position) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "extended_element") {
		ModelElement * extended_element = dynamic_cast<ModelElement *>(role);
		
		if(!extended_element) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_extended_element(extended_element, index);
		return true;
	    }
	    
	    if(role_name == "stereotype_constraint") {
		Constraint * stereotype_constraint = dynamic_cast<Constraint *>(role);
		
		if(!stereotype_constraint) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_stereotype_constraint(stereotype_constraint, index);
		return true;
	    }
	    
	    if(role_name == "defined_tag") {
		TagDefinition * defined_tag = dynamic_cast<TagDefinition *>(role);
		
		if(!defined_tag) {
		    WARNING("Wrong type for role: " + role->ref_get_type());
		    throw REF::BadType("Wrong type for role: " + role->ref_get_type() +
				       ", in class: " + ref_get_type());
		}
		
		int index = -1;
		
		add_defined_tag(defined_tag, index);
		return true;
	    }
	    
	    if(GeneralizableElement::ref_add_role_impl(visited, role_name, role, position)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_remove_role_impl(set<string> & visited,
				     const string & role_name,
				     REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_remove_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_get_role_impl(set<string> & visited,
				  REF::Class * & role,
				  const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_role_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_set_role_impl(set<string> & visited,
				  const string & role_name,
				  REF::Class * role) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_role_impl(visited, role_name, role)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_create_class_impl(set<string> & visited,
				      REF::Class * & role,
				      const string & role_name) const {
	if(visited.insert(TYPE).second) {
	    if(role_name == "extended_element") {
		role = new ModelElement();
		return true;
	    }
	    
	    if(role_name == "stereotype_constraint") {
		role = new Constraint();
		return true;
	    }
	    
	    if(role_name == "defined_tag") {
		role = new TagDefinition();
		return true;
	    }
	    
	    if(GeneralizableElement::ref_create_class_impl(visited, role, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_get_role_iterator_impl(set<string> & visited,
					   REF::Iterator * & iterator,
					   const string & role_name) {
	if(visited.insert(TYPE).second) {
	    if(role_name == "extended_element") {
		iterator = REF::iterator<ModelElement>(get_extended_element());
		return true;
	    }
	    
	    if(role_name == "stereotype_constraint") {
		iterator = REF::iterator<Constraint>(get_stereotype_constraint());
		return true;
	    }
	    
	    if(role_name == "defined_tag") {
		iterator = REF::iterator<TagDefinition>(get_defined_tag());
		return true;
	    }
	    
	    if(GeneralizableElement::ref_get_role_iterator_impl(visited, iterator, role_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_get_attribute_impl(set<string> & visited,
				       const REF::Attribute * & attribute,
				       const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_set_attribute_impl(set<string> & visited,
				       const string & attribute_name,
				       const REF::Attribute * attribute) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_attribute_impl(visited, attribute_name, attribute)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_get_attribute_set_impl(set<string> & visited,
					   const vector<const REF::Attribute *> * & attribute_set,
					   const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_get_attribute_set_impl(visited, attribute_set, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_set_attribute_set_impl(set<string> & visited,
					   const string & attribute_name,
					   const vector<const REF::Attribute *> * attribute_set) {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_set_attribute_set_impl(visited, attribute_name, attribute_set)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_create_attribute_impl(set<string> & visited,
					  REF::Attribute * & attribute,
					  const string & attribute_name) const {
	if(visited.insert(TYPE).second) {
	    if(GeneralizableElement::ref_create_attribute_impl(visited, attribute, attribute_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    bool
    Stereotype::ref_is_a_type_of_impl(set<string> & visited,
				      const string & class_name) const {
	if(visited.insert(TYPE).second) {
	    if(class_name == "Stereotype") {
		return true;
	    }
	    
	    if(GeneralizableElement::ref_is_a_type_of_impl(visited, class_name)) {
		return true;
	    }
	    
	    return false;
	}
	else {
	    return false;
	}
    }
    
    const string &
    Stereotype::ref_get_type() const {
	return TYPE;
    }
    
    void
    Stereotype::ref_delete_impl(set<string> & visited) {
	if(visited.insert(TYPE).second) {
	    list<Constraint *> stereotype_constraint_list = get_stereotype_constraint();
	    for_each(stereotype_constraint_list.begin(), stereotype_constraint_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<TagDefinition *> defined_tag_list = get_defined_tag();
	    for_each(defined_tag_list.begin(), defined_tag_list.end(),
		     mem_fun(&REF::Class::ref_delete));
	    
	    list<ModelElement *> extended_element;
	    set_extended_element(extended_element);
	    
	    list<Constraint *> stereotype_constraint;
	    set_stereotype_constraint(stereotype_constraint);
	    
	    list<TagDefinition *> defined_tag;
	    set_defined_tag(defined_tag);
	    
	    GeneralizableElement::ref_delete_impl(visited);
	}
    }
    
} // namespace UML
