// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif // HAVE_CONFIG_H

#include "property.h"

#include "foundation/functional.h"
#include "foundation/macros.h"
#if 0
#include "foundation/changes.h"
#endif
#include "foundation/commands.h"

#include <iostream>
using namespace std;

namespace XML {

    Property::Property() : Object() {
    }

    Property::Property(const string & name, const string & value) : Object(),
	mName(name),
	mValue(value) {

    }

    Property::~Property() {
    }

    void
    Property::set_name(const string & name) {
	if(mName != name) {
	    begin_transaction();

	    Command * command = command_flip_argument(this, &Property::set_name, name, mName);
#if 0
	    ChangeEvent * change = new ChangeAttributeChanged(this, "name", name, mName);
#endif

	    // make the changes

	    mName = name;

	    // changes made

	    log_command(command);
#if 0
	    log_change(change);
#endif

#if 0
	    change->unref();
#endif
	    command->unref();

	    end_transaction();
	}
    }

    void
    Property::set_value(const string & value) {
	if(mValue != value) {
	    begin_transaction();

	    Command * command = command_flip_argument(this, &Property::set_value,
						      value, mValue);
#if 0
	    ChangeEvent * change = new ChangeAttributeChanged(this, "value", value, mValue);
#endif

	    // make the changes

	    mValue = value;

	    // changes made

	    log_command(command);
#if 0
	    log_change(change);
#endif

#if 0
	    change->unref();
#endif
	    command->unref();

	    end_transaction();
	}
    }

    ostream &
    operator << (ostream & ost, const Property & property) {
	ost << property.get_name() << "=\"" << property.get_value() << "\"";

	return ost;
    }

} // namespace XML
