#ifndef XML_SAX_H
#define XML_SAX_H

#include "foundation/object.h"

#include <string>
#include <iosfwd>
#include <stack>
#include <vector>
using namespace std;

namespace XML {

    class Properties;
    class SAXHandler;

    class SAXParser : public Object {
    public:
	explicit SAXParser(SAXHandler * root_sax_handler);
	~SAXParser();

        // start parsing the input stream

	void parse(istream & is, int passes = 1);

        // during parsing a new handler can be used to replace the current one

        void use(SAXHandler * sax_handler);

        // controls the produce stack

	void push();
        vector<Object *> & top() { return mProduceStack.top(); }
        void pop();

        // which parse is the parser currently doing

	int get_pass() const { return mPass; }

        // what is the current level of tag nesting

	int get_tag_depth() const { return mTagDepth; }

    private:
	int mPass;
	int mTagDepth;
	stack<pair<SAXHandler *, int> > mHandlerStack;
	stack<vector<Object *> > mProduceStack;

	void warning(const string & msg);
	void error(const string & msg);
	void fatal(const string & msg);

	void comment(const string & comment);
	void start_document();
	void end_document();
	void start_element(const string & name, const Properties & properties);
	void end_element(const string & name);
	void characters(const string & data);
	void reference(const string & name);
	void cdata_block(const string & data);
    };

} // namespace XML

#endif // XML_SAX_H
