// Dachshund,
// Copyright (C) 2000 David Bryant
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.

#ifndef XML_STREAM_H
#define XML_STREAM_H

#include <string>
#include <iostream>
#include <sstream>
#include <stack>
using namespace std;

namespace XML {

    enum DTDKind {
	DTDK_SYSTEM,
	DTDK_PUBLIC
    };

    struct XmlBeginOpen {
	explicit XmlBeginOpen(const string & n) : name(n) { }
	const string name;
    };

    struct XmlAttribute {
	XmlAttribute(const string & n, const string & v) : name(n), value(v) { }
	const string name;
	const string value;
    };

    struct XmlBeginClose {
    };

    struct XmlEnd {
    };

    struct XmlComment {
	explicit XmlComment(const string & c) : comment(c) { }
	const string comment;
    };

    class Stream {
    public:
	Stream(ostream & ost, const string & doctype, DTDKind dtd_kind, const string & dtd);
	~Stream();

	void comment(const string & c);
	void begin_open(const string & n);
	void attribute(const string & n, const string & v);
	void begin_close();
	void write(const string & s); 	// cdata/pcdata
	void end();

	void set_indent_width(int new_value);
	int get_indent_width() const;

	friend Stream & operator << (Stream & xos, ostream & ost) {
	    xos.mOs << ost;

	    return xos;
	}

    protected:
	void error(const string & m) const;

    private:
	Stream & operator = (const Stream &) { return *this; }
	Stream(const Stream & xos) : mOs(xos.mOs) { }

	stack<string> mStack;

	int mIndentLevel;
	int mIndentWidth;

	static const int DEFAULT_INDENT_WIDTH = 4;

	ostream & mOs;

	enum State {
	    STATE_START,
	    STATE_IN_TAG,
	    STATE_OUT_OF_EMPTY_TAG,
	    STATE_OUT_OF_NESTED_TAG,
	    STATE_OUT_OF_CDATA_TAG
	};

	State mState;

	bool mHadRootElement;

	string mDocType;
    };

    template<class T>
    Stream &
    operator << (Stream & xos, const T & data) {
	stringstream ss;

	ss << data;

	xos.write(ss.str());

	return xos;
    }

    inline Stream &
    operator << (Stream & xos, const XmlBeginOpen & begin_open) {
	xos.begin_open(begin_open.name);

	return xos;
    }

    inline Stream &
    operator << (Stream & xos, const XmlAttribute & attribute) {
	xos.attribute(attribute.name, attribute.value);

	return xos;
    }

    inline Stream &
    operator << (Stream & xos, const XmlBeginClose & begin_close) {
	xos.begin_close();

	return xos;
    }

    inline Stream &
    operator << (Stream & xos, const XmlEnd & end) {
	xos.end();

	return xos;
    }

    inline Stream &
    operator << (Stream & xos, const XmlComment & comment) {
	xos.comment(comment.comment);

	return xos;
    }

} // namespace XML

#endif // XML_STREAM_H
